/*                                                                
**  Copyright (C) 2008-2010,2017,2020,2022-2023  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/* dmulib.h                                                      */
/*                                                               */
/* Datamodel library utility level data structures and methods   */
/*                                                               */
/*****************************************************************/

#ifndef DMULIB_H
#define DMULIB_H

/* Trigger loading of nutils.h */
#ifndef DMLIB_INTERNAL
#define DMLIB_INTERNAL
#endif

#include "dmlib.h"   // globals.. includes dmerror.. 
#include <string.h>

#include <float.h>   // dependencies in 'dmi' (move)
#include <math.h>    // dependencies in 'dmi' (move)


#define DM_NULLTYPE_BAD -1
#define DM_NULLTYPE_NONE 0
#define DM_NULLTYPE_NAN  1
#define DM_NULLTYPE_VAL  2


/* ==============================  dmuList ============================= */
/* Generic List or Flexible Array.                                       */
typedef struct dmuList_s
{
  long n;         /* Number of elements (length)     */
  long current;   /* Index of last element accessed  */
  long capacity;  /* Allocated capacity              */
  void** data;    /* Array elements                  */
} dmuList;

/* dmuList methods */
dmuList* dmu_list_alloc( long );
void     dmu_list_free( dmuList* );
long     dmu_list_length( dmuList* );
void     dmu_list_add( dmuList*, void* );
void*    dmu_list_delete( dmuList*, long );
int      dmu_list_insert( dmuList*, void*, long );
void*    dmu_list_next( dmuList* );
void*    dmu_list_get( dmuList*, long );
int      dmu_list_hasNext( dmuList* );
void     dmu_list_rewind( dmuList* );



/* ==============================  dmuKey ============================== */
#define KERNEL_TYPE_KEY   60   // Match FTK_KEY value from FITS kernel 

typedef long dmuKeyType;
typedef struct dmuKey_s {
  long  type;          /* descriptor type (reserved field) */
  long  index;         /* number in file (one based)       */
  char* name;          /* name                             */
  char* ucname;        /* name in upper case (helper)      */
  GenericType value;   /* value                            */ /*MCD: TODO GenericType == double ??? */
  char* desc;          /* description                      */
  char* unit;          /* unit                             */
  int   mask;          /* visibility; 0=visible, 1=hidden  */
  long  group;         /* Used for marking grouped keys    */
  dmuKeyType keytype;  /* kernel dtype flag                */ /*MCD: TODO ~redundant with datatype? */
  dmDataType datatype; /* DM dtype flag                    */
 } dmuKey;

/* dmuKey methods */
dmuKey* dmu_key_alloc( void );
dmuKey* dmu_key_create( const char* name, const char* unit, const char* desc, 
			const dmuKeyType ktype, const dmDataType dtype,
			const void* value, const dmBool mask );
void    dmu_key_free( dmuKey* key );
dmDataType dmu_key_get_datatype( dmuKey* key );
char*   dmu_key_get_desc( dmuKey* key );
char*   dmu_key_get_name( dmuKey* key );
long    dmu_key_get_no( dmuKey* key );
char*   dmu_key_get_unit( dmuKey* key );
void*   dmu_key_get_value( dmuKey* key );
void    dmu_key_get_typed_value( dmuKey* key, dmDataType type, long maxlen, void* value );
dmBool  dmu_key_isHidden( dmuKey* key );
logical dmu_key_match( const char* name, const char* pattern );
void    dmu_key_print( dmuKey* key, void (*NativeTypeName)( dmuKeyType, char*, long ), int opts, char* buf, long maxlen );
void    dmu_key_set( dmuKey* key, const dmuKeyType ktype, const dmDataType dtype,
		     const char* name, const void* value, const char* unit,
		     const char* desc, const dmBool mask );
void    dmu_key_set_desc( dmuKey* key, const char* desc );
void    dmu_key_set_hidden( dmuKey* key, const dmBool mask );
void    dmu_key_set_name( dmuKey* key, const char* name );
void    dmu_key_set_no( dmuKey* key, long keyno );
void    dmu_key_set_unit( dmuKey* key, const char* unit );
void    dmu_key_set_value( dmuKey* key, const void* value, const dmDataType dtype );

void    dmu_key_typed_value( dmuKey* key, dmDataType type, void* value );


/* ============================  dmuKeyList ============================ */
/* dmuKeyList is a simple extension of the base dmuList with specialized */
/* interface for working with dmuKey data structures in the list.        */
/*                                                                       */
typedef struct dmuKeyList_s
{
  dmuList* list;  /* Base List          */
} *dmuKeyList;

/* dmuKeyList methods */
dmuKeyList dmu_keylist_alloc( void );
long    dmu_keylist_check_index( const dmuKeyList keylist, const char* index_name, const char* name );
dmBool  dmu_keylist_delete( dmuKeyList keylist, dmuKey* key );
dmuKey* dmu_keylist_find( dmuKeyList keylist, char* name );
void    dmu_keylist_free( dmuKeyList list );
dmuKey* dmu_keylist_get( dmuKeyList list, long ndx );
long    dmu_keylist_insert( dmuKeyList keylist, dmuKey* ftkey, long posp );
long    dmu_keylist_length( const dmuKeyList keylist );
void    dmu_keylist_print( const dmuKeyList keylist,  void (*NativeTypeName)( dmuKeyType, char*, long ) );
dmuKey* dmu_keylist_read_typed_value( dmuKeyList keylist, char* name, dmDataType type, void* value );
dmuKey* dmu_keylist_read_value_c( dmuKeyList keylist, char* name, char* value, long maxlen );
long    dmu_keyword_max_index( dmuKeyList keylist, char* pattern );


/* ============================  dmuVectorKey ?? ======================= */
/* The vector key object stores the key indices of a structured key
   for conventions analogous to the CTYPEn convention. */

typedef struct dmuVectorKey_s {
 dmuKey* name;
 dmuKey* val;
 dmuKey* form;
 dmuKey* unit;
} *dmuVectorKey;

void dmu_vector_key_init( dmuVectorKey v );
MixArray dmu_vector_key_make_list( dmuKeyList keylist, char* name_tag, char* form_tag, char* unit_tag, char* val_tag, long tagsize );


/* ==========================  dmuAliasKeyList ?? ====================== */
/*  The AliasKeyList object supports the MTYPE, MFORM convention */
typedef struct dmuAliasKeyList_s {
 long n;
 char** desc;  /* Key desc index for MTYPE */
 AliasList names;
 AliasList cpts;
 MixArray etypes;
} *dmuAliasKeyList;

dmuAliasKeyList dmu_alias_key_make_list( dmuKeyList keylist, char* name_tag, char* form_tag, char* etype_tag, char* ename_tag, long tagsize );
void   dmu_alias_key_list_free( dmuAliasKeyList mlist );


/* ===========================  dmElementType ========================== */
 /* Element types */
 void dmu_etype_name( dmElementType etype, char* name, long maxlen );
 long dmu_etype_dim( dmElementType etype );
 dmElementType dmu_parse_etype_code( char* code );
 
/* ===============================  Other ============================== */
/* Filename manipulation */
char*  dmu_extract_filename( const char* vspec );
char*  dmu_extract_filename_extension( const char* filename );

dmBool dmu_bit_intersect( const char* bufA, const char* bufB, char* obuf );
int    dmu_bit_match( unsigned char* buf, long n, char* pattern );
dmBool dmu_bit_union( const char* bufA, const char* bufB, char* obuf );
void   dmu_block_type_name( const dmBlockType type, char* name );
void   dmu_format_number_d( const double val, char* buf );
char*  dmu_make_range_string( void* mins, void* maxes, long n, int incnull, dmDataType type, dmBool cast, char* fmt );
char*  dmu_parse_name_value( char* buf, char* name, long maxlen, char* op );
int    dmu_parse_null( const char* val, dmDataType type, GenericType* null );
char*  dmu_parse_token( char* ptr, char sep, char** result, long maxlen );
char*  dmu_parse_paren_token( char* ptr, char lsep, char rsep, char** result, long maxlen );
void   dmu_print_number( const double x, char* buf, const long maxlen );
void   dmu_print_pixel( long* pixel, long naxes, char* buf );
char*  dmu_split_token( char* buf, char* name, long maxlen, char* op );
char*  dmu_split_token_chars( char* buf, char* name, long maxlen, char* op, char* opset );
int    dmu_string_cast_numeric(     char *strval, dmDataType type, int base, void* value);
dmErrCode    dmu_string_cast_logical(     const char *strval, logical* value);
void   dmu_string_strip_whitespace(      char *strval, char *whites, dmBool rm_leading, dmBool rm_trailing ); 
void   dmu_string_strip_whites_leading(  char *strval, char *whites);
void   dmu_string_strip_whites_trailing( char *strval, char *whites);
void   dmu_string_strip_quotes(          char *strval, char *quotes);
void   dmu_string_remove_backslash(      char *strval, char *escapes);

void dmu_pixel_from_pixelno( long PixelNo, long* axes, long naxes, long* Pixel );
long dmu_pixelno_from_pixel( long* Pixel, long* axes, long naxes );
 

 void dmu_collapse_ranges_d( double** dmin, double** dmax, long* nvals );
 void dmu_union_d( double xmin, double xmax, double* amin, double* amax, long* nvals );
 void dmu_intersect_d( double xmin, double xmax, double* dmin, double* dmax, double* rmin, double* rmax,  long* nvals  );
 void dmu_intersect_array_d( double* xmin, double* xmax, long n, double** dmin, double** dmax, long* nvals, long* nmax );
 
 
 dmErrCode dmu_err_set_code( long code, char* text, long fcode );
 dmErrCode dmu_err_get( void );
 dmErrCode dmu_err_set( long code, char* text );
 void dmu_err_clear( void );
 void dmu_err_disp_select_msgs( long lErrCode);
long dmu_err_get_num_occur(long code, const char* text);
 void dmuSetVerbose( long level );
 
 long dmu_count_commas( const char* buf );
 int dmu_lex_number( char* buf, double* dval );
 void dmu_err_init( void );
 void dmu_err_status( void );
 char* dmu_string_delta( char* buf, long n );
 int dmu_text_in_range( char* buf, char* start, char* end );
 void dmu_err_get_message( char* buf, long maxlen );
 void dmu_map_bits( unsigned char* buf, long n, char* bits );
 void dmu_key_dump( dmuKeyList list, char* tag );
 void dx( dmuKeyList keylist ); /* debug */
#endif
