// 
//  Copyright (C) 1998-1999,2001,2003,2005-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:     @(#)imaging.hh

* DEVELOPEMENT:  Chandra X-ray Science Center rmf library
                 hhe@cfa.harvard.edu (initial version)
 
* DESCRIPTION:   Header file
 
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             07/10/98
 
*H****************************************************************************/

#ifndef __IMAGING__HH__
#define __IMAGING__HH__

#include <vector>
#include <iostream>

extern "C" 
{
#include <float.h>  // defines macro,  DBL_EPSILON, used here
#include <math.h>
}

//************************************************************
//
// class container of Imaging
// 2-D grid process: area, mean value at specified grid point
//
//************************************************************
class Imaging
{

public:
  Imaging() { 
    d_ncell = new long [2]; d_ncell[0] = d_ncell[1] = 0;
    d_gridpts = new  vector<float> [2]; 
    d_start= new long [2];  d_start[0] = 0; d_start[1] = 0;

    d_gridval = NULL;
     
  };
  ~Imaging(){ 
    delete [] d_ncell; if(d_gridval) delete [] d_gridval; 
    delete [] d_gridpts;
    delete [] d_start;
  };

  void SetGridPoints(short xy_flag, float* gridpts, long start, long sz);
  void SetCellAlongAxis(short xy_flag, long start, long len);

  long SetGridValues(long num_rows, vector<float> *value);
  void SetGridValues(vector<float> *value);
  vector<float> *InitGridValueArray();


  float GetWeightedArea();

private:

  long *d_start;
  long *d_ncell;
  vector<float> *d_gridpts;
  vector<float> *d_gridval;

  
};

#endif //__IMAGING__HH__

//--------------------------------------------
//
// Imaging class inline function definitions
//
//--------------------------------------------
//
inline void Imaging::SetCellAlongAxis(short xy_flag, long start, long len)
{
  d_start[xy_flag] = start;
  d_ncell[xy_flag] = len-1;

}
//
inline void Imaging:: SetGridPoints(short xy_flag, float* gridpts, long start, long len)
{

  // clean up the stalk
  d_gridpts[xy_flag].clear();

  for(long ii = start; ii < len+start; ii++)
    d_gridpts[xy_flag].push_back( gridpts[ii] ); 

  //  SetGridPoints(xy_flag,start, len);
}
//
inline vector<float> *Imaging::InitGridValueArray()
{
  static short sz=(short)d_ncell[1];

  if(sz == d_ncell[1] && d_gridval != NULL)
    return d_gridval;
  else if( sz == d_ncell[1] && d_gridval == NULL){
    d_gridval =  new vector<float> [ sz+1];
    return d_gridval;
  } if( sz != 0 && d_gridval != NULL) {
    delete [] d_gridval; 
    sz=(short)d_ncell[1];
    d_gridval =  new vector<float> [ sz+1];
    return d_gridval;
  } else 
    return NULL;

}
//
//
inline void Imaging::SetGridValues(vector<float> *value)
{
  for(long ii = 0; ii <d_ncell[1]+1; ii++){
    d_gridval[ii].clear();
    d_gridval[ii] = value[ii];
  }
} 
inline long Imaging::SetGridValues(long nrows, vector<float> *value)
{
  long _count=0; //Helen@9/10

  // define the number of rows of the grid-val array
  // number of columns of the array is automatically strached through
  // "=" operator. see below

  if( d_ncell[1]+1 != nrows){
    cerr<<"\nError: inconsistent number of rows betwen two inputs.\n";
    return 0;
  }

  for(long ii = 0; ii < nrows; ii++){
    //d_gridval[ii] = value[ii];
    d_gridval[ii].clear();
    long kk = value[ii].size();
    for(long jj = 0; jj < kk; jj++){
      d_gridval[ii].push_back(value[ii][jj]);
      if(value[ii][jj] > DBL_EPSILON)
	_count++;
    }
  }
  return _count;
}
//
//
inline float Imaging::GetWeightedArea()
{
  long 	x_indx, y_indx;
  float length[2], wght, area;
  float sum = 0.0;


  for(long cell_at_y = 0; cell_at_y < d_ncell[1]; cell_at_y++) {
    y_indx = d_start[1] + cell_at_y;
    for(long cell_at_x = 0; cell_at_x< d_ncell[0]; cell_at_x++) {
      x_indx = d_start[0] + cell_at_x;
      //get mean grid point over 4 corners
      wght =  ( d_gridval[cell_at_y][  x_indx  ] +
                d_gridval[cell_at_y][  x_indx+1] +
                d_gridval[cell_at_y+1][x_indx  ] +
                d_gridval[cell_at_y+1][x_indx+1]  )/4.0;
      if(wght > DBL_EPSILON ) {
	//get the area enclosed by the 4 corners
	length[0] =  fabs(d_gridpts[0][x_indx+1] - d_gridpts[0][x_indx]);
	length[1] =  fabs(d_gridpts[1][y_indx+1] - d_gridpts[1][y_indx]);
	area = length[0] * length[1];
	sum += area*wght;
	/*
	printf("\nstart= (%d,%d), cell_sz = (%d, %d)", d_start[0],d_start[1], d_ncell[0], d_ncell[1]);
	printf("\t wght=%7.2e, area=%7.2e", wght,area);
	*/
      }
    }
  }
  return sum; 
}



