/*                                                                
**  Copyright (C) 1998,2000,2007  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


#ifndef L1_ALIGNMENT_DEFS_H
#define L1_ALIGNMENT_DEFS_H 


/*  the following are include files on which the alignment functions 
 *  rely.
 *
 *  INCLUDE FILES
 */

#ifndef DSLIB_H
#include "dslib.h"
#define DSLIB_H
#endif 

#ifndef PIXLIB_H
#include "pixlib.h"
#define PIXLIB_H
#endif 

#ifndef STKLIB
#include "stack.h"
#define STKLIB
#endif 
 
/*  the following defines are used for mapping the alignment file columns
 *  into a structure used to store alignment records. 
 *
 *  ALIGNMENT FILE COLUMN MAPPINGS
 */
 
#define ALN_TSTART        0
#define ALN_TSTOP         1 
#define ALN_XPOS          2 
#define ALN_YPOS          3 
#define ALN_ZPOS          4 
#define ALN_XTHETA        5 
#define ALN_YTHETA        6 
#define ALN_ZTHETA        7 
#define ALN_DITHID        8 
#define ALN_DITHSTEP      9 
#define ALN_LASSZ        10 
#define ALN_GOOD_BAD     11 
#define ALN_STATE        12
#define ALN_TIME         13
#define ALN_UNKNOWN      14
 

/*  The following macros are used to define the known alignment file column
 *  names. These names are used in determining the mapping of file column
 *  data into a structure representing an alignment  file entry/record. 
 *
 *  ALIGNMENT FILE COLUMN NAMES
 */
 
#define ALN_TSTART_COL       "start"  
#define ALN_TSTART_COL_ALT1  "tstart"  
#define ALN_TSTART_COL_ALT2  "time"
#define ALN_TSTOP_COL        "stop"     
#define ALN_TSTOP_COL_ALT1   "tstop"     
#define ALN_XPOS_COL         "stf_x"   
#define ALN_YPOS_COL         "stf_y"   
#define ALN_YPOS_COL_ALT1    "dy"
#define ALN_ZPOS_COL         "stf_z"   
#define ALN_ZPOS_COL_ALT1    "dz"
#define ALN_XTHETA_COL       "stf_az"   
#define ALN_YTHETA_COL       "stf_el"   
#define ALN_ZTHETA_COL       "stf_roll"   
#define ALN_ZTHETA_COL_ALT1  "dtheta"
#define ALN_DITHID_COL       "dith_id"   
#define ALN_DITHSTEP_COL     "dith_st"   
#define ALN_LASSZ_COL        "sim_z"  
#define ALN_GOOD_BAD_COL     "gbflag"  
#define ALN_STATE_COL        "state"

#define ALN_EXTENSION        "ALIGN"
#define ALN_EXTENSION_ALT1   "ASPOFF"
#define ALN_EXTENSION_ALT2   "ASPSOL"
 
#define ASP_DELTA_T_KEY      "DTASPSOL"
#define ASP_DELTA_T_KEY_ALT1 "TIMEDEL"


/*  the following macros lay out the bit mapsused to determine if all of the
 *  columns necessary to perform alignment adjustments are contained in the 
 *  input file. The mask basically has a bit assigned to each required data 
 *  column. 
 *
 *  ALIGNMENT DEPENDENCY MASK 
 */ 

#define ALN_MASK_INIT        ((unsigned short) 0x000) 
#define ALN_TSTART_MASK      ((unsigned short) 0x001) 
#define ALN_TSTOP_MASK       ((unsigned short) 0x002) 
#define ALN_TIME_MASK        (ALN_TSTART_MASK | ALN_TSTOP_MASK)
#define ALN_XPOS_MASK        ((unsigned short) 0x004) 
#define ALN_YPOS_MASK        ((unsigned short) 0x008)
#define ALN_ZPOS_MASK        ((unsigned short) 0x010)
#define ALN_XTHETA_MASK      ((unsigned short) 0x020)
#define ALN_YTHETA_MASK      ((unsigned short) 0x040)
#define ALN_ZTHETA_MASK      ((unsigned short) 0x080)
#define ALN_LASSZ_MASK       ((unsigned short) 0x100) 

#define ALN_ALL_DEPS_MET     ((unsigned short) 0x1FF) 
#define ALN_FLT_DEPS_MET     ((unsigned short) 0x9B)
#define ALN_XRCF_DEPS_MET    ((unsigned short) 0x1FF)

#define ALN_FLIGHT_DATA      0
#define ALN_XRCF_DATA        1 


/*  the following structure holds all information pertaining to any single
 *  alignment record and is used to pass this data to routines for processing.
 *
 *  ALIGNMENT RECORD STRUCTURE
 */
 
typedef struct alignment_rec_t {
    double tstart;    /* start time of dither interval */
    double tstop;     /* end time of dither interval   */
    double state;     /* dither state value            */
    VEC3_DBLE pos;    /* dither x,y,z position         */ 
    VEC3_DBLE theta;  /* dither x,y,z angle            */ 
    VEC3_DBLE lass;   /* lass z value                  */ 
    VEC2_DBLE hpy;    /* hrma pitch yaw                */
    long   dithid;    /* id of dither record           */
    long   dithstep;  /*                               */
    short  good_bad;  /*                               */
} ALIGNMENT_REC_T, *ALIGNMENT_REC_P_T;
 


/*  The following structure holds the housekeeping type of parameters used
 *  by the alignment routines. This structure contains the alignment file 
 *  pointer, as well as information such as column data, and number of
 *  rows in the file.
 *
 *  ALIGNMENT FILE HOUSEKEEPING STRUCTURE 
 */

typedef struct alignment_infra_t {
       long        row_check;   /* flag to indicate last row read   */
       int         curr_row;    /* current row  read from exp tbl  */
       int         num_cols;    /* number of exposure table columns*/
       int         num_rows;    /* total number of rows in exp tbl */
       int         stk_cnt;     /* number of items in the stack    */
       short       processing;  /* flight (EVT_FLT_FORMAT) or xrcf */ 
       Stack       stack;       /* input aspect file(s) 'stack'     */
       dmDataset*  dataset;     /* dataset (file) handle            */
       dmBlock*    primary;     /* header keyword externsion handle */
       dmBlock*    extension;   /* event data extension handle      */
       dmDescriptor**  desc;    /* event attributes                 */
       short*      map;         /* mapping of file columns to data structure */
       char*       file;        /* name of aspect file              */
       boolean     update; 
       double      dtaspsol;  /* delta T for aspect solution file */
} ALIGNMENT_INFRA_T, *ALIGNMENT_INFRA_P_T; 



/*  the following are function prototypes of the various routines intended
 *  to be used for level 1 (and 1.5) alignment corrections for coordinate
 *  transformations.
 * 
 *  FUNCTION PROTOTYPES
 */

/* function to map a column of the dither file to the dth data structure */
extern short map_alignment_table_column(char[]);
 
/* function to verify that all dither file dependencies are met */
extern unsigned short alignment_file_dependencies(ALIGNMENT_INFRA_P_T);
 
/* function to load an dither column data field based on mapping */
extern void load_alignment_entry(ALIGNMENT_INFRA_P_T, ALIGNMENT_REC_P_T);
 
/* function to open a dither file table and setup the mapping */
extern boolean open_alignment_file(ALIGNMENT_INFRA_P_T);

/* function to manage dithering updates based on event time */ 
extern boolean alignment_update(ALIGNMENT_REC_P_T, ALIGNMENT_INFRA_P_T, 
                             double);

/* routine to close a dither file and deallocate memory */
extern void close_alignment_file(ALIGNMENT_INFRA_P_T); 

/* routine to set up alignment file stack */
extern void set_up_alignment_correction(char*, ALIGNMENT_INFRA_P_T,
                                        ALIGNMENT_REC_P_T,
                                        dsErrList*);

#endif /* must be last line of file */
