/*                                                                
**  Copyright (C) 1998-2000,2007  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


#ifndef L1_ASPECT_DEFS_H
#define L1_ASPECT_DEFS_H 


/*  the following are include files on which the aspect functions
 *  rely.
 *
 *  INCLUDE FILES
 */

#ifndef DSLIB_H
#include "dslib.h"
#define DSLIB_H
#endif 

#ifndef PIXLIB_H
#include "pixlib.h"
#define PIXLIB_H
#endif 

#ifndef STKLIB
#include "stack.h"
#define STKLIB
#endif

#ifndef LIBERR_H
#include "liberr.h"
#define LIBERR_H
#endif
 
/*  the following defines are used for internally mapping out the columns 
 *  of the input aspect file. These values are used in determining how to
 *  load the input data as well as for dependency checks.  
 *
 *  ASPECT FILE COLUMN MAPPINGS 
 */
 
#define ASP_TIME          0
#define ASP_MJD_INT       1 
#define ASP_MJD_FRAC      2 
#define ASP_ASP_QUAL      3 
#define ASP_STT_QUAL      4 
#define ASP_RA_SC         5 
#define ASP_DEC_SC        6 
#define ASP_ROAN_SC       7 
#define ASP_RA_ERR        8 
#define ASP_DEC_ERR       9 
#define ASP_ROAN_ERR     10 
#define ASP_OBI_NUM      11 
#define ASP_UNKNOWN      12 


 
/*  the following defines represent the names that the code expects the
 *  aspect file to use as column names. 
 *
 *  ASPECT FILE COLUMN NAMES
 */
 
#define ASP_TIME_COL        "time" 
#define ASP_TIME_COL_ALT1   "duration" 
#define ASP_MJD_INT_COL     "mjd_int" 
#define ASP_MJD_FRAC_COL    "mjd_frac" 
#define ASP_ASP_QUAL_COL    "asp_qual" 
#define ASP_STT_QUAL_COL    "stt_qual" 
#define ASP_RA_SC_COL       "x_offsets" 
#define ASP_RA_SC_COL_ALT1  "ra" 
#define ASP_DEC_SC_COL      "y_offsets" 
#define ASP_DEC_SC_COL_ALT1 "dec"
#define ASP_ROAN_SC_COL     "roll_offsets" 
#define ASP_ROAN_SC_COL_ALT1 "roll"
#define ASP_RA_ERR_COL      "ra_err" 
#define ASP_DEC_ERR_COL     "dec_err" 
#define ASP_ROAN_ERR_COL    "roan_err" 
#define ASP_OBI_NUM_COL     "obi_num" 


#define ASP_NOM_ROLL_KEY    "ROLL_NOM"
#define ASP_TYPE_KEY        "ASPTYPE"


#define ASP_FTYPE_NONE	    0
#define ASP_FTYPE_OFFSETS   1
#define ASP_FTYPE_SOLUTION  2


/*  the following defines are bit masks used by the aspect code to 
 *  determine if all of the appropriate data dependencies have been
 *  met. 
 *
 *  ASPECT DATA DEPENDENCY MASKS
 */

#define ASP_MASK_INIT        ((unsigned short) 0x000) 
#define ASP_TIME_MASK        ((unsigned short) 0x001)  
#define ASP_MJD_INT_MASK     ((unsigned short) 0x002)  
#define ASP_MJD_FRAC_MASK    ((unsigned short) 0x004)  
#define ASP_ASP_QUAL_MASK    ((unsigned short) 0x008)  
#define ASP_STT_QUAL_MASK    ((unsigned short) 0x010)  
#define ASP_RA_SC_MASK       ((unsigned short) 0x020)  
#define ASP_DEC_SC_MASK      ((unsigned short) 0x040)  
#define ASP_ROAN_SC_MASK     ((unsigned short) 0x080)  
#define ASP_RA_ERR_MASK      ((unsigned short) 0x100)  
#define ASP_DEC_ERR_MASK     ((unsigned short) 0x200)  
#define ASP_ROAN_ERR_MASK    ((unsigned short) 0x400)  
#define ASP_OBI_NUM_MASK     ((unsigned short) 0x800)  

#define ASP_ALL_DEPS_MET     ((unsigned short) 0xE1) 



/*  the following structure holds all information pertaining to any single
 *  aspect record and is used to pass this data to routines for processing.
 *
 *  ASPECT RECORD STRUCTURE
 */
 
typedef struct aspect_rec_t {
    double    time;      /* time field of aspect file */
    double    mjd_frac;  /*  */
    VEC3_DBLE asp_sol;   /* aspect solution- ra, dec, and roll angle */ 
    VEC3_DBLE asp_err;   /* aspect error- ra, dec, and roll angle */ 
    int       mjd_int;   /*  */
    short     asp_qual;  /*  */
    short     stt_qual;  /*  */
    short     obi_num;   /*  */ 
    unsigned short dep_mask; /* dependency mask */
} ASPECT_REC_T, *ASPECT_REC_P_T;
 


/*  the following structure maintains the information loaded from the
 *  aspect file. It stores two aspect records- the one being currently 
 *  used as well as the one which will be sued next. As items are read 
 *  from the input aspect file, they are loaded into the array position
 *  specified by 'next'. This position is toggled with the current 
 *  position whenever an entry needs to be read in.  
 *
 *  ASPECT FILE ENTRIES STRUCTURE 
 */ 

#define NUM_ASP_ENTRIES 2

typedef struct aspect_entries_t {
    short        curr;                 /* index to current aspect entry */
    short        next;         /* index to next sequential aspect entry */
    ASPECT_REC_T entry[NUM_ASP_ENTRIES];     /* array of aspect entries */ 
} ASPECT_ENTRY_T, *ASPECT_ENTRY_P_T; 



/*  The following structure holds the housekeeping type of parameters used
 *  by the aspectt routines. This structure contains the aspect file
 *  descriptor, as well as information such as column data, and number of
 *  rows in the file.
 *
 *  ASPECT FILE HOUSEKEEPING STRUCTURE
 */

typedef struct aspect_infra_t {
   double        nom_roll;    /* value of nominal roll header key */ 
   long          row_check;   /* flag to indicate last row read   */ 
   int           curr_row;    /* current row  read from exp tbl   */
   int           num_cols;    /* number of aspect table columns   */
   int           num_rows;    /* total number of rows in asp tbl  */
   int           stk_cnt;     /* number of elements in the stack  */  
   Stack         stack;       /* input aspect file(s) 'stack'     */
   dmDataset*    dataset;     /* dataset (file) handle            */
   dmBlock*      primary;     /* header keyword externsion handle */
   dmBlock*      extension;   /* event data extension handle      */
   dmDescriptor**  desc;      /* event attributes                 */
   short*        mapping;     /* mapping of file columns to data structure */
   char*         asp_file;    /* name of aspect file              */ 
   char          asp_type[DS_SZ_KEYWORD];  /* value of asptype key*/
   short	 asp_file_type; /* flag indicating aspect offsets or solution */
} ASPECT_INFRA_T, *ASPECT_INFRA_P_T; 



/*  the following are function prototypes of the various routines intended
 *  to be used for level 1 (and 1.5) aspect corrections for coordinate
 *  transformations.
 *
 *  FUNCTION PROTOTYPES
 */

/* function to map a column of the aspect file to the asp data structure */
extern short map_aspect_table_column(char[]);
 
/* function to verify that all aspect file dependencies are met */
extern unsigned short aspect_file_dependencies(ASPECT_INFRA_P_T);
 
/* function to load an aspect column data field based on mapping */
extern void load_aspect_entry(ASPECT_INFRA_P_T, ASPECT_ENTRY_P_T);
 
/* function to open an aspect file table and setup the mapping */
extern boolean open_aspect_file(ASPECT_INFRA_P_T);

extern boolean aspect_update(ASPECT_ENTRY_P_T, ASPECT_INFRA_P_T, 
                             double);

/* function to intiailize aspect correction files/stack */ 
extern void set_up_aspect_correction(char*, ASPECT_INFRA_P_T, 
                                     ASPECT_ENTRY_P_T,
                                     dsErrList*);

/* routine to close an aspect file and deallocate memory */
extern void close_aspect_file(ASPECT_INFRA_P_T);

#endif /* must be last line of file */
