#! /bin/sh

# 17 April 2008

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.

# 19 Feb 2003:
# !!10, !!11, !!12 and !!14, !!15, !!16 have been replaced with
# a dmdiff's instead of dmlist + keyfilter

# !!1
# mytool.t
# test script for mytool


# !!2
# syntax:
# mytool.t [<testid> ... ]




######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM,
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
  punlearn $1
}

######################################################################
# subroutine
# zerotest <file>
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="acis_find_afterglow"

# NOTE: test5 and test5a/b have nothing to do with each other
# however they were setup in parallel and changing the output products
# seemed more of a hassel then it's worth. If it it get's really confusing
# the names can always be updated accordingly.

# set up list of tests
# !!4
alltests="test1 test2 test3 test4 test5 test5a test5b"

# "short" test to run
# !!5
shortlist="test1 test2 test3 test4 test5 test5a test5b"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined"
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on
# both the output and reference files of a tool the resultant utility output
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE

  badId="badpix0 badpix1 badpix2 badpix3 badpix6 badpix7"

  ####################################################################
  # run the tool
  case ${testid} in
    # tests obi 2556
    test1 ) test_string1="acis_find_afterglow infile=$INDIR/evt1_input.fits outfile=$OUTDIR/test1.fits badpixfile=$INDIR/badpix1.fits maskfile=$INDIR/maskfile1.fits statfile=$INDIR/statfile1.fits clobber=yes verbose=0"

            echo $test_string1 | tee -a  $LOGFILE
      eval $test_string1  | tee -a  $LOGFILE  2>&1

            ;;

    # tests obi 6111
    test2 ) test_string2="acis_find_afterglow infile=$INDIR/evt2_input.fits outfile=$OUTDIR/test2.fits badpixfile=$INDIR/badpix2.fits maskfile=$INDIR/maskfile2.fits statfile=$INDIR/statfile2.fits clobber=yes verbose=0"

            echo $test_string2 | tee -a  $LOGFILE
      eval $test_string2  | tee -a  $LOGFILE  2>&1

           ;;

    # tests obi 3752
    # tests regwidth parameter
    test3 ) test_string3="acis_find_afterglow infile=$INDIR/evt3_input.fits outfile=$OUTDIR/test3.fits badpixfile=$INDIR/badpix3.fits maskfile=$INDIR/maskfile3.fits statfile=$INDIR/statfile3.fits regwidth=51 clobber=yes verbose=0"

            echo $test_string3 | tee -a  $LOGFILE
      eval $test_string3  | tee -a  $LOGFILE  2>&1

           ;;

    # tests obi 1902
    # tests "t-plane latchup" dataset
    test4 ) test_string4="acis_find_afterglow infile=$INDIR/evt4_input.fits outfile=$OUTDIR/test4.fits badpixfile=$INDIR/badpix4.fits maskfile=$INDIR/maskfile4.fits statfile=$INDIR/statfile4.fits clobber=yes verbose=0"

            echo $test_string4 | tee -a  $LOGFILE
      eval $test_string4  | tee -a  $LOGFILE  2>&1

           ;;

    # tests data set that caused issues with long double precision, so reverted to double precision
    test5 ) test_string5="acis_find_afterglow infile=$INDIR/evt5_input.fits outfile=$OUTDIR/test5.fits badpixfile=$INDIR/badpix5.fits maskfile=$INDIR/maskfile5.fits statfile=$INDIR/statfile5.fits expnowindow=10 probthresh=0.001 cntthresh=4 regwidth=51 nfpixreg=32 nfrepeat=10 tolerance=1e-15 runhotpix=yes clobber=yes verbose=0"

            echo $test_string5 | tee -a  $LOGFILE
      eval $test_string5  | tee -a  $LOGFILE  2>&1

           ;;

    #PTOBST-10: Rejection image mask syntax
    #Empty mask
    test5a ) test_string5a="acis_find_afterglow infile=$INDIR/evt1_input.fits outfile=$OUTDIR/test5a.fits badpixfile=$INDIR/badpix1.fits maskfile='mask()' statfile=$INDIR/statfile1.fits clobber=yes verbose=0"
       echo $test_string5a | tee -a  $LOGFILE
       eval $test_string5a
       mystatus=$?
       echo " -- status of call was $mystatus"  >>  $LOGFILE
       echo " -- status of call was $mystatus"  > ${outfile}.status

            ;;
    #Pointer to mask file
    test5b ) test_string5b="acis_find_afterglow infile=$INDIR/evt1_input.fits outfile=$OUTDIR/test5b.fits badpixfile=$INDIR/badpix1.fits maskfile='mask($INDIR/maskfile1.fits)' statfile=$INDIR/statfile1.fits clobber=yes verbose=0"
      echo $test_string5b | tee -a  $LOGFILE
      eval $test_string5b
      mystatus=$?
      echo " -- status of call was $mystatus"  >>  $LOGFILE
      echo " -- status of call was $mystatus"  > ${outfile}.status


            ;;

  esac


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  # dmlist $outfile header,data > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
  # dmlist $savfile header,data > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #          2>>$LOGFILE

  # compare
  # !!12
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #      /dev/null 2>>$LOGFILE
  if [ ${testid} = "test5a" ] || [ ${testid} = "test5b" ]; then
     # this test checks for expected errors so diff the exit status of run
     diff $outfile.status $savfile.status  > /dev/null 2>>$LOGFILE
     if  test $? -ne 0 ; then
       echo "ERROR: MISMATCH in $outfile.status" >> $LOGFILE
       mismatch=0
     fi
  else
   dmdiff $outfile $savfile keys- tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
   if  test $? -ne 0 ; then
     echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
     mismatch=0
   fi
  fi

#  \rm -f $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  echo ""
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
