#! /bin/sh

# 27 January 2005

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# acis_process_events.t
# test script for acis_process_events


# !!2
# syntax:
# acis_process_events.t [<testid> ... ]




######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}


######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}


######################################################################
# Initialization

# !!3
toolname="acis_process_events"

# set up list of tests
# !!4
alltests="test1 test1b test_aspsolfile test2 test2_expno test3 test4 test4b test5 \
           test6 test7 test7b test8 test9 test10 test10b test11 test12 \
           test13 test13_3D apply_cti_faint apply_cti_graded apply_cti_timed_graded gratings_cc \
           gratings_cc_2 gratings_cc_3 gratings_cc_4 \
           grade_file test_cal test_lev0 test_frame_store test_frame_store2 test_lev0_2 \
           test_pha_ro test_pha_ro_2 test_pha_ro_3 test_pha_ro_4 \
           test_cti_app_key test_cti_app_key2 test_subpix_edser test_subpix_centroid1 \
           test_subpix_centroid2 test_subpix_centroid3 test_subpix_centroid4 \
           test_subpix_centroid5 test_subpix_rand test_subpix_none blank_sky lev3"


# "short" test to run
# !!5
shortlist="test4 test7 test10 test12 test13 test13_3D apply_cti_faint"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}.log

#get rid of old logs
rm -f $LOGDIR/${toolname}.log


# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined"
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on
# both the output and reference files of a tool the resultant utility output
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff


# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose
#

script_succeeded=0

#set up regression CALDB
#source /data/regression_test/development/CALDB/software/tools/caldbinit.unix;

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}_out.fits
  savfile=$SAVDIR/${testid}_sav.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  punlearn $toolname

  case ${testid} in

    # FAINT
    # apply gain map and coord transforms
    # no CTI
    # no TGAIN
    # Obsid = 1838
    test1 ) test1_string="acis_process_events \
		    infile=$INDIR/${testid}_in.fits \
		    outfile=${outfile} \
		    apply_cti   = "no" \
		    apply_tgain = "no" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test1_string | tee -a  $LOGFILE
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
           ;;

    # FAINT
    # apply gain map and coord transforms
    # no CTI
    # no TGAIN
    # with asol and flt files
    # Obsid = 1838
    test1b ) test1b_string="acis_process_events \
        infile=$INDIR/test1_in.fits \
        outfile=${outfile} \
        acaofffile=$INDIR/pcadf084244404N003_asol1.fits \
        apply_cti   = "no" \
        apply_tgain = "no" \
              pix_adj=NONE mode=h > /dev/null"
      echo $test1b_string | tee -a  $LOGFILE
      eval $test1b_string  | tee -a  $LOGFILE  2>&1
           ;;

    #checks the new aspect solution content keyword aspsolobi
    test_aspsolfile ) test_aspsolfile_string="acis_process_events \
        infile=$INDIR/test1_in.fits \
        outfile=${outfile} \
        acaofffile=$INDIR/pcadf084244404N003_asolobi1.fits \
        apply_cti   = "no" \
        apply_tgain = "no" \
              pix_adj=NONE mode=h > /dev/null"
      echo $test_aspsolfile_string | tee -a  $LOGFILE
      eval $test_aspsolfile_string  | tee -a  $LOGFILE  2>&1
           ;;

    # FAINT
    # apply gain map and coord transforms
    # CTI
    # no TGAIN
    # Obsid = 1838
    test2 ) test2_string="acis_process_events \
		    infile=$INDIR/${testid}_in.fits \
		    outfile=${outfile} \
		    apply_cti   = "yes" \
		    apply_tgain = "no" \
        eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,f:phas_adj,l:pha,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
      echo $test2_string | tee -a  $LOGFILE
	    eval $test2_string  | tee -a  $LOGFILE  2>&1
            ;;

    # 9/29/2015 - same as test2 except mv expno before ccd_id, and use L data type
    #  note:
    #     % dmdiff  test2_expno_out.fits  test2_out.fits  - shows no difference.
    #     % dmlist  <file> col   - shows the expno column being placed in a different order
    #                            - both files have expno 'Int4'
    test2_expno ) test2_string="acis_process_events \
		    infile=$INDIR/test2_in.fits \
		    outfile=${outfile} \
		    apply_cti   = "yes" \
		    apply_tgain = "no" \
        eventdef=\"{d:time,l:expno,s:ccd_id,s:node_id,s:chip,s:tdet,f:det,f:sky,s:phas,f:phas_adj,l:pha,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
      echo $test2_string | tee -a  $LOGFILE
	    eval $test2_string  | tee -a  $LOGFILE  2>&1
            ;;

    # FAINT
    # apply gain map and coord transforms
    # no CTI
    # TGAIN
    # Obsid = 1838
    test3 ) test3_string="acis_process_events \
		    infile=$INDIR/${testid}_in.fits \
		    outfile=${outfile} \
		    apply_cti   = "no" \
		    apply_tgain = "yes" \
                    eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test3_string | tee -a  $LOGFILE
	    eval $test3_string  | tee -a  $LOGFILE  2>&1
           ;;

    # FAINT
    # apply gain map and coord transforms
    # CTI
    # TGAIN
    # Obsid = 1838
    test4 ) test4_string="acis_process_events \
		    infile=$INDIR/${testid}_in.fits \
		    outfile=${outfile} \
		    apply_cti   = "yes" \
		    apply_tgain = "yes" \
                    eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,f:phas_adj,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test4_string | tee -a  $LOGFILE
	    eval $test4_string  | tee -a  $LOGFILE  2>&1
           ;;

    # FAINT
    # apply gain map and coord transforms
    # CTI
    # TGAIN
    # with asol and flt files
    # Obsid = 1838
    test4b ) test4b_string="acis_process_events \
        infile=$INDIR/test4_in.fits \
        outfile=${outfile} \
        acaofffile=$INDIR/pcadf084244404N003_asol1.fits \
        apply_cti   = "yes" \
        apply_tgain = "yes" \
                    eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,f:phas_adj,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
              pix_adj=NONE mode=h > /dev/null"
      echo $test4b_string | tee -a  $LOGFILE
      eval $test4b_string  | tee -a  $LOGFILE  2>&1
           ;;

    ####################
    #  VERY FAINT
    ####################

    # VFAINT
    # apply gain map and coord transforms
    # no CTI
    # no TGAIN
    # no CHECK_VF
    # Obsid = 884
    test5 ) test5_string="acis_process_events \
		    infile=$INDIR/${testid}_in.fits \
		    outfile=${outfile} \
		    apply_cti    = "no" \
		    apply_tgain  = "no" \
                    check_vf_pha = "no" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test5_string | tee -a  $LOGFILE
	    eval $test5_string  | tee -a  $LOGFILE  2>&1
           ;;

    # VFAINT
    # apply gain map and coord transforms
    # no CTI
    # no TGAIN
    # CHECK_VF
    # Obsid = 884
    test6 ) test6_string="acis_process_events \
		    infile=$INDIR/${testid}_in.fits \
		    outfile=${outfile} \
		    apply_cti    = "no" \
		    apply_tgain  = "no" \
                    check_vf_pha = "yes" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test6_string | tee -a  $LOGFILE
	    eval $test6_string  | tee -a  $LOGFILE  2>&1
           ;;

    # VFAINT
    # apply gain map and coord transforms
    # CTI
    # TGAIN
    # CHECK_VF
    # Obsid = 884
    test7 ) test7_string="acis_process_events \
		    infile=$INDIR/${testid}_in.fits \
		    outfile=${outfile} \
		    apply_cti    = "yes" \
		    apply_tgain  = "yes" \
                    check_vf_pha = "yes" \
                    eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,f:phas_adj,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test7_string | tee -a  $LOGFILE
	    eval $test7_string  | tee -a  $LOGFILE  2>&1
           ;;

    # VFAINT
    # apply gain map and coord transforms
    # CTI
    # TGAIN
    # CHECK_VF
    # with asol and flt
    # Obsid = 884
    test7b ) test7b_string="acis_process_events \
        infile=$INDIR/test7_in.fits \
        outfile=${outfile} \
        acaofffile=$INDIR/pcadf083202635N003_asol1.fits \
        apply_cti    = "yes" \
        apply_tgain  = "yes" \
                    check_vf_pha = "yes" \
                    eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,f:phas_adj,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
              pix_adj=NONE mode=h > /dev/null"
      echo $test7b_string | tee -a  $LOGFILE
      eval $test7b_string  | tee -a  $LOGFILE  2>&1
           ;;

    ####################
    #  CC - MODE
    ####################

    # CC
    # apply gain map and coord transforms
    # no CTI
    # no TGAIN
    # no TOA
    # Obsid = 3927
    test8 ) test8_string="acis_process_events \
		    infile=$INDIR/3927_evt1.fits \
		    outfile=${outfile} \
		    acaofffile=$INDIR/pcadf174405532N003_asol1.fits \
		    gainfile='CALDB' \
                    apply_cti     = "no" \
                    apply_tgain   = "no" \
                    eventdef=\")cclev1\" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test8_string | tee -a  $LOGFILE
	    eval $test8_string  | tee -a  $LOGFILE  2>&1
           ;;


    # CC
    # apply gain map and coord transforms
    # no CTI
    # no TGAIN
    # TOA
    # Obsid = 3927
    test9 ) test9_string="acis_process_events \
		    infile=$INDIR/3927_evt1.fits \
		    outfile=${outfile} \
		    acaofffile=$INDIR/pcadf174405532N003_asol1.fits \
        gainfile='CALDB' \
                    apply_cti     = "no" \
                    apply_tgain   = "no" \
                    eventdef=\"{d:time,d:time_ro,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,f:sky_1d,s:phas,l:pha,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test9_string | tee -a  $LOGFILE
	    eval $test9_string  | tee -a  $LOGFILE  2>&1
           ;;

    # CC
    # apply gain map and coord transforms
    # CTI
    # TGAIN
    # TOA
    # Obsid = 3927
    test10 ) test10_string="acis_process_events \
		    infile=$INDIR/3927_evt1.fits \
		    outfile=${outfile} \
		    acaofffile=$INDIR/pcadf174405532N003_asol1.fits \
                    apply_cti     = "yes" \
                    apply_tgain   = "yes" \
                    eventdef=\"{d:time,d:time_ro,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,f:phas_adj,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test10_string | tee -a  $LOGFILE
	    eval $test10_string  | tee -a  $LOGFILE  2>&1
           ;;


    # CC
    # apply gain map and coord transforms
    # CTI
    # TGAIN
    # TOA
    # with flt file
    # Obsid = 3927
    test10b ) test10b_string="acis_process_events \
        infile=$INDIR/3927_evt1.fits \
        outfile=${outfile} \
        acaofffile=$INDIR/pcadf174405532N003_asol1.fits \
                    apply_cti     = "yes" \
                    apply_tgain   = "yes" \
                    eventdef=\"{d:time,d:time_ro,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,f:phas_adj,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
              pix_adj=NONE mode=h > /dev/null"
      echo $test10b_string | tee -a  $LOGFILE
      eval $test10b_string  | tee -a  $LOGFILE  2>&1
           ;;

    ####################
    #  GRADED
    ####################

    # GRADED TE
    # apply gain map and coord transforms
    # no CTI
    # TGAIN
    # Obsid = 680
    test11 ) test11_string="acis_process_events \
		    infile=$INDIR/680_evt1.fits \
		    outfile=${outfile} \
		    acaofffile=$INDIR/pcadf073776186N004_asol1.fits \
		    gainfile="CALDB" \
		    tgainfile="CALDB" \
                    apply_cti     = "no" \
                    apply_tgain   = "yes" \
                    eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test11_string | tee -a  $LOGFILE
	    eval $test11_string  | tee -a  $LOGFILE  2>&1
           ;;

    # GRADED CC
    # apply gain map and coord transforms
    # no CTI
    # TGAIN
    # TOA
    # Obsid = 680
    test12 ) test12_string="acis_process_events \
		    infile=$INDIR/680_evt1.fits \
		    outfile=${outfile} \
		    acaofffile=$INDIR/pcadf073776186N004_asol1.fits \
		    gainfile="CALDB" \
		    tgainfile="CALDB" \
                    apply_cti     = "no" \
                    apply_tgain   = "yes" \
                    eventdef=\"{d:time,d:time_ro,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
	            pix_adj=NONE mode=h > /dev/null"
	    echo $test12_string | tee -a  $LOGFILE
	    eval $test12_string  | tee -a  $LOGFILE  2>&1
           ;;

    # Obsid = 426
    # GRADED MODE CTI Correction
    test13 ) test13_string="acis_process_events \
        infile=$INDIR/${testid}_in.fits \
        outfile=${outfile} \
        badpixfile=$INDIR/${testid}_in_bpix.fits \
        acaofffile=$INDIR/${testid}_in_acaoff.fits \
        ctifile=$INDIR/${testid}_in_cti.fits \
        tgainfile=$INDIR/acisD2000-01-29t_gainN0005.fits \
        grade_image_file=$INDIR/${testid}_in_grade_image.fits \
        apply_cti="yes" \
        apply_tgain="yes" \
        pix_adj=NONE mode=h > /dev/null"
        echo $test13_string | tee -a  $LOGFILE
        eval $test13_string  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 426
    # GRADED MODE CTI Correction - CTI with TEMP LO/HI/IND and 3D data
    test13_3D ) test13_3D_string="acis_process_events \
        infile=$INDIR/test13_in.fits \
        outfile=${outfile} \
        badpixfile=$INDIR/test13_in_bpix.fits \
        acaofffile=$INDIR/test13_in_acaoff.fits \
        ctifile=$INDIR/${testid}_in_cti.fits \
        tgainfile=$INDIR/acisD2000-01-29t_gainN0005.fits \
        grade_image_file=$INDIR/test13_in_grade_image.fits \
        apply_cti="yes" \
        apply_tgain="yes" \
        pix_adj=NONE mode=h > /dev/null"
        echo $test13_3D_string | tee -a  $LOGFILE
        eval $test13_3D_string | tee -a  $LOGFILE  2>&1
        ;;


    # Obsid = 4654
    # GRADED MODE CTI Correction CC33_FAINT
    apply_cti_faint ) apply_cti_faint_string="acis_process_events \
        infile=$INDIR/apply_cti_faint/acism04654_000N000_tmp_evt1.fits[#row=1:100000] \
        obsfile=$INDIR/apply_cti_faint/axafm04654_000N000_obs0a.par \
        tgainfile=$INDIR/apply_cti_faint/acisD2004-05-01t_gainN0006.fits \
        outfile=${outfile} \
        badpixfile=$INDIR/apply_cti_faint/acism04654_000N000_bpix1.fits \
        acaofffile=@$INDIR/apply_cti_faint/pcadm04654_000N000_asol1.lis \
        mtlfile=$INDIR/apply_cti_faint/acism04654_000N000_fptemp_egti1.fits \
        ctifile=$INDIR/apply_cti_faint/acisD2002-08-01ctiN0008.fits \
        apply_cti="yes" \
        apply_tgain="yes" \
        rand_pha="no" \
        pix_adj=NONE mode=h > /dev/null"
        echo $apply_cti_faint_string | tee -a  $LOGFILE
        eval $apply_cti_faint_string  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 4568
    # GRADED MODE CTI Correction CC33_GRADED
    apply_cti_graded ) apply_cti_graded_string="acis_process_events \
        infile=$INDIR/apply_cti_graded/acism04568_002N000_tmp_evt1.fits[#row=1:100000] \
        obsfile=$INDIR/apply_cti_graded/axafm04568_002N000_obs0a.par \
        outfile=${outfile} \
        badpixfile=$INDIR/apply_cti_graded/acism04568_002N000_bpix1.fits \
        acaofffile=@$INDIR/apply_cti_graded/pcadm04568_002N000_asol1.lis \
        mtlfile=$INDIR/apply_cti_graded/acism04568_002N000_fptemp_egti1.fits \
        tgainfile="${INDIR}/apply_cti_graded/acisD2004-08-01t_gainN0005.fits" \
        grade_image_file="${INDIR}/apply_cti_graded/acisD2000-01-29grdimgN0001.fits" \
        apply_cti="yes" \
        apply_tgain="yes" \
        rand_pha="no" \
        pix_adj=NONE mode=h > /dev/null"
        echo $apply_cti_graded_string | tee -a  $LOGFILE
        eval $apply_cti_graded_string  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 425
    # GRADED MODE CTI timed GRADED
    apply_cti_timed_graded ) apply_cti_timed_graded_string="acis_process_events \
        infile=$INDIR/apply_cti_timed_graded/acism00425_000N000_e2_tmp_evt1.fits[#row=1:100000] \
        obsfile=$INDIR/apply_cti_timed_graded/axafm00425_000N000_obs0a.par \
        outfile=${outfile} \
        badpixfile=$INDIR/apply_cti_timed_graded/acism00425_000N000_e2_bpix1.fits \
        acaofffile=@$INDIR/apply_cti_timed_graded/pcadm00425_000N000_e2_asol1.lis \
        mtlfile=$INDIR/apply_cti_timed_graded/acism00425_000N000_e2_mtl1.fits \
        grade_image_file="${INDIR}/apply_cti_timed_graded/acisD2000-01-29grdimgN0001.fits" \
        tgainfile="${INDIR}/apply_cti_timed_graded/acisD2004-08-01t_gainN0005.fits" \
        apply_cti="yes" \
        apply_tgain="yes" \
        rand_pha="no" \
        pix_adj=NONE mode=h > /dev/null"
        echo $apply_cti_timed_graded_string | tee -a  $LOGFILE
        eval $apply_cti_timed_graded_string  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 6297
    # CC MODE Gratings
    gratings_cc ) gratings_cc_string="acis_process_events \
        infile=$INDIR/6297_tg_evt1a.fits \
        outfile=${outfile} \
        obsfile=$INDIR/acisf06297_000N004_obs1.par \
        badpixfile=NONE \
        eventdef=\"{d:time,i:expno,f:rd,s:chip,s:tdet,f:det,f:sky,s:ccd_id,s:phas,l:pha,s:pi,f:energy,s:grade,s:fltgrade,s:node_id,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,f:chipy_tg,f:chipy_zo,l:pha_ro,x:status}\" \
        acaofffile=@$INDIR/acisf06297_000N004_asol1.lis \
        mtlfile=@$INDIR/acisf06297_000N004_mtl1.lis \
        subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
        ctifile=$INDIR/acisD2002-08-01ctiN0007.fits \
        tgainfile=$INDIR/acisD2005-05-01t_gainN0006.fits \
        apply_cti="yes" \
        apply_tgain="yes" \
        rand_pha="no" \
        pix_adj=EDSER mode=h > /dev/null"
        echo $gratings_cc_string | tee -a  $LOGFILE
        eval $gratings_cc_string  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 6297
    # CC MODE Gratings   ( this is same as 'gratings_cc' except set pix_adj=RANDOMIZE )
    gratings_cc_2 ) gratings_cc_string="acis_process_events \
        infile=$INDIR/6297_tg_evt1a.fits \
        outfile=${outfile} \
        obsfile=$INDIR/acisf06297_000N004_obs1.par \
        badpixfile=NONE \
        eventdef=\"{d:time,i:expno,f:rd,s:chip,s:tdet,f:det,f:sky,s:ccd_id,s:phas,l:pha,s:pi,f:energy,s:grade,s:fltgrade,s:node_id,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,f:chipy_tg,f:chipy_zo,l:pha_ro,x:status}\" \
        acaofffile=@$INDIR/acisf06297_000N004_asol1.lis \
        mtlfile=@$INDIR/acisf06297_000N004_mtl1.lis \
        subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
        ctifile=$INDIR/acisD2002-08-01ctiN0007.fits \
        tgainfile=$INDIR/acisD2005-05-01t_gainN0006.fits \
        apply_cti="yes" \
        apply_tgain="yes" \
        pix_adj=RANDOMIZE mode=h > /dev/null"
        echo $gratings_cc_string | tee -a  $LOGFILE
        eval $gratings_cc_string  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 6297
    # CC MODE Gratings - same as gratings_cc but includes the new ctifile columns, but set to zero
    gratings_cc_3 ) gratings_cc_string="acis_process_events \
        infile=$INDIR/6297_tg_evt1a.fits \
        outfile=${outfile} \
        obsfile=$INDIR/acisf06297_000N004_obs1.par \
        badpixfile=NONE \
        eventdef=\"{d:time,i:expno,f:rd,s:chip,s:tdet,f:det,f:sky,s:ccd_id,s:phas,l:pha,s:pi,f:energy,s:grade,s:fltgrade,s:node_id,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,f:chipy_tg,f:chipy_zo,l:pha_ro,x:status}\" \
        acaofffile=@$INDIR/acisf06297_000N004_asol1.lis \
        mtlfile=@$INDIR/acisf06297_000N004_mtl1.lis \
        subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
        ctifile=$INDIR/acisD2002-08-01ctiN0007_zeros.fits \
        tgainfile=$INDIR/acisD2005-05-01t_gainN0006.fits \
        apply_cti="yes" \
        apply_tgain="yes" \
        rand_pha="no" \
        pix_adj=EDSER mode=h > /dev/null"
        echo $gratings_cc_string | tee -a  $LOGFILE
        eval $gratings_cc_string  | tee -a  $LOGFILE  2>&1
        ;;
    # Obsid = 6297
    # CC MODE Gratings - same as gratings_cc but includes the new ctifile columns, but set to artificial values
    gratings_cc_4 ) gratings_cc_string="acis_process_events \
        infile=$INDIR/6297_tg_evt1a.fits \
        outfile=${outfile} \
        obsfile=$INDIR/acisf06297_000N004_obs1.par \
        badpixfile=NONE \
        eventdef=\"{d:time,i:expno,f:rd,s:chip,s:tdet,f:det,f:sky,s:ccd_id,s:phas,l:pha,s:pi,f:energy,s:grade,s:fltgrade,s:node_id,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,f:chipy_tg,f:chipy_zo,l:pha_ro,x:status}\" \
        acaofffile=@$INDIR/acisf06297_000N004_asol1.lis \
        mtlfile=@$INDIR/acisf06297_000N004_mtl1.lis \
        subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
        ctifile=$INDIR/acisD2002-08-01ctiN0007_new.fits \
        tgainfile=$INDIR/acisD2005-05-01t_gainN0006.fits \
        apply_cti="yes" \
        apply_tgain="yes" \
        rand_pha="no" \
        pix_adj=EDSER mode=h > /dev/null"
        echo $gratings_cc_string | tee -a  $LOGFILE
        eval $gratings_cc_string  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 11824
    # GRADED MODE User Supplied Grade File
    grade_file ) grade_file_string="acis_process_events \
        infile=$INDIR/${testid}_in.fits \
        outfile=${outfile} \
        acaofffile=$INDIR/pcadf378021907N002_asol1.fits \
        apply_cti="no" \
        apply_tgain="no" \
        gradefile=\"$INDIR/${testid}_in_grade.fits[AXAF_GRADE3]\" \
        badpixfile="none" \
        mtlfile="none" \
        eventdef=\")cclev1\" \
        doevtgrade="yes" \
        calculate_pi="no" \
        stop="none" \
        rand_pha="no" \
        pix_adj="NONE" mode=h > /dev/null"
        echo $grade_file_string | tee -a  $LOGFILE
        eval $grade_file_string  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 62724
    # Obs_mode = secondary, calibration file
    # tests removal of calc_cc_times
    test_cal ) test_cal_string="acis_process_events \
        infile=$INDIR/${testid}_in.fits \
        outfile=${outfile} \
        acaofffile="NONE" \
        badpixfile="$INDIR/acisf62724_000N005_bpix1.fits" \
        mtlfile="$INDIR/acisf62724_000N005_mtl1.fits" \
        eventdef=\"{d:time,d:time_ro,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        ctifile="CALDB" \
        gainfile="$INDIR/acisD1999-07-22gainN0002.fits" \
        tgainfile="CALDB" \
        gradefile="CALDB" \
        grade_image_file="CALDB" \
        threshfile="CALDB" \
        apply_cti="no" \
        apply_tgain="no" \
        geompar="geom" \
        pix_adj="NONE" mode=h > /dev/null"
        echo $test_cal_string | tee -a $LOGFILE
        eval $test_cal_string | tee -a $LOGFILE 2>&1
        ;;

  #Test lev0 data sets
  #Obsid = 680
    test_lev0 ) test_lev0_string="acis_process_events \
        infile=$INDIR/${testid}_in.fits \
        outfile=${outfile} \
        acaofffile="$INDIR/pcadf073776186N004_asol1.fits" \
        badpixfile="$INDIR/acisf00680_000N001_bpix1.fits" \
        mtlfile="$INDIR/acisf00680_000N001_mtl1.fits" \
        eventdef=\"{d:time,d:time_ro,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        ctifile="CALDB" \
        gainfile="CALDB" \
        tgainfile="CALDB" \
        gradefile="CALDB" \
        grade_image_file="CALDB" \
        threshfile="CALDB" \
        apply_cti="no" \
        apply_tgain="no" \
        rand_pha="no" \
        geompar="geom" \
        subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
        pix_adj="EDSER" mode=h > /dev/null"
        echo $test_lev0_string | tee -a $LOGFILE
        eval $test_lev0_string | tee -a $LOGFILE 2>&1
        ;;

  #Test frame-store bit
  #Obsid = 680
  #datamode = CC33_GRADED
  test_frame_store ) test_frame_store_string="acis_process_events \
      infile=$INDIR/test_lev0_in.fits \
      outfile=${outfile} \
      acaofffile="$INDIR/pcadf073776186N004_asol1.fits" \
      badpixfile="$INDIR/frame_store_bpix1.fits" \
      mtlfile="$INDIR/acisf00680_000N001_mtl1.fits" \
      eventdef=\"{d:time,d:time_ro,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
      ctifile="CALDB" \
      gainfile="CALDB" \
      tgainfile="CALDB" \
      gradefile="CALDB" \
      grade_image_file="CALDB" \
      threshfile="CALDB" \
      apply_cti="no" \
      apply_tgain="no" \
      rand_pha="no" \
      geompar="geom" \
      subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
      pix_adj="EDSER" mode=h > /dev/null"
      echo $test_frame_store_string | tee -a $LOGFILE
      eval $test_frame_store_string | tee -a $LOGFILE 2>&1
      ;;

    #Obsid = 62836
    #datamode = VFAINT
    test_frame_store2 ) test_frame_store2_string="acis_process_events \
        infile=$INDIR/test_lev0_2_in.fits \
        outfile=${outfile} \
        acaofffile="NONE" \
        badpixfile="$INDIR/frame_store2_bpix1.fits" \
        mtlfile="$INDIR/acisf62836_000N001_mtl1.fits" \
        eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        doevtgrade="yes" \
        apply_cti="yes" \
        apply_tgain="yes" \
        tgainfile="$INDIR/acisD2004-11-01t_gainN0006.fits" \
        ctifile=$INDIR/acisD2002-08-01ctiN0007.fits \
        geompar="geom" \
        stop="sky" \
        rand_pha="no" \
        subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
        pix_adj="EDSER" mode=h > /dev/null"
        echo $test_frame_store2_string | tee -a $LOGFILE
        eval $test_frame_store2_string | tee -a $LOGFILE 2>&1
        ;;

    #Obsid = 62836
    test_lev0_2 ) test_lev0_2_string="acis_process_events \
        infile=$INDIR/${testid}_in.fits \
        outfile=${outfile} \
        acaofffile="NONE" \
        badpixfile="$INDIR/acisf62836_000N001_bpix1.fits" \
        mtlfile="$INDIR/acisf62836_000N001_mtl1.fits" \
        eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        doevtgrade="yes" \
        apply_cti="yes" \
        apply_tgain="yes" \
        tgainfile="$INDIR/acisD2004-11-01t_gainN0006.fits" \
        ctifile=$INDIR/acisD2002-08-01ctiN0007.fits \
        geompar="geom" \
        stop="sky" \
        rand_pha="no" \
        subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
        pix_adj="EDSER" mode=h > /dev/null"
        echo $test_lev0_2_string | tee -a $LOGFILE
        eval $test_lev0_2_string | tee -a $LOGFILE 2>&1
        ;;

   #Obsid = 1225
   #Test pha_ro values in non cti corrected data set
   test_pha_ro ) test_pha_ro_string="acis_process_events \
        infile="$INDIR/acisf01225_000N004_evt1.fits[ccd_id=5][expno=79][chipx=284]" \
        outfile=${outfile} \
        acaofffile="$INDIR/pcadf058683814N003_asol1.fits" \
        apply_cti=yes \
        apply_tgain=yes \
        badpixfile="$INDIR/acisf01225_000N004_bpix1.fits" \
        mtlfile="$INDIR/acisf01225_000N004_mtl1.fits" \
        tgainfile="$INDIR/acisD1999-09-16t_gainN0005.fits" \
        eventdef=\"{d:time,d:time_ro,l:expno,s:ccd_id,s:node_id,s:chip,s:tdet,f:det,f:sky,f:sky_1d,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        doevtgrade=yes \
        calculate_pi=yes \
        stop="sky" \
        rand_pha="no" \
        pix_adj="EDSER" \
        subpixfile="$INDIR/acisD1999-07-22subpixN0001.fits" mode=h > /dev/null"
        echo $test_pha_ro_string | tee -a $LOGFILE
        eval $test_pha_ro_string | tee -a $LOGFILE 2>&1
        ;;

   test_pha_ro_2 ) test_pha_ro_2_string="acis_process_events \
        infile="$INDIR/pha_ro_tests/acisf00732_000N002_evt1.fits" \
        outfile=${outfile} \
        acaofffile="${INDIR}/pha_ro_tests/pcadf079650047N002_asol1.fits,${INDIR}/pha_ro_tests/pcadf079670342N002_asol1.fits,${INDIR}/pha_ro_tests/pcadf079706250N002_asol1.fits" \
        mtlfile="NONE" \
        doevtgrade=no \
        calculate_pi=no \
        stop="sky" \
        pix_adj="NONE" mode=h > /dev/null"
        echo $test_pha_ro_2_string | tee -a $LOGFILE
        eval $test_pha_ro_2_string | tee -a $LOGFILE 2>&1
        ;;

   test_pha_ro_3 ) test_pha_ro_3_string="acis_process_events \
        infile="$INDIR/pha_ro_tests/acisf00732_000N002_evt1.fits" \
        outfile=${outfile} \
        acaofffile="${INDIR}/pha_ro_tests/pcadf079670342N002_asol1.fits,${INDIR}/pha_ro_tests/pcadf079650047N002_asol1.fits,${INDIR}/pha_ro_tests/pcadf079706250N002_asol1.fits" \
        mtlfile="NONE" \
        doevtgrade=no \
        calculate_pi=no \
        stop="sky" \
        pix_adj="NONE" mode=h > /dev/null"
        echo $test_pha_ro_3_string | tee -a $LOGFILE
        eval $test_pha_ro_3_string | tee -a $LOGFILE 2>&1
        ;;

   test_pha_ro_4 ) test_pha_ro_4_string="acis_process_events \
        infile="$INDIR/pha_ro_tests/acisf00732_000N002_evt1.fits" \
        outfile=${outfile} \
        acaofffile="${INDIR}/pha_ro_tests/pcadf158187124N001_asol1.fits" \
        mtlfile="NONE" \
        doevtgrade=no \
        calculate_pi=no \
        stop="sky" \
        pix_adj="NONE" mode=h > /dev/null"
        echo $test_pha_ro_4_string | tee -a $LOGFILE
        eval $test_pha_ro_4_string | tee -a $LOGFILE 2>&1
        ;;

   test_cti_app_key ) test_cti_app_key_string="acis_process_events \
        infile="$INDIR/cti_app_key/acis7sD2012-01-01bkgrnd_ctiN0002.fits[#row=1:100000]" \
        gainfile="$INDIR/cti_app_key/acisD2000-01-29gain_ctiN0007.fits" \
        outfile=${outfile} \
        acaofffile="NONE" \
        stop="none" \
        doevtgrade="no" \
        apply_cti="yes" \
        apply_tgain="no" \
        eventdef=\"{s:ccd_id,s:node_id,i:expno,s:chip,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        mode=h > /dev/null"
        echo $test_cti_app_key_string | tee -a $LOGFILE
        eval $test_cti_app_key_string | tee -a $LOGFILE 2>&1
        ;;

   test_cti_app_key2 ) test_cti_app_key2_string="acis_process_events \
        infile="$INDIR/cti_app_key/acis7sD2012-01-01bkgrnd_ctiN0002.fits[#row=1:100000]" \
        gainfile="$INDIR/cti_app_key/acisD2000-01-29gain_ctiN0007.fits" \
        outfile=${outfile} \
        acaofffile="NONE" \
        stop="none" \
        doevtgrade="no" \
        apply_cti="no" \
        apply_tgain="no" \
        eventdef=\"{s:ccd_id,s:node_id,i:expno,s:chip,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        mode=h > /dev/null"
        echo $test_cti_app_key2_string | tee -a $LOGFILE
        eval $test_cti_app_key2_string | tee -a $LOGFILE 2>&1
        ;;

  #
  # Test pix_adj algorithms
  #
    # Obsid = 5794
    #acaofffile=$INDIR/5794_asol.fits.gz
    test_subpix_edser )
	      teststr="acis_process_events \
        infile=$INDIR/5794_evt.fits.gz \
        outfile=$outfile \
        acaofffile=NONE \
        apply_cti=yes \
        apply_tgain=yes \
        pix_adj=EDSER \
        subpixfile=$INDIR/acisD1999-07-22subpixN0001.fits \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
	      ;;

    # Obsid = 5794
    #acaofffile=$INDIR/5794_asol.fits.gz
    test_subpix_centroid1 )
	      teststr="acis_process_events \
        infile=$INDIR/5794_evt.fits.gz \
        outfile=$outfile \
        acaofffile=NONE \
        apply_cti=yes \
        apply_tgain=yes \
        badpixfile=NONE \
        eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,f:chip_adj,s:tdet,f:det,f:sky,s:phas,d:phas_adj,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        pix_adj=CENTROID \
        subpixfile= \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
	      ;;

	  # Obsid = 3846
	  #acaofffile=$INDIR/centroid_asol1.fits
    test_subpix_centroid2 )
        teststr="acis_process_events \
        infile=$INDIR/centroid_evt1.fits[ccd_id=2] \
        outfile=$outfile \
        acaofffile=NONE \
        apply_cti=yes \
        apply_tgain=yes \
        badpixfile=$INDIR/centroid_bpix1.fits \
        mtlfile=$INDIR/centroid_mtl1.fits \
        eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,f:chip_adj,s:tdet,f:det,f:sky,s:phas,d:phas_adj,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        pix_adj=CENTROID \
        subpixfile= \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 3846
    # acaofffile=$INDIR/centroid_asol1.fits
    test_subpix_centroid3 )
        teststr="acis_process_events \
        infile=$INDIR/centroid_evt1.fits[ccd_id=5] \
        outfile=$outfile \
        acaofffile=NONE \
        apply_cti=no \
        apply_tgain=yes \
        badpixfile=$INDIR/centroid_bpix1.fits \
        mtlfile=$INDIR/centroid_mtl1.fits \
        eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,f:chip_adj,s:tdet,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        pix_adj=CENTROID \
        subpixfile= \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 3846
    # acaofffile=$INDIR/centroid_asol1.fits
    test_subpix_centroid4 )
        teststr="acis_process_events \
        infile=$INDIR/centroid_evt1.fits[ccd_id=5] \
        outfile=$outfile \
        acaofffile=NONE \
        apply_cti=no \
        apply_tgain=yes \
        badpixfile=NONE \
        mtlfile=$INDIR/centroid_mtl1.fits \
        eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,f:chip_adj,s:tdet,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        pix_adj=CENTROID \
        subpixfile= \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 14533
    # acaofffile=$INDIR/centroid5_asol1.fits
    test_subpix_centroid5 )
        teststr="acis_process_events \
        infile=$INDIR/centroid5_evt1.fits \
        outfile=$outfile \
        acaofffile=NONE \
        apply_cti=no \
        apply_tgain=no \
        badpixfile=$INDIR/centroid5_bpix1.fits \
        mtlfile=$INDIR/centroid5_mtl1.fits \
        eventdef=\"{d:time,s:ccd_id,s:node_id,i:expno,s:chip,f:chip_adj,s:tdet,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        pix_adj=CENTROID \
        subpixfile= \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
        ;;

    # Obsid = 5794
    # acaofffile=$INDIR/5794_asol.fits.gz
    test_subpix_rand )
	      teststr="acis_process_events \
        infile=$INDIR/5794_evt.fits.gz \
        outfile=$outfile \
        acaofffile=NONE \
        apply_cti=yes \
        apply_tgain=yes \
        pix_adj=RAND \
        subpixfile=\
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
	      ;;

    # Obsid = 5794
    # acaofffile=$INDIR/5794_asol.fits.gz
    test_subpix_none )
	      teststr="acis_process_events \
        infile=$INDIR/5794_evt.fits.gz \
        outfile=$outfile \
        acaofffile=NONE \
        apply_cti=yes \
        apply_tgain=yes \
        pix_adj=NONE \
        subpixfile= \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
	      ;;

    # blank sky infile error check
    blank_sky )
	      teststr="acis_process_events \
        infile=$INDIR/${testid}/acis0iD2009-09-21bkgrnd_ctiN0003.fits \
        outfile=$outfile \
        acaofffile=NONE \
        stop=\"none\" \
        doevtgrade=no \
        apply_cti=yes \
        apply_tgain=no \
        calculate_pi=yes \
        gainfile=$INDIR/${testid}/acisD2000-01-29gain_ctiN0007.fits \
        pix_adj=NONE \
        eventdef=\"{s:ccd_id,s:node_id,i:expno,s:chip,s:tdet,f:det,f:sky,s:phas,l:pha,l:pha_ro,f:energy,l:pi,s:fltgrade,s:grade,x:status}\" \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
	      ;;

    # Check for level3/CAT processing
    lev3 )
        teststr="acis_process_events \
        infile=$INDIR/${testid}/acisf10993_000N020_tmpBevt3.fits[#row=1:10] \
        outfile=$outfile \
        acaofffile=$INDIR/${testid}/pcadf10993_000N020_asol3.fits \
        mode=h > /dev/null"
        echo $teststr | tee -a  $LOGFILE
        eval $teststr  | tee -a  $LOGFILE  2>&1
        ;;
  esac


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # compare
  # !!12

   dmdiff $outfile $savfile tol=$INDIR/tolerance > /dev/null 2>>$LOGFILE
   if  test $? -ne 0 ; then
     echo "ERROR: MISMATCH in $outfile $savfile" >> $LOGFILE
     echo "dmdiff $outfile $savfile tol=$INDIR/tolerance" >> $LOGFILE
     # echo to std out for easier copy
     echo "ERROR: MISMATCH in $outfile $savfile"
     echo "dmdiff $outfile $savfile tol=$INDIR/tolerance"
     mismatch=0
   fi

#   \rm -r $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK" | tee -a $LOGFILE
    script_succeeded=1
  else
    # No
    echo "${testid} OK" | tee -a $LOGFILE
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
