#! /bin/sh

# 10 September 2018

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# aconvolve.t
# test script for aconvolve


# !!2
# syntax:
# aconvolve.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/KERNELSP/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g;
  s/COMMENT/Dataset/g;' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="aconvolve"

# set up list of tests
# !!4
alltests="test_lib test_file test_3d test_table test_cell1DD test_free test_units"

# "short" test to run
# !!5
shortlist="test_lib test_file test_3d"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi

# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test_lib ) test1_string="aconvolve infile=$INDIR/aconvolve_in.fits outfile=$outfile kernelspec=\"lib:gaus(2,3,1,3,5)\" clob+ normker=none"
            echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    # !!7
    test_file ) test2_string="aconvolve infile=$INDIR/aconvolve_in.fits outfile=$outfile kernelspec=\"file:$INDIR/aconvolve_kernel.fits\" method=fft edges=wrap pad=no center=no clob+  normker=none"
	    echo $test2_string | tee -a  $LOGFILE
	    eval $test2_string  | tee -a  $LOGFILE  2>&1
           ;;

      test_3d) test3_string="aconvolve infile=$INDIR/3d_unit.fits outfile=$outfile kernelspec=\"lib:box(3,1,3,3,3):(1,1,1)\" clob+ normker=none method=fft edges=wrap"
	    echo $test3_string | tee -a  $LOGFILE
	    eval $test3_string  | tee -a  $LOGFILE  2>&1
           ;;

      test_table) test4_string="aconvolve infile=$INDIR/test_table.fits outfile=$outfile kernelspec=\"txt:(1)\" clob+ normker=none "
	    echo $test4_string | tee -a  $LOGFILE
	    eval $test4_string  | tee -a  $LOGFILE  2>&1
           ;;

      test_cell1DD) test5_string="aconvolve infile=$INDIR/test_cell1DD.fits outfile=$outfile kernelspec=\"lib:box(1,1,5)\" edge=renorm method=slide clob+ normker=area "
	    echo $test5_string | tee -a  $LOGFILE
	    eval $test5_string  | tee -a  $LOGFILE  2>&1
           ;;
      test_free) test6_string="aconvolve infile=$INDIR/img.fits outfile=$outfile kernelspec=\"txt:((1))\" writefft=yes meth=fft fftroot=ffts clob+ "
           echo $test6_string | tee -a  $LOGFILE
           eval $test6_string  | tee -a  $LOGFILE  2>&1
           ;;
      test_units) test7_string="aconvolve $INDIR/img $outfile \"lib:box(2,1,3,3):(1,1)\" clob+"
                  test7_dmlist_string="dmlist $outfile cols | grep -i electrons_t > /dev/null"    
	    echo "$test7_string | tee -a  $LOGFILE"
	    eval $test7_string  | tee -a  $LOGFILE  2>&1
           ;;

  esac


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13

  if test $testid != "test_table" -a $testid != "test_cell1DD"
      then
      ext=1
  else
      ext=2
  fi

  if test $testid != "test_3d"
      then
      dmdiff  "$outfile[$ext]" "$savfile[$ext]" tol=$SAVDIR/tolerance 2>&1 >> $LOGFILE
      if test $? -ne 0; then
	  echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
	  mismatch=0
      fi
  else
      for slice in 0 1 2
	do
	z=`expr 49 + $slice`
	dmcopy $outfile"[1:100,1:100,${z}:${z}]" $OUTDIR/slice${slice}.fits clobber=yes
        dmdiff  $OUTDIR/slice${slice}.fits "$SAVDIR/slice${slice}.fits" tol=$SAVDIR/tolerance 2>&1 >> $LOGFILE
	if test $? -ne 0; then
	    echo "ERROR: data MISMATCH in slice $slice" >> $LOGFILE
	    mismatch=0
	fi
	done
  fi

  if test $testid = "test_units" ; then
    ##################################################################
    # check Scale image
    echo ""

    dmlist $outfile cols | grep -i "electrons" >> /dev/null

    if test $? -ne 0
    then
      echo "ERROR: IMAGE UNITS MISMATCH in $outfile" >> $LOGFILE
      echo "ERROR: Units should be electrons" >> $LOGFILE
      mismatch=0
    fi
  fi

  ####################################################################
  # Did we get an error?
  echo ""
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded

