#! /bin/sh

# 1 August 2010

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# mytool.t
# test script for mytool


# !!2
# syntax:
# mytool.t [<testid> ... ]
 
# Developer's comments:
# lack of comparison of the higher extensions of outputs (FITS)
#
######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/WasteDataset/g;s/COMMENT/WasteDataset/g;
  s/DATE/WasteDataset/g;s/CREATOR/WasteDataset/g;s/HISTORY/WasteDataset/g; 
  s/DATASUM/WasteDataset/g;s/ASCDSVER/WasteDataset/g;s/HISTNUM/WasteDataset/g; 
  s/CVSD0001/WasteDataset/g; s/CVST0001/WasteDataset/g; s/FEFFILE/WasteDataset/g' | \
  grep -v WasteDataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable
find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}
######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}

######################################################################
# Initialization

# !!3
toolname="addresp"

# set up list of tests
# !!4

alltests="stk_s3 stk_3 fil_3 heg_ord1 grat_ord0 rmf_tlminmax xmm_rmf"


# "short" test to run
# !!5
#shortlist="stk_2"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi

# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi

# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an o/data/regression_test/development/indata/mkrmf/6128.wgtutput directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmdiff


# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    

  echo""
  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool

  # Set up file names
  savfile=$SAVDIR/rmf_${testid}.fits
  savarf=$SAVDIR/arf_${testid}.fits
  ofile=$OUTDIR/rmf_${testid}.fits  
  oarf=$OUTDIR/arf_${testid}.fits 
 
  case ${testid} in

       stk_s3)	 
	  command_str="${toolname} \
	infile=@${INDIR}/rmf_${testid}.lis type="rmf"\
	arffile=@${INDIR}/arf_${testid}.lis \
	phafile=@${INDIR}/arf_${testid}.lis \
	method="sum" outfile=$ofile 	outarf=$oarf clobber=yes "
	  ;;
      stk_3)	 
	  command_str="${toolname} \
	infile=@${INDIR}/rmf_${testid}.lis type="rmf"\
	arffile=@${INDIR}/arf_${testid}.lis \
	phafile=@${INDIR}/arf_${testid}.lis \
	method="sum" outfile=$ofile 	outarf=$oarf clobber=yes "
	  ;;

     fil_3)
	  command_str="${toolname} \
	      infile= type=rmf \
	      arffile=${INDIR}/acisf02810_001N001_r0023_arf3.fits,${INDIR}/acisf02810_001N001_r0051_arf3.fits,${INDIR}/acisf02810_001N001_r0058_arf3.fits\
	      phafile=${INDIR}/acisf02810_001N001_r0023_arf3.fits,${INDIR}/acisf02810_001N001_r0051_arf3.fits,${INDIR}/acisf02810_001N001_r0058_arf3.fits\
	      method="avg" outfile=""  outarf=$oarf  clobber=yes mode=h"
	  ;;
      
      heg_ord1)	 
	  command_str="${toolname} \
	infile=@${INDIR}/heg_rmfs_orig.at type="rsp"\
	arffile=@${INDIR}/heg_arfs_orig.at \
	phafile=@${INDIR}/heg_phas_orig.at \
	method="avg" outfile=$ofile  outarf=""\
        clobber=yes "
	  ;;

      grat_ord0)	 
	  command_str="${toolname} \
	infile=@${INDIR}/zero_order_rmfs.at type="rsp"\
	arffile=@${INDIR}/zero_order_arfs.at \
 	phafile=@${INDIR}/zero_order_phas.at \
        method="avg" outfile=$ofile  outarf=""\
        clobber=yes "
	  ;;

      rmf_tlminmax)	 
	  command_str="${toolname} \
	infile=${INDIR}/coadd_src10.rmf,${INDIR}/coadd_src11.rmf  type="rmf"\
	arffile=${INDIR}/coadd_src10.arf,${INDIR}/coadd_src11.arf \
 	phafile=${INDIR}/coadd_src10.pi,${INDIR}/coadd_src11.pi \
        method="sum" outfile=$ofile  outarf=$oarf\
        clobber=yes "
	  ;;

      xmm_rmf)	 
	  command_str="${toolname} \
	infile=${INDIR}/0603190101/mos1S001.rmf,${INDIR}/0603190201/mos1S001.rmf  type="rmf"\
	arffile=${INDIR}/0603190101/mos1S001.arf,${INDIR}/0603190201/mos1S001.arf \
 	phafile=${INDIR}/0603190101/mos1S001-obj.pi,${INDIR}/0603190201/mos1S001-obj.pi \
        method="sum" outfile=$ofile  outarf=$oarf \
        clobber=yes "
	  ;;
 esac

  echo $command_str  | tee -a $LOGFILE
  eval $command_str  | tee -a $LOGFILE  2>&1
 
  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  case ${testid} in
      heg_ord1)
	  ;;
      grat_ord0)
	  ;;
      *)
	  dmdiff $oarf $savarf tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
  esac
  
  if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $oarf" >> $LOGFILE
      mismatch=0
  fi
  
  case ${testid} in
      fil_3)
	  ;;
      *)
	  dmdiff $ofile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
	  ;;
  esac
  if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $ofile" >> $LOGFILE
      mismatch=0
  fi


  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"

exit $script_succeeded
