#! /bin/sh

# aprates.t
# test script for aprates

# syntax:
# aprates.t [<testid> ... ]

######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}


######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}


######################################################################
# Initialization

toolname="aprates"

# set up list of tests
alltests="test1 test2 test3"

# "short" test to run
shortlist="$alltests"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi

# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*


# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff


# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

script_succeeded=0

######################################################################
# Begin per-test loop

  for testid in $testlist
  do
  echo ""
  echo "running $testid"
  echo ""
    
  # delete old outputs
  rm -f $OUTDIR/*${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.par
  outfits=$OUTDIR/${testid}.fits
  outtext=$OUTDIR/${testid}.txt
  savfile=$SAVDIR/${testid}.par
  savfits=$OUTDIR/sav${testid}.fits
  savtext=$OUTDIR/sav${testid}.txt

  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in
    test1 )  test_string="aprates n=10 A_s=1 alpha=0.99 T_s=1 E_s=1 eng_s=1 flux_s=1 m=100 A_b=10 beta=0.01 T_b=1 E_b=1 eng_b=1 flux_b=1 conf=0.68 outfile=$outfile max_counts=150"
           ;;

    test2)  test_string="aprates n=10 A_s=1 alpha=0.99 T_s=1 E_s=1 eng_s=1 flux_s=1 m=10 A_b=10 beta=0.01 T_b=1 E_b=1 eng_b=1 flux_b=1 conf=0.68 outfile=$outfile max_counts=150"
	  ;;

    test3)   test_string="aprates n=10 A_s=1 alpha=0.99 T_s=1 E_s=1 eng_s=1 flux_s=1 m=10 A_b=10 beta=0.01 T_b=1 E_b=1 eng_b=1 flux_b=1 conf=0.68 outfile=$outfile max_counts=15"
	  ;;

  esac

  echo $test_string | tee -a $LOGFILE

  eval $test_string | tee -a $LOGFILE  2>&1

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  ####################################################################
  # check confidence intervals

#  pars2tab $outfile $outfits > /dev/null 2>&1 
#  pars2tab $savfile $savfits > /dev/null 2>&1 
  

  echo 1 > $outtext
  dmcopy $outtext"[1][mydata]" $outfits"[mydata]"
  dmreadpar $outfile $outfits"[mydata]" clob+

  echo 1 > $savtext
  dmcopy $savtext"[1][mydata]" $savfits"[mydata]"
  dmreadpar $savfile $savfits"[mydata]" clob+


  dmdiff $outfits $savfits tol=$SAVDIR/tolerance \
	> /dev/null 2>>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
    mismatch=0
  fi


  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
