#!/usr/bin/env perl
# 
#  Copyright (C) 2004-2008,2010-2011,2017,2019,2021-2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '001';
my $semantic = 'dmlist';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);


# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmlist"; # PROGRAMMER - put tool name here

my @testlist =
(
# PROGRAMMER - List the test ids here
 "1",   # <description>
 "2", 
 "3", 
 "4",
 "5",
 "6",
 "7",
 "8",
 "9",
 "10",
 "11",
 "12",
 "13",
 "14",
 "15",
 "16",
 "17",
 "18",
 "19",
 "20",
 "21",
 "22",
 "23",
 "24",
 "25",
 "26",
 "27",
 "28",
 "29",
 "30",
 "31",
 "32",
 "33",
 "34"
);

my @test_strings = ( 
"' opt=full,all rows=10:200 cells=1:8 verbose=1 > $testout/dt${id}m001.lis_tmp 2>> $logfname",
"' opt=full cells=1:8 verbose=1 >  $testout/dt${id}m002.lis_tmp 2>> $logfname",
"' opt=full cells=1:8 verbose=1 >  $testout/dt${id}m003.lis_tmp 2>> $logfname",
"[stdevt][pha=20:30]' opt=blocks,cols rows=\"\" cells=1:8 verbose=1 >  $testout/dt${id}m004.lis_tmp 2>> $logfname",
"[stdevt][bin x=32, y=32]' opt=cols rows=\"\" cells=1:8 verbose=1 >  $testout/dt${id}m005.lis_tmp 2>> $logfname",
"' opt=full,all > $testout/dt${id}m006.lis_tmp 2>> $logfname",
"' opt=data rows=\"1:3\" cells=1:8 verbose=1 >  $testout/dt${id}m007.lis_tmp 2>> $logfname",
"[status=10][cols x,y,status]' opt=data,clean rows=1:10 cells=1:20 verbose=0 >  $testout/dt${id}m008.lis_tmp 2>> $logfname",
"' opt=data cells=1:8 verbose=1 >  $testout/dt${id}m009.lis_tmp 2>> $logfname",
"[sky=ellipse(4000,5000,200,600,70)-box(4000,5000,200,600,70)+circle(4100,5000,50)][cols time,sky]' opt=data,raw cells=1:8 verbose=0 >  $testout/dt${id}m010.lis_tmp 2>> $logfname",
"' opt=data cells=1:1000 verbose=0 >  $testout/dt${id}m011.lis_tmp 2>> $logfname",
"' opt=subspace verbose=0 >  $testout/dt${id}m012.lis_tmp 2>> $logfname",
"' opt=subspace verbose=0 >  $testout/dt${id}m013.lis_tmp 2>> $logfname",
# Test Bug #6561
"[cols CAL_CBD]' opt=data,array,clean outfile=$testout/dt${id}m014.lis_tmp >> $logfname 2>&1",
# Test Bug #8722
"' opt=subspace outfile=$testout/dt${id}m015.lis_tmp >> $logfname 2>&1",
# Test Bug #8230
"' opt=array outfile=$testout/dt${id}m016.lis_tmp >> $logfname 2>&1",
# Test Bug #9362
"[bin channel=1:2:2001]' opt=bins > $testout/dt${id}m017.lis_tmp 2>&1",
# Test Bug #8120 - filter 2 rows which are very far apart -- test 14 from dmlist.t
"[#row=5000,900000]' opt=data verbose=0 > $testout/dt${id}m018.lis_tmp 2>&1",
# Test Bug #9385 - list FITS file with ascii data table
"' opt=data verbose=0 > $testout/dt${id}m019.lis_tmp 2>&1",
# Test WCS name pair matching - FITS Standards compliance Sept 2010
"' opt=cols verbose=0 > $testout/dt${id}m020.lis_tmp 2>&1",
# Test BSCALE/BZERO values being applied, bug 12880
"' cells=1:5 opt=block,data > $testout/dt${id}m021.lis_tmp 2>&1",
# Test SL-57 - interpretation of complex subspace region including Mask shapes
"' opt=subspace outfile=$testout/dt${id}m022.lis_tmp >> $logfname 2>&1",
# Test SL-180 - ensure inf & -inf display as such and not nan for fits and dtf file
"' opt=data outfile=$testout/dt${id}m023.lis_tmp >> $logfname 2>&1 ",
"' opt=data outfile=$testout/dt${id}m024.lis_tmp >> $logfname 2>&1 ",
# Start SL-35 Test Plan - 1.1 straight dmlist
"[1D_I2_BASIC]' opt=full outfile=$testout/dt${id}m025.lis_tmp >> $logfname 2>&1 ",
"[2D_I2_BASIC]' opt=full,array outfile=$testout/dt${id}m026.lis_tmp >> $logfname 2>&1 ", 
"[3D_I2_BASIC]' opt=full,array outfile=$testout/dt${id}m027.lis_tmp >> $logfname 2>&1 ", 
"[4D_I2_BASIC]' opt=full,array outfile=$testout/dt${id}m028.lis_tmp >> $logfname 2>&1 ", 
"[1D_I2_SCALED]' opt=full outfile=$testout/dt${id}m029.lis_tmp >> $logfname 2>&1 ",
"[2D_I2_SCALED]' opt=full,array outfile=$testout/dt${id}m030.lis_tmp >> $logfname 2>&1 ", 
"[3D_I2_SCALED]' opt=full,array outfile=$testout/dt${id}m031.lis_tmp >> $logfname 2>&1 ", 
"[4D_I2_SCALED]' opt=full,array outfile=$testout/dt${id}m032.lis_tmp >> $logfname 2>&1 ", 
# End SL-35 Tests
# Start SL-105 Tests  - Print Long Bits data
"[cols manybits]' opt=data > $testout/dt${id}m033.lis_tmp 2>> $logfname",
"[cols manybits]' opt=data cell=1:1024 > $testout/dt${id}m034.lis_tmp 2>> $logfname",
# End SL-105 Tests    
);

# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg;
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf"); 
my %links = ("4" => "indt001m001",
             "5" => "indt001m001",
             "17" => "indt001m012");
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
     # Init per-test variables.
     my $mismatch = 0;
     my $time_user;
     my $time_sys;
     $err_msg = "";
     my $skipDelete = 0;
     my $linkname = "";

     # change extension based on kernel
     $in_ext = $exten{$kernelName};

     # delete old outputs
     my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
     system("rm -f $testout/${basename}*");
   
     # set up file names
     my $tmpoutfile = "$testout/${basename}.lis_tmp";

     my $infile = "$testin/in${basename}.$in_ext";

     if ($testid == 6)
     {
       $infile = "$testin/in${basename}.simple";
     }
     
     if ($testid == 24 || $testid == 33 || $testid == 34 )
     {
       $infile = "$testin/in${basename}.dtf";
     }

     # if testing any kernel other than FITS, the FITS input files need to
     # be copied into the present kernel's format
     if ( $kernelName ne "FITS" )
     {
       my $sourceFile = "$testin/in${basename}.$source_ext";
       my $copyFile   = "$testout/in${basename}.$in_ext";
       my $options    = "opt=all";
       my $kernelOpts = $copyOpts{"$kernelName"};

       if ($testid == 12 || $testid == 17)
       {
         $options = "";
       }

       if ( ! -e $infile )
       {
         my $linkbase = "";
         my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

         # does a previous copy file exist?
         if ($testid == 4 || $testid == 5 || $testid == 17 )
         {
           $linkbase = $links{"$testid"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }

         # if not, make one
         if ( ! -e $copyFile )
         {
           if ( $linkname ne "" )
           {
             if ( -e $linkname )
             {
               # create link
               $teststring = "ln -s $linkname $copyFile";
             }
           }

           # copy fits file to text/dtf file or create link
           ($rc, $time_user, $time_sys) = system($teststring);
         }

         $infile = $copyFile;

         # Tests 1, 4 and 5 use the same input file.
         # Tests 12 and 17 use the same input file
         # Don't delete generated input file
         if ( $testid == 1 || $testid == 12 )
         {
           $skipDelete = 1;
         }
       }
       else
       {
         $skipDelete = 1;
       }
     }

     my $outfile = "$testout/${basename}.lis";
     my $savfile = "$testsav/${basename}.lis";
     my $cmpfile = "$testout/${basename}.lis_std";

     # --------------------------------------------------------------------------------
     # set test command
     my $test_string = "$DMLIST infile='$infile";
     $test_string .= $test_strings[$testid-1];

     # --------------------------------------------------------------------------------
     # run the test
     dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
     close $logfile;
     
     ( $rc, $time_user, $time_sys ) = dt_run_test( $test_string );

     open $logfile, ">>$logfname";
     select($logfile); $| = 1; select($printfile);
     unless ( $rc == 0 )
     {
       $err_msg = "ERROR: $toolname failed to run\n";
       push( @faillist, $testid );
       $mismatch = 1;
     }
     
     # --------------------------------------------------------------------------------
     # validate results.
     keyfilter( $tmpoutfile, $outfile, \@tolkeys );
     keyfilter( $savfile, $cmpfile, \@tolkeys );
   
     $rc = system( "diff $outfile $cmpfile > /dev/null 2>> $logfname" );

     # --------------------------------------------------------------------------------
     # report results.
     unless ( $rc == 0 || $mismatch )
     {
       # Put the kernel name in front of the list of failed tests
       if ( $errFlag == 1 )
       {
         push (@faillist, $kernelName);
         $errFlag = 0;
       }

       $err_msg = "MISMATCH in $outfile";
       push( @faillist, $testid );
       $mismatch = 1;
     }
     dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
     $time_user_total += $time_user;
     $time_sys_total  += $time_sys;

     # --------------------------------------------------------------------------------
     # delete any input files created just for this kernel
     # exceptions are made for files too large to recreate
     # and one text simple file (6)
     if ( $kernelName ne "FITS" )
     {
       if ($skipDelete == 0 )
       {
	 system("$RM -f $infile");
       }
       if ($testid == 5 || $testid == 17 )
       {
	   system("$RM -f $linkname");
       }
     }
  }
}


if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
