#!/usr/bin/env perl
# 
#  Copyright (C) 2004-2005,2008,2021,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '013';
my $semantic = 'filtrebin';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16,
17,
18,
19,
20,
21,
22,
23,
24,
25,
26,
27,
28,
);

my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
1 => { "image" => [1] },
2 => { "image" => [1] },
3 => { "image" => [1] },
4 => { "image" => [1] },
5 => { "image" => [1] },
6 => { "image" => [1] },
7 => { "image" => [1] },
8 => { "image" => [1] },
9 => { "image" => [1] },
10 => { "image" => [1] },
11 => { "image" => [1] },
12 => { "image" => [1] },
13 => { "image" => [1] },
14 => { "image" => [1] },
15 => { "image" => [1] },
16 => { "list"  => [ ] },
17 => { "list"  => [ ] },
18 => { "list " => [ ] },
19 => { "list"  => [ ] },
20 => { "list"  => [ ] },
21 => { "list"  => [ ] },
22 => { "list"  => [ ] },
23 => { "list"  => [ ] },
24 => { "list"  => [ ] },
25 => { "list"  => [ ] },
26 => { "list"  => [ ] },
27 => { "list"  => [ ] },
28 => { "list"  => [ ] },
);

my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
$version => "ASCDS_VERSION" 
);


# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf");
my %links = ("1" => "indt013m001");
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#######################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # if testing any kernel other than FITS, the FITS input files need to
    # be copied into the present kernel's format
    if ( $kernelName ne "FITS" )
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testout/in${basename}.$in_ext";
      my $options    = "opt=all";
      my $kernelOpts = $copyOpts{"$kernelName"};

      if ( ! -e "$infile" )
      {
         if ($testid == 11)
         {
           $options = "";
         }

         my $linkbase = "";
         my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

         # does a previous copy file exist?
         if ($testid >= 2 && $testid <= 8 )
         {
           $linkbase = $links{"1"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }

         # if not, make one
         if ( ! -e $copyFile )
         {
           if ( $linkname ne "" )
           {
             if ( -e $linkname )
             {
               # create link
               $teststring = "ln -s $linkname $copyFile";
             }
           }

          # copy fits file to text/dtf file
          ($rc, $time_user, $time_sys) = system($teststring);
        }
        $infile = $copyFile;

         # Tests 1 through 8 use the same input file.
         # Don't delete generated input file
         if ( $testid == 1 )
         {
           $skipDelete = 1;
         }
      }
      else
      {
        $skipDelete = 1;
      }
    }

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.fits";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.fits";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string = "";
   
    if ( $testid == 1 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[x=80.5:8000.5,y=2864.5:7200.5\]\[bin x=16,y=16\]\" outfile=$outfile";
    }
    elsif ( $testid == 2 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[x=80.5:8000.5,y=2864.5:7200.5\]\[bin x=16,y=16\]\[opt full,null=NaN\]\" outfile=$outfile";
    }
    elsif ( $testid == 3 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[#1=11:1000,#2=359:900\]\[bin #1=2,#2=2\]\" outfile=$outfile";
    }
    elsif ( $testid == 4 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[#1=11:1000,#2=359:900\]\[bin #1=2,#2=2\]\[opt full,null=NaN\]\" outfile=$outfile";
    }
    elsif ( $testid == 5 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[sky=circle(5000,5000,2000)\]\[bin sky=32\]\[opt null=NaN\]\" outfile=$outfile";
    }
    elsif ( $testid == 6 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[sky=circle(5000,5000,2000)\]\[bin sky=32\]\[opt full,null=NaN\]\" outfile=$outfile";
    }
    elsif ( $testid == 7 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[#1=12:1000,#2=360:900\]\[bin #1=2,#2=2\]\" outfile=$outfile";
    }
    elsif ( $testid == 8 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[#1=12:1000,#2=360:900\]\[bin #1=2,#2=2\]\[opt full,null=NaN\]\" outfile=$outfile";
    }
    elsif ( $testid == 9 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[#1=50:450,#2=50:450\]\[bin #1=2,#2=2\]\[opt full,null=-999\]\" outfile=$outfile";
    }
    elsif ( $testid == 10 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[#1=50:450,#2=50:450\]\[bin #1=2,#2=2\]\[opt full,null=-999\]\" outfile=$outfile";
    }
    elsif ( $testid == 11 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[#1=50:450,#2=50:450\]\[bin #1=2,#2=2\]\[opt full,null=NaN\]\" outfile=$outfile";
    }
    elsif ( $testid == 12 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin #1=2,#2=2\]\[opt full,null=-999\]\" outfile=$outfile";
    }
    elsif ( $testid == 13 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[bin #1=2,#2=2\]\[opt full,null=-999\]\" outfile=$outfile";
    }
    # Bug #8651
    elsif ( $testid == 14 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[sky=box(3431.29,3862.73,685.5,424,0)\]\[bin 1\]\" outfile=$outfile";
    }
    elsif ( $testid == 15 )
    {
      # SL-35 Test Plan - 2.4.1 filter and bin - straight copy on 2D image 
      $test_string = "$DMCOPY infile=\"$infile\[2D_I2_BASIC\]\[x=150:250,y=150:250\]\[bin x=40,y=40\]\" outfile=$outfile";
    }
    elsif ( $testid == 16 )
    {
      # SL-35 Test Plan - 2.4.2 filter and bin - straight copy on 2D scaled image
      $test_string = "$DMCOPY infile=\"$infile\[2D_I2_SCALED\]\[x=150:250,y=150:250\]\[bin x=40,y=40 \]\" outfile=$outfile";
    }
    elsif ( $testid == 17 )
    {
      # SL-35 Test Plan - 2.5.1 filter and bin - copy with 'opt full' on 2D image
      $test_string = "$DMCOPY infile=\"$infile\[2D_I2_BASIC\]\[x=150:250,y=150:250\]\[bin x=40,y=40\]\[opt full\]\" outfile=$outfile";
    }
    elsif ( $testid == 18 )
    {
      # SL-35 Test Plan - 2.5.2 filter and bin - copy with 'opt full' on 2D scaled image
      $test_string = "$DMCOPY infile=\"$infile\[2D_I2_SCALED\]\[x=150:250,y=150:250\]\[bin x=40,y=40\]\[opt full\]\" outfile=$outfile";
    }
    elsif ( $testid == 19 )
    {
      # SL-35 Test Plan - 3.3.1 slice - [z=a][bin x,y] on 3D image
      $test_string = "$DMCOPY infile=\"$infile\[3D_I2_BASIC\]\[#3=3\]\[bin x,y\]\" outfile=$outfile";
    }
    elsif ( $testid == 20 )
    {
      # SL-35 Test Plan - 3.3.2 slice - [z=a][bin x,y] on 3D scaled image
      $test_string = "$DMCOPY infile=\"$infile\[3D_I2_SCALED\]\[#3=3\]\[bin x,y\]\" outfile=$outfile";
    }
    elsif ( $testid == 21 )
    {
      # SL-35 Test Plan - 3.4.1 thick slice - [z=a:b][bin x,y] on 3D image
      $test_string = "$DMCOPY infile=\"$infile\[3D_I2_BASIC\]\[#3=1:2\]\[bin x,y\]\" outfile=$outfile";
    }
    elsif ( $testid == 22 )
    {
      # SL-35 Test Plan - 3.4.2 thick slice - [z=a:b][bin x,y] on 3D scaled image
      $test_string = "$DMCOPY infile=\"$infile\[3D_I2_SCALED\]\[#3=1:2\]\[bin x,y\]\" outfile=$outfile";
    }
    elsif ( $testid == 23 )
    {
      # SL-35 Test Plan - 3.5.1 alternate axes - [y=<a:b>][bin x,z] on 3D image
      $test_string = "$DMCOPY infile=\"$infile\[3D_I2_BASIC\]\[y=150:250\]\[bin x,z\]\" outfile=$outfile";
    }
    elsif ( $testid == 24 )
    {
      # SL-35 Test Plan - 3.5.2 alternate axes - [y=<a:b>][bin x,z] on 3D scaled image
      $test_string = "$DMCOPY infile=\"$infile\[3D_I2_SCALED\]\[y=150:250\]\[bin x,z\]\" outfile=$outfile";
    }
    elsif ( $testid == 25 )
    {
      # SL-35 Test Plan - 3.6.1 opt=full on 3D image
      $test_string = "$DMCOPY infile=\"$infile\[3D_I2_BASIC\]\[x=150:250,y=150:250,z=600\]\[bin x,y,z\]\[opt full\]\" outfile=$outfile";
    }
    elsif ( $testid == 26 )
    {
      # SL-35 Test Plan - 3.6.2 opt=full on 3D scaled image
      $test_string = "$DMCOPY infile=\"$infile\[3D_I2_SCALED\]\[x=150:250,y=150:250,z=600\]\[bin x,y,z\]\[opt full\]\" outfile=$outfile";
    }
    elsif ( $testid == 27 )
    {
      # SL-35 Test Plan - 4.2.1 filter and bin - [4=a][bin x,y,z] (4D->3D) on 4D image
      $test_string = "$DMCOPY infile=\"$infile\[4D_I2_BASIC\]\[e=2\]\[bin x,y,z\]\" outfile=$outfile";
    }
    elsif ( $testid == 28 )
    {
      # SL-35 Test Plan - 4.2.2 filter and bin - [4=a][bin x,y,z] (4D->3D) on 4D scaled image
      $test_string = "$DMCOPY infile=\"$infile\[4D_I2_SCALED\]\[e=2\]\[bin x,y,z\]\" outfile=$outfile";
    }
    $test_string .= " >> $logfname 2>&1";

    # --------------------------------------------------------------------------------
    # run the test
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);
    
    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
	$err_msg = "ERROR: $toolname failed to run\n";
    }
   
    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "${outfile}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt", $savfile, "${outfile}_savfilt", \%test_words, $logfname, $test_diff );
    }
    if ( $block_types{"dtf"} )
    {
      $err_msg .=  dtf_test( $outfile, "${outfile}_filt", $savfile, "${outfile}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;

    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
      if ( $testid == 8 )
      {
        system("$RM -f $linkname");
      }                
    }
  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
