#!/usr/bin/env perl
# 
#  Copyright (C) 2004-2005,2007-2009,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '014';
my $semantic = 'rowfilt';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $caldb   = %infra{'caldb'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "rowfilter"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11
);

my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
1 => { "text" => [1] },
2 => { "text" => [1] },
3 => { "text" => [1] },
4 => { "text" => [1] },
5 => { "text" => [1] },
6 => { "text" => [1] },
7 => { "text" => [1] },
8 => { "text" => [1] },
9 => { "text" => [1] },
10 => { "table" => [2] },
11 => { "text" => [1] },
);

my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
$version => "ASCDS_VERSION",
  "nan"  => "NAN",
  "NaN"  => "NAN"   
);

		     
# Find Tool Executable in PATH
my $toolpath = find_inpath( "rowfilter", ".:${TEXEC_DIR}", ':' );
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
#system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf");
my %links = ("1" => "indt014m001",
             "10" => "indt014m010");
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # if testing any kernel other than FITS, the FITS input files need to
    # be copied into the present kernel's format
    if ( $kernelName ne "FITS" )
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testout/in${basename}.$in_ext";
      my $options    = "opt=all";
      my $kernelOpts = $copyOpts{"$kernelName"};

      if ( ! -e "$infile" )
      {
         my $linkbase = "";
         my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

         # does a previous copy file exist?
         if ($testid >= 2 && $testid <= 9 )
         {
           $linkbase = $links{"1"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }
         elsif ($testid == 11)
         {
           $linkbase = $links{"10"};
           $linkname = "$testout/${linkbase}.$in_ext";
         }

         # if not, make one
         if ( ! -e $copyFile )
         {
           if ( $linkname ne "" )
           {
             if ( -e $linkname )
             {
               # create link
               $teststring = "ln -s $linkname $copyFile";
             }
           }

          # copy fits file to text/dtf file
          ($rc, $time_user, $time_sys) = system($teststring);
         }

         $infile = $copyFile;

         # Tests 1 to 9 use the same input file.
         # Tests 10 and 11 use the same input file
         # Don't delete generated input file
         if ( $testid == 1 || $testid == 10 )
         {
           $skipDelete = 1;
         }
      }
      else
      {
        $skipDelete = 1;
      }
    }

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.txt";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.txt";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string = "";
    my $ROWFILTER = $toolpath;
    #
    # Tests 1-9:
    #    Enh. 8645: provide API routine to enable application of external filter (dmTableAllocRowFilter)
    #               
    if ( $testid == 1 )
    {
       $test_string = "$ROWFILTER $infile '[time=159532210.3380066156:159533620.6523556411,159534374.4952971637:159534905.3263894022,159536488.5198356509:159537905.1166149676]' > $outfile";
    }
    elsif ( $testid == 2 )
    {
       $test_string = "$ROWFILTER $infile \"\[pha=839\]\" > $outfile";
    }
    elsif ( $testid == 3 )
    {
       $test_string = "$ROWFILTER $infile \"\[status=xxx0000010000000000000000000xxx0\]\" > $outfile";
    }
    elsif ( $testid == 4 )
    {
       $test_string = "$ROWFILTER $infile \"\[y=2885.371093750:2887.0849609375\]\" > $outfile";
    }
    elsif ( $testid == 5 )
    {
       $test_string = "$ROWFILTER $infile \"\[time=159532210.3380066156:159533620.6523556411,159534374.4952971637:159534905.3263894022,159536488.5198356509:159537905.1166149676,y=2900:3000\]\" > $outfile";
    }
    # test with only certain cols selected
    elsif ( $testid == 6 )
    {
       $test_string = "$ROWFILTER $infile\"\[cols time,y\]\" \"\[time=159532210.3380066156:159533620.6523556411,159534374.4952971637:159534905.3263894022,159536488.5198356509:159537905.1166149676,y=2900:3000\]\" > $outfile";
    }
    elsif ( $testid == 7 )
    {
       $test_string = "$ROWFILTER $infile\"\[cols sky,time\]\" \"\[time=159532210.3380066156:159533620.6523556411,159534374.4952971637:159534905.3263894022,159536488.5198356509:159537905.1166149676,y=2900:3000\]\" > $outfile";
    }
    elsif ( $testid == 8 )
    {
        $test_string = "$ROWFILTER $infile\"\[cols sky,time\]\" \"\[time=159532210.3380066156:159533620.6523556411,159534374.4952971637:159534905.3263894022,159536488.5198356509:159537905.1166149676,sky=circle(4300,2900,100)\]\" > $outfile";
    }
    elsif ( $testid == 9 )
    {
        $test_string = "$ROWFILTER $infile \"\[time=159532210.3380066156:159533620.6523556411,159534374.4952971637:159534905.3263894022,159536488.5198356509:159537905.1166149676,sky=circle(4300,2900,100)\]\" > $outfile";
    }
    elsif ( $testid == 10 )
    {
      # Test 10:
      #    Bug 9301: Problem filtering vector when components are not in same order as in file
      $outfile     = "$testout/${basename}.fits";
      $savfile     = "$testsav/${basename}.fits.gz";
      my $regfile  = "$caldb/data/chandra/hrc/tgpimask2/letgD1999-07-22pireg075_N0001.fits";
      $test_string = "$DMCOPY infile=${infile}'[EVENTS][cols time,pha,pi,tg_lam][pha=0:254,(tg_lam,pi)=region($regfile)]' outfile=$outfile opt=all";
    }
    elsif ( $testid == 11 )
    {
      # Test 11:
      #    Bug 10035: Same as above, but using dmTableRowFilter()
      my $regfile = "$caldb/data/chandra/hrc/tgpimask2/letgD1999-07-22pireg075_N0001.fits";
      $test_string = "$ROWFILTER ${infile}'[EVENTS][cols time,pha,pi,tg_lam]' '[pha=0:254,(tg_lam,pi)=region($regfile)]' > $outfile";
    }
    $test_string .= " 2>> $logfname";

    # --------------------------------------------------------------------------------
    # run the test
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);
    
    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
       $err_msg = "ERROR: $toolname failed to run\n";
    }

    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"table"} )
    {
       my @blocks = @{$block_types{"table"}};
       foreach my $block ( @blocks )
       {
         $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
       }
    }
    if ( $block_types{"image"} )
    {
       my @blocks = @{$block_types{"image"}};
       foreach my $block ( @blocks )
       {
         $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
       }
    }
    if ( $block_types{"list"} )
    {
       $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
    }
    if ( $block_types{"text"} )
    {
       $err_msg .= diff_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \%test_words, $logfname, $test_diff );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;
    
    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
      if ( $testid == 9 || $testid == 11 )
      {
        system("$RM -f $linkname");
      }
    }
  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
