#!/usr/bin/env perl
# 
#  Copyright (C) 2004-2005,2007-2008,2015,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '037';
my $semantic = 'RegionBlocks';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmmakereg"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
  "1",
  "2",
  "3",
  "4",
  "5",
  "6",
  "7",
  "8",
  "9",
  "10",
  "11"
);

my %test_blocks = ( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 "1" => { "list" => [ ] },
 "2" => { "list" => [ ] },
 "3" => { "text" => [ ] },
 "4" => { "list" => [ ] },
 "5" => { "list" => [ ] },
 "6" => { "text" => [ ] },
 "7" => { "text" => [ ] },
 "8" => { "list" => [ ] },
 "9" => { "list" => [ ] },
 "10" => { "list_header" => [ ],
           "table" => [ 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 ] },
 "11" => { "list_header" => [ ],
           "table" => [ 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 ] }
);

my %test_words = (
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
 $testin => "TESTIN",
 $version => "ASCDS_VERSION",
 'CIAO\d(\.\d+)*' => "ASCDS_VERSION",
 'DS\d(\.\d+)*' => "ASCDS_VERSION"
);


# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}
my $DMMAKEREG = $toolpath;

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf");
my %links = ("5" => "indt037m005",
             "10" => "indt037m010"); 
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $inbase = sprintf "$testin/indt%.3d", $id;
    my $infile = "$testin/in${basename}.$in_ext";

    # if testing any kernel other than FITS, the FITS input files need
    # to be copied into the present kernel's format
    # Test 6, 7, and 8 are already text files, no need to copy

    if ( $testid == 6 || $testid == 7 || $testid == 8 )
    {
       $infile = "$testin/in${basename}.fits";
       $skipDelete = 1;
    }
    elsif ( $kernelName ne "FITS")
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testout/in${basename}.$in_ext";
      my $options    = "opt=all";
      my $kernelOpts = $copyOpts{"$kernelName"};

      if ( ! -e $infile )
      {
        my $linkbase = "";
   
     my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

        # does a previous copy file exist?
        if ($testid == 9 )
        {
          $linkbase = $links{"5"};
          $linkname = "$testout/${linkbase}.$in_ext";
        }
        elsif ($testid == 10 || $testid == 11 )
        {
          $linkbase = $links{"10"};
          $linkname = "$testout/${linkbase}.$in_ext";
        }

        # if not, make one
        if ( ! -e $copyFile )
        {
          if ( $linkname ne "" )
          {
            if ( -e $linkname )
            {
              # create link
              $teststring = "ln -s $linkname $copyFile";
            }
          }

          # copy fits file to text/dtf file or create link
          ($rc, $time_user, $time_sys) = system($teststring);
        }

        $infile = $copyFile;

        # Tests 5, 6, 7 and 8 use the same input file.
        # Tests 10 and 11 use the same input file
        # Don't delete generated input file
        if ( $testid == 5 || $testid == 10 )
        {
          $skipDelete = 1;
        }
      }
      else
      {
        $skipDelete = 1;
      }
    }


    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.fits";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.fits.gz";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string;

    if ( $testid == 1 )
    {
      # FITS-FITS: Region block with POS column, 1 shape in physical coords
      #  basically dmRegParse test, interpreting input file to 'region' object
      $test_string = "$DMMAKEREG region=\"region($infile)\" outfile=$outfile";
    }
    elsif ( $testid == 2 )
    {
      # FITS-FITS: Region block with X,Y columns; 8 Circle shapes in physical coords
      #  basically dmRegParse test, interpreting input file to 'region' object
      #  DM adds POS vector definition to block
      $test_string = "$DMMAKEREG region=\"region($infile)\" outfile=$outfile";
    }
    elsif ( $testid == 3 )
    {
      # FITS-FITS: Region block with X,Z columns; 8 Circle shapes in physical coords
      #  basically dmRegParse test, interpreting input file to 'region' object
      # Expected FAILURE test.. invalid region block content.
      $outfile = "$testout/${basename}.txt";
      $savfile = "$testsav/${basename}.txt";
      # Test is supposed to raise an error; clear exit status with 'true'
      $test_string = "( $DMMAKEREG region=\"region($infile)\" outfile=$testout/${basename}.fits 2> $outfile ; true )";
    }
    elsif ( $testid == 4 )
    {
      # dmcopy: file with region subspace on 'sky' as header subspace key string.
      #  Region subspace handling test - should preserve subspace header string.
      $test_string = "$DMCOPY $infile outfile=$outfile";
    }
    elsif ( $testid == 5 )
    {
      #dmcopy: file with region subspace on 'sky' as REGION block
      # Region subspace handling test - should preserve subspace region block
      $test_string = "$DMCOPY $infile outfile=$outfile";
    }
    elsif ( $testid == 6 )
    {
      #dmcopy: file with region subspace on 'sky' as REGION block TO ASCII RAW FORMAT
      # Region subspace output to ASCII test - raw format has no subspace/header info.. so content is lost
      $outfile = "$testout/${basename}.raw";
      $savfile = "$testsav/${basename}.raw";
      $test_string = "$DMCOPY $infile outfile=$outfile'[opt kernel=text/raw]'";
    }
    elsif ( $testid == 7 )
    {
      #dmcopy: file with region subspace on 'sky' as REGION block TO ASCII SIMPLE FORMAT
      # Region subspace output to ASCII test - simple format has no subspace/header info.. so content is lost
      $outfile = "$testout/${basename}.simple";
      $savfile = "$testsav/${basename}.simple";
      $test_string = "$DMCOPY $infile outfile=$outfile'[opt kernel=text/simple]'";
    }
    elsif ( $testid == 8 )
    {
      #dmcopy: file with region subspace on 'sky' as REGION block TO ASCII DTF FORMAT
      # Region subspace output to ASCII test - outfile preserves subspace region block.
      $outfile = "$testout/${basename}.dtf";
      $savfile = "$testsav/${basename}.dtf";
      $test_string = "$DMCOPY $infile outfile=$outfile'[opt kernel=text/dtf]'";
    }
    elsif ( $testid == 9 )
    {
      #dmcopy: file with region subspace on 'sky' as REGION blocks, with additional, overlapping, sky region filter.
      # New + Existing Region subspace interaction test.
      # secondary filter region is added to REGION block
      $test_string = "$DMCOPY $infile\"[sky=circle(4062,4240,80.0)]\" outfile=$outfile";
    }
    elsif ( $testid == 10 )
    {
      #dmcopy: file with multiple subspace REGION blocks (multiple ss components) plus an EXTRA_REGION
      # Region subspace handling - multiple region blocks
      # straight dmcopy preserves subspace blocks, drops 'extra' region block.
      $test_string = "$DMCOPY $infile outfile=$outfile";
    }
    elsif ( $testid == 11 )
    {
      #dmcopy: same as #10, but with opt=all
      # Region subspace handling - multiple region blocks
      # retains secondary blocks, so the 'extra' region block is included in copy.
      $test_string = "$DMCOPY $infile  outfile=$outfile opt=all";
    }
    $test_string .= " >> $logfname 2>&1";

    # --------------------------------------------------------------------------------
    # run the test
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);
    
    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
      $err_msg = "ERROR: $toolname failed to run\n";
    }

    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
	  $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, "${test_diff}_table_${block}" );
      }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, "${test_diff}_image_${block}" );
      }
    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt_list", $savfile, "$testout/${basename}_savfilt_list", \@tolkeys, $logfname, "${test_diff}_list", "" );
    }
    if ( $block_types{"list_header"} )
    {
      $err_msg .= dmlist_header_test( $outfile, "${outfile}_filt_list_header", $savfile, "$testout/${basename}_savfilt_list_header", \@tolkeys, $logfname, "${test_diff}_list_header" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt_text", $savfile, "$testout/${basename}_savfilt_text", \%test_words, $logfname, "${test_diff}_text" );
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;
    
    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ($skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
      if ($testid == 9 || $testid == 11 )
      {
          system("$RM -f $linkname");
      }
    }
  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
