#!/usr/bin/env perl
# 
#  Copyright (C) 2008,2016-2017,2019,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '045';
my $semantic = 'asciiStrings';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16,
);

my %test_blocks = ( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 1 => { "text" => [] },
 2 => { "text" => [] },
 3 => { "text" => [] },
 4 => { "text" => [] },
 5 => { "text" => [] },
 6 => { "text" => [] },
 7 => { "text" => [] },
 8 => { "text" => [] },
 9 => { "text" => [] },
10 => { "text" => [] },
11 => { "text" => [] },
12 => { "text" => [] },
13 => { "text" => [] },
14 => { "text" => [] },
15 => { "text" => [] },
16 => { "text" => [] },
);

my %test_words = (
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
$version => "ASCDS_VERSION",
'\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d' => "TEST_DATE",
'HISTORY.*\n' => "",
'HISTNUM.*' => "HISTNUM",
 'NaN' => "NAN",
 'nan\s' => "NAN ",
$testin  => "<TESTIN>"
);

# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

#########################################################################
# Loop over all tests
foreach my $testid (@testlist)
{
   # Init per-test variables.
   my $mismatch = 0;
   my $time_user;
   my $time_sys;
   $err_msg = "";

   # delete old outputs
   my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
   system("$RM -f $testout/${basename}*");

   my $infile = "$testin/in${basename}.txt";
   # PROGRAMMER - put output filename here
   my $outfile = "$testout/${basename}.txt";
   # PROGRAMMER - put output filter filename here
   my $outfile_filt = "$testout/${basename}.txt_filt";
   # PROGRAMMER - put saved output filename here
   my $savfile = "$testsav/${basename}.txt";
   # PROGRAMMER - put name of diff file here, if needed
   my $test_diff = "$testout/${basename}.diff";
   # PROGRAMMER - put saved output filter filename here
   my $savfile_filt = "$testout/${basename}.txt_savfilt";

   # --------------------------------------------------------------------------------
   # set test command
   my $test_string = "";

   # Test Descriptions:
   #  These tests work the string handling logic of the ascii kernel.
   #  T1/T2:
   #    DTF-FIXED format input with TBCOL/TDISP keys defining fields.
   #    values are single/multi-word, quoted/unquoted, with and without
   #    significant whitespace.
   #
   #  T3/T4:
   #    DTF format input with TFORM keys defining fields and ',' delimited.
   #    values are single/multi-word, quoted/unquoted, with and without
   #    significant whitespace.
   #
   #  T5:
   #    Tests how the DM interprets string column data.. each instance has
   #    various significant/insignificant quotes to handle.
   #
   if ( $testid == 1 )
   {
      # DTF-FIXED -> DTF;  sep == ','; with TDISP keys
      $test_string = "$DMCOPY infile=$infile outfile=$outfile'[opt kernel=text/dtf,sep=\",\"]'";
   }
   elsif ( $testid == 2 )
   {
      # DTF-FIXED -> DTF;  sep == ' '; with TDISP keys
      $test_string = "$DMCOPY infile=$infile outfile=$outfile'[opt kernel=text/dtf]'";
   }
   elsif ( $testid == 3 )
   {
      # DTF -> DTF;  sep == ','
      $test_string = "$DMCOPY infile=$infile'[opt sep=\",\"]' outfile=$outfile'[opt kernel=text/dtf,sep=\",\"]'";
   }
   elsif ( $testid == 4 )
   {
      # DTF-FIXED -> DTF;  sep == ' '
      $test_string = "$DMCOPY infile=$infile'[opt sep=\",\"]' outfile=$outfile'[opt kernel=text/dtf]'";
   }
   elsif ( $testid == 5 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=0]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 6 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=1]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 7 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=2]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 8 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=3]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 9 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=4]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 10 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=5]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 11 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=6]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 12 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=7]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 13 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=8]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 14 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=9]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 15 )
   {
      # interp of string columns, escaped separators.
      $test_string = "$DMLIST infile=$infile'[opt skip=10]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 16 )
   {
      # SL-99 date to string + check consolidation preserves previous behavior
      $test_string = "$DMLIST infile=$infile'[opt colnames=last]' outfile=$outfile opt=cols";
   }
   $test_string .= " >> $logfname 2>&1";

   # --------------------------------------------------------------------------------
   # run the test
   dt_print_test_header( $testid, $test_string, $logfile, $printfile );
   close $logfile;
   
   ($rc, $time_user, $time_sys) = dt_run_test($test_string);
   
   open $logfile, ">>$logfname";
   select($logfile); $| = 1; select($printfile);
   unless ( $rc == 0 )
   {
      $err_msg = "ERROR: $toolname failed to run\n";
   }

   # --------------------------------------------------------------------------------
   # validate results
   my $block_typesref = $test_blocks{$testid};
   my %block_types = %$block_typesref;
   if ( $block_types{"table"} )
   {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"image"} )
   {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"list"} )
   {
      $err_msg .= dmlist_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \@tolkeys, $logfname, $test_diff, "" );
   }
   if ( $block_types{"text"} )
   {
      $err_msg .= diff_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \%test_words, $logfname, $test_diff );
   }

   # --------------------------------------------------------------------------------
   # report results
   unless ( $err_msg eq "" )
   {
      push( @faillist, $testid );
      $mismatch = 1;
   }
   dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
   $time_user_total += $time_user;
   $time_sys_total  += $time_sys;
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
