#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# destreak.t
# test script for destreak


# !!2
# syntax:
# destreak.t [<testid> ... ]




######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM,
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file>
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="destreak"

# set up list of tests
# !!4
alltests="test1 test2 test3 test4 test5 test6 test7a test7b"

# "short" test to run
# !!5
shortlist="test1"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined"
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on
# both the output and reference files of a tool the resultant utility output
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}_out.fits
  savfile=$SAVDIR/${testid}_sav.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test1 ) test1_string="destreak infile=$INDIR/${testid}_in.fits outfile=${outfile} max=\"1:1:2:1\" max_rowloss_fraction="1" filter=no mask= ccd_id=8 ccd_col=ccd_id node_col=node_id exptime=-1 countfile=$OUTDIR/${testid}_count.fits fracfile=$OUTDIR/${testid}_frac.fits timefile=$OUTDIR/${testid}_time.fits verbose=0 clobber=no num_sigma="3.0" > /dev/null"
            echo $test1_string | tee -a  $LOGFILE
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    # !!7
    test2 ) test2_string="destreak infile=$INDIR/${testid}_in.fits outfile=${outfile} max= max_rowloss_fraction="1" filter=no mask= ccd_id=8 ccd_col=ccd_id node_col=node_id exptime=-1 countfile=$OUTDIR/${testid}_count.fits fracfile=$OUTDIR/${testid}_frac.fits timefile=$OUTDIR/${testid}_time.fits verbose=0 clobber=no num_sigma="3.0" > /dev/null"
	    echo $test2_string | tee -a  $LOGFILE
	    eval $test2_string  | tee -a  $LOGFILE  2>&1
           ;;

    # !!8
    test3 ) test3_string="destreak infile=$INDIR/${testid}_in.fits outfile=${outfile} max=\"1:1:2:1\" max_rowloss_fraction="1" filter=yes mask=\"region(${testid}.reg)\" ccd_id=8 ccd_col=ccd_id node_col=node_id exptime=-1 countfile= fracfile= timefile= verbose=0 clobber=no num_sigma="3.0" > /dev/null"
	    echo $test3_string | tee -a  $LOGFILE
	    eval $test3_string  | tee -a  $LOGFILE  2>&1
           ;;

    # !!9
    test4 ) test4_string="destreak infile=$INDIR/${testid}_in.fits outfile=${outfile} max=\"1:1:2:1\" max_rowloss_fraction="1" filter=no mask="[status=0,grade=0,2:4,6]" ccd_id=8 ccd_col=ccd_id node_col=node_id exptime= countfile= fracfile= timefile= verbose=0 clobber=no num_sigma="3.0" > /dev/null"
	    echo $test4_string | tee -a  $LOGFILE
	    eval $test4_string  | tee -a  $LOGFILE  2>&1
           ;;

    test5 ) test5_string="destreak infile=$INDIR/${testid}_in.fits"[#row=:250000]" outfile=${outfile} max="" max_rowloss_fraction="5e-05" filter=no mask="[status=0,grade=0,2:4,6]" ccd_id=8 ccd_col=ccd_id node_col=node_id exptime="-1" countfile= fracfile= timefile= verbose=0 clobber=no num_sigma="1" > /dev/null"
	    echo $test5_string | tee -a  $LOGFILE
	    eval $test5_string  | tee -a  $LOGFILE  2>&1
           ;;

    test6 ) test6_string="destreak infile=$INDIR/${testid}_in.fits outfile=${outfile} exptime="1.7" timefile=$OUTDIR/${testid}_time.fits verbose=0 clobber=no > /dev/null"
      echo $test6_string | tee -a  $LOGFILE
      eval $test6_string  | tee -a  $LOGFILE  2>&1
           ;;

    #PTOBST-9: Test for handling new dmRegParse behavior
    #Empty mask won't be processed. Verbose=1 will show warning. Same baseline as test3.
    test7a ) test7a_string="destreak infile=$INDIR/${testid}_in.fits outfile=${outfile} max=\"1:1:2:1\" max_rowloss_fraction="1" filter=yes mask=\"mask()\" ccd_id=8 ccd_col=ccd_id node_col=node_id exptime=-1 countfile= fracfile= timefile= verbose=1 clobber=no num_sigma="3.0""
      echo $test7a_string | tee -a  $LOGFILE
      eval $test7a_string  | tee -a  $LOGFILE  2>&1s
           ;;
    #Pointer to mask file.
    test7b ) test7b_string="destreak infile=$INDIR/${testid}_in.fits outfile=${outfile} max=\"1:1:2:1\" max_rowloss_fraction="1" filter=yes mask=\"mask($INDIR/${testid}_mask.fits)\" ccd_id=8 ccd_col=ccd_id node_col=node_id exptime=-1 countfile= fracfile= timefile= verbose=0 clobber=no num_sigma="3.0" > /dev/null"
      echo $test7b_string | tee -a  $LOGFILE
      eval $test7b_string  | tee -a  $LOGFILE  2>&1
           ;;
  esac


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  # dmlist $outfile header,data > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
  # dmlist $savfile header,data > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #          2>>$LOGFILE

  # compare
  # !!12
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #      /dev/null 2>>$LOGFILE
 dmdiff $savfile $outfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
 if  test $? -ne 0 ; then
   echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
   mismatch=0
 fi

  #
  # if they exist, look at the output statistics files
  #
  optlist="time frac count"
  for optfile in $optlist
  do
     if [ -r $OUTDIR/${testid}_${optfile}.fits ]; then
       dmdiff $SAVDIR/${testid}_${optfile}.fits $OUTDIR/${testid}_${optfile}.fits tol=$INDIR/stat_tolerance > /dev/null 2>>$LOGFILE
       if  test $? -ne 0 ; then
         echo "ERROR: MISMATCH in $OUTDIR/${testid}_${optfile}.fits" >> $LOGFILE
         mismatch=0
       fi
     fi
   done

#  \rm -f $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  echo ""
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
