#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# dmcontour.t
# test script for dmcontour


# !!2
# syntax:
# dmcontour.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="dmcontour"

# set up list of tests
# !!4
alltests="test1 test_nan test_outdir_dne test_edge test_wcs test_nowcs test_wcs_linear test_wcs_sep test_invalid_polygon test_grid_stack test_grid_parse1 test_grid_parse2"

# "short" test to run
# !!5
shortlist="test1 test_nan test_wcs"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi

# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "running $testid" |tee -a $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test1 ) test1_string="dmcontour infile=$INDIR/dmcontour_input.fits outfile=$outfile levels=3,15 clobber+"
            ;;
    test_nan ) test1_string="dmcontour infile=$INDIR/flux_sm.fits outfile=$outfile levels=0.002 clobber+"
            ;;
    test_wcs ) test1_string="dmcontour infile=$INDIR/img outfile=$outfile levels=0.5 clobber+"
            ;;
    test_wcs_sep ) test1_string="dmcontour infile=$INDIR/separate_linear_wcs.img outfile=$outfile clob+ levels=3,15"
            ;;
    test_wcs_linear ) test1_string="dmcontour infile=$INDIR/faked_linear2_sm.fits level=1.0,1.1,1.2 outfile=$outfile clobber+"
            ;;
    test_nowcs ) test1_string="dmcontour infile=$INDIR/nowcs.img outfile=$outfile levels=0.5 clobber+"
            ;;
    test_edge ) test1_string=""
            ;;

    test_outdir_dne ) test1_string="dmcontour infile=$INDIR/flux_sm.fits outfile=/fake/dirDNE/$outfile levels=0.002 clobber+"
            ;;
    test_invalid_polygon) test1_string="dmcontour infile=$INDIR/invalid_polygon_db.fits out=$outfile level=1 clob+"
    ;;

    test_grid_stack ) test1_string="dmcontour $INDIR/dmcontour_input.fits outfile=$outfile levels=@-$INDIR/levels.lis clobber+"
    ;;
    test_grid_parse1) test1_string="dmcontour $INDIR/dmcontour_input.fits outfile=$outfile levels=1:100:#2l clobber+"
    ;;
    test_grid_parse2) test1_string="dmcontour $INDIR/dmcontour_input.fits outfile=$outfile levels='grid($INDIR/grid.dat)' clobber+"
    ;;


  esac

  if [ ${testid} = "test_outdir_dne" ]; then 
     echo $test1_string | tee -a  $LOGFILE 
     eval $test1_string > $OUTDIR/${testid}_scrub.0.tmp 2>&1 
     echo "status: " $? >> $OUTDIR/${testid}_scrub.0.tmp 
  elif [ ${testid} = "test_edge" ]; then
    echo ""
    echo "The test_edge dmcontour commands set to quiet. Several dmcontours to run, takes several moments ..."
    echo ""
    for d in $INDIR/edge/[uml]* 
    do 
      out=`basename "${d}"` 
      dmcontour infile=$d outfile=$OUTDIR/cont.$out levels=0.5 clobber+
      #echo "dmcontour infile=$d outfile=$OUTDIR/$out levels=0.5 clobber+"
    done
  else
     echo $test1_string | tee -a  $LOGFILE 
     eval $test1_string  | tee -a  $LOGFILE  2>&1
  fi


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
#dmlist $outfile header,data,array > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
#keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
#dmlist $savfile header,data,array > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
#keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
#          2>>$LOGFILE

  # compare
  # !!12
#diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
#     /dev/null 2>>$LOGFILE
  if [ ${testid} = "test_outdir_dne" ]; then 
    sed -e "s;${INDIR};;g" $OUTDIR/${testid}_scrub.0.tmp | sed -e "s;${OUTDIR};;g" | sed -e "s;# dmcontour (.*):;# dmcontour:;g" > $OUTDIR/${testid}_output.txt

    diff $OUTDIR/${testid}_output.txt $SAVDIR/${testid}_output.txt > /dev/null 2>> ${LOGFILE}
    if  test $? -ne 0 ; then
      echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}_output.txt" >> $LOGFILE
      mismatch=0
    fi
  elif [ ${testid} = "test_edge" ]; then
    sum=0
    for d in $SAVDIR/edge/cont.*
    do
      out=$(basename "$d")
      dmdiff $d $OUTDIR/${out} tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
      if [ $? -gt 0 ]; then
        sum=`expr $sum + 1`
      #  echo "dmdiff $d $OUTDIR/${out} tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
      else
        sum=`expr $sum + 0`
      fi
    done
    if [ $sum -gt 0 ]; then
      echo "ERROR: MISMATCH in a total of $sum files" >> $LOGFILE
      mismatch=0
    else
      test=0
    fi
  else 
    #echo ""
    #echo "dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    #echo ""
    dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
      mismatch=0
    fi
  fi


  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
