#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# dmdiff.t
# test script for dmdiff


# !!2
# syntax:
# dmdiff.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="dmdiff"

# set up list of tests
# !!4
alltests="test1 test2 test3 ulong ushort long short double float byte byte1 test4 test4_2 test5 test6 invalidtol1\
          invalidtol2 invalidtol3 invalidtol4 invalidtol5 invalidtol6 invalidtol7 test7 test8 test9 test10 test11\
          test12 test13 test14 test15 test16 test17 test18 col_test1 col_test2 col_test3 test19 test20"

# "short" test to run
# !!5
shortlist="test1"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.txt
  savfile=$SAVDIR/${testid}.txt_std

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test1 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    test2 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    test3 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    ulong ) test1_string="dmdiff infile1=$INDIR/dtypes/${testid}.fits infile2=$INDIR/dtypes/${testid}.thresh tolfile= outfile=${outfile} verbose=1"
            ;;
    long ) test1_string="dmdiff infile1=$INDIR/dtypes/${testid}.fits infile2=$INDIR/dtypes/${testid}.thresh tolfile= outfile=${outfile} verbose=1"
            ;;
    ushort ) test1_string="dmdiff infile1=$INDIR/dtypes/${testid}.fits infile2=$INDIR/dtypes/${testid}.thresh tolfile= outfile=${outfile} verbose=1"
            ;;
    short ) test1_string="dmdiff infile1=$INDIR/dtypes/${testid}.fits infile2=$INDIR/dtypes/${testid}.thresh tolfile= outfile=${outfile} verbose=1"
            ;;
    double ) test1_string="dmdiff infile1=$INDIR/dtypes/${testid}.fits infile2=$INDIR/dtypes/${testid}.thresh tolfile= outfile=${outfile} verbose=1"
            ;;
    float ) test1_string="dmdiff infile1=$INDIR/dtypes/${testid}.fits infile2=$INDIR/dtypes/${testid}.thresh tolfile= outfile=${outfile} verbose=1"
            ;;
    byte ) test1_string="dmdiff infile1=$INDIR/dtypes/${testid}.fits infile2=$INDIR/dtypes/${testid}.thresh tolfile= outfile=${outfile} verbose=1"
            ;;
    byte1 ) test1_string="dmdiff infile1=$INDIR/dtypes/${testid}_a.fits infile2=$INDIR/dtypes/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    test4 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
          ;;
    test4_2 ) test1_string="dmdiff infile1=$INDIR/test4_a.fits infile2=$INDIR/test4_b.fits tolfile=$INDIR/test4.tol outfile=${outfile} wcs=no verbose=1"
            ;;
    test5 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
####Test for string that contains % so that it is taken as literal and as formatting directive####
    test6 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    invalidtol1 ) test1_string="dmdiff infile1=$INDIR/3cols_a.dtf infile2=$INDIR/3cols_b.dtf tolfile=$INDIR/err1.tol outfile=${outfile} wcs=no verbose=1"
            ;;
    invalidtol2 ) test1_string="dmdiff infile1=$INDIR/3cols_a.dtf infile2=$INDIR/3cols_b.dtf tolfile=$INDIR/err2.tol outfile=${outfile} wcs=no verbose=1"
            ;;
    invalidtol3 ) test1_string="dmdiff infile1=$INDIR/3cols_a.dtf infile2=$INDIR/3cols_b.dtf tolfile=$INDIR/err3.tol outfile=${outfile} wcs=no verbose=1"
            ;;
    invalidtol4 ) test1_string="dmdiff infile1=$INDIR/3cols_a.dtf infile2=$INDIR/3cols_b.dtf tolfile=$INDIR/err4.tol outfile=${outfile} wcs=no verbose=1"
            ;;
    invalidtol5 ) test1_string="dmdiff infile1=$INDIR/image_a.dtf infile2=$INDIR/image_b.dtf tolfile=$INDIR/err5.tol outfile=${outfile} subspace=no verbose=1"
            ;;
    invalidtol6 ) test1_string="dmdiff infile1=$INDIR/image_a.dtf infile2=$INDIR/image_b.dtf tolfile=$INDIR/err6.tol outfile=${outfile} subspace=no verbose=1"
            ;;
    invalidtol7 ) test1_string="dmdiff infile1=$INDIR/image_a.dtf infile2=$INDIR/image_b.dtf tolfile=$INDIR/err7.tol outfile=${outfile} subspace=no verbose=1"
            ;;
#### Testing comparison two image files that have different unit value####
    test7 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.img infile2=$INDIR/${testid}_b.img tolfile= outfile=${outfile} verbose=1"
            ;;
################# Testing for units=no is at test18###########

#### Testing comparision of Subspace elements####
    test8 ) test1_string="dmdiff infile1=$INDIR/${testid}_ss_a.dtf infile2=$INDIR/${testid}_ss_b.dtf subspaces=yes tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
#### Test comparing transformation function for all dimensions ####
    test9 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=yes verbose=1"
            ;;
#### Test comparing of image for all dimensions ####
    test10 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} subspace=no verbose=1"
            ;;
    test11 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    test12 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} verbose=1"
            ;;
#### Test13-17 are test for various combinations of Number of columns and rows in the two infiles to be compared ####
    test13 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.txt infile2=$INDIR/${testid}_b.txt tolfile= outfile=${outfile} verbose=1"
            ;;
    test14 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.dtf infile2=$INDIR/${testid}_b.dtf tolfile= outfile=${outfile} verbose=1"
            ;;
    test15 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    test16 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    test17 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits tolfile= outfile=${outfile} verbose=1"
            ;;
#### Testing comparison two image files that units argument specified as no####
    test18 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.img infile2=$INDIR/${testid}_b.img units=no tolfile=$INDIR/test18.tol outfile=${outfile} subspace=no verbose=1"
            ;;
#### col_test[123] == Bug 13385
####    col_test1 vector cpt vs vector cpt (going under the hood) == no counterpart in new tests yet
####    col_test2 vector vs vector cpt                            == generic column set mismatch (each shows 1 unmatched column; exits)
####    col_test3 vector vs scalar with same name                 == covered by basic Vector tests in new suite ('det' column)
    col_test1 ) test1_string="dmdiff infile1=$INDIR/chip.fits'[cols chipx]' infile2=$INDIR/chip.fits'[cols chipx]' tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
    col_test2 ) test1_string="dmdiff infile1=$INDIR/chip.fits'[cols chip]' infile2=$INDIR/chip.fits'[cols chipx]' tolfile= outfile=${outfile} verbose=1"
            ;;
    col_test3 ) test1_string="dmdiff infile1=$INDIR/chip.fits'[cols chip]' infile2=$INDIR/chips_single_vec.fits'[cols chip]' tolfile= outfile=${outfile} verbose=1"
            ;;

      test19 ) test1_string="dmdiff infile1=$INDIR/SL26_a1 infile2=$INDIR/SL26_a2  units=no tolfile= outfile=${outfile} wcs=no verbose=1"
            ;;
      test20 ) test1_string="dmdiff infile1=$INDIR/${testid}_a.fits infile2=$INDIR/${testid}_b.fits  units=no tolfile= outfile=${outfile} verbose=1"
	    ;;

    esac

  echo $test1_string | tee -a  $LOGFILE 
  eval $test1_string  | tee -a  $LOGFILE  2>&1

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ######################################################################
  # ascii files
  # !!17

   
  mv ${outfile} ${outfile}_tmp
  cat ${outfile}_tmp | sed 's,# dmdiff (.*):,# dmdiff ( ),' \
      | grep -v "Infile " > ${outfile}
  diff ${outfile} ${savfile} > \
      /dev/null 2>>$LOGFILE
  
  if  test $? -ne 0 ; then
      echo "ERROR: TEXT MISMATCH in ${outfile}" >> $LOGFILE
      mismatch=0
  fi


  rm ${outfile}_tmp
 
  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
