#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.


# dmfilth.t
# test script for dmfilth


# syntax:
# dmfilth.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="dmfilth"

# set up list of tests
alltests="test1 test2 test3 test4 test5 test6 test7a test7b test7c test8a test8b sat93"

# "short" test to run
shortlist="test4"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  infile=$INDIR/${testid}.fits
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in

    test1 )
	    # method=DIST
	    cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=DIST srclist=@$INDIR/srcin.lis bkglist=\"circle(3809.0,3951.0,7.0)\" randseed=1"
	    ;;

    test2 )
	    # method=BILING
	    cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=BILINT srclist=\"circle(4219.0,3649.0,15.0)\" bkglist=\"circle(3809.0,3951.0,7.0)\" randseed=1"
	    ;;

    test3 )
	    # method=POLY
	    cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=POLY srclist=\"circle(3809.0,3951.0,7.0)\" bkglist=\"circle(3809.0,3951.0,15.0)\" randseed=1"
	    ;;

    test4 )
	    # method=POISSON
	    cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=POISSON srclist=\"circle(4219.0,3649.0,15.0)\" bkglist=\"circle(3809.0,3951.0,7.0)\" randseed=1"
	    ;;

    test5 )
	    # method=GLOBAL
	    cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=GLOBAL srclist=\"circle(4219.0,3649.0,15.0)\" bkglist=\"circle(3809.0,3951.0,7.0)\" randseed=1"
	    ;;
    
    test6 )
	    # method=POISSON
	    cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=POISSON srclist=@$INDIR/test6_src.lis bkglist=@$INDIR/test6_bkg.lis randseed=123"
	    ;;

    test7a )
      # Test7: Bug SAT-37. Catch error when 'mask'syntax is used in a 'region'-like argument
      # Test7ba mask(maskfile) in srclist. Should complete and output a file.byot 
      cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=BILINT srclist=\"mask(${infile}.mask)\" bkglist=\"\" randseed=1"
      ;;
    test7b )
      # Test7b mask() in srclist. Should fail.
      cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=POLY srclist=\"mask()\" bkglist=\"circle(3809.0,3951.0,15.0)\" randseed=1"
      ;;

    test7c )
      # Test7c mask() in bkglist. Should fail.
      cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=POLY srclist=\"circle(3809.0,3951.0,7.0)\" bkglist=\"mask()\" randseed=1"
      ;;
    test8a )
      # sat-190. When a fits input has no wcs transform, all coordinates should behave as physical
      cmd_string="dmfilth infile=$infile outfile=$outfile\"[image]\" clobber=yes method=DIST src=\"circle(50,50,5)\" bkg=\"annulus(50,50,20,30)\" randseed=1"
      ;;
    test8b )
      #sat-190 same case as above but with poly (significant enough code path changes make this necessary)
      cmd_string="dmfilth infile=$infile outfile=$outfile\"[image]\" clobber=yes method=POLY src=\"circle(50,50,5)\" bkg=\"circle(50,50,30)\" randseed=1"
      ;;
    sat93 )
      # sat93: Bug SAT-93. If a polygon falls outside the background region, return non-zero status
      cmd_string="dmfilth infile=$infile outfile=$outfile clobber=yes method=POLY srclist=\"circle(3990.5,4008.5,32)\" bkglist=\"circle(3786.5,3861.5,38)\" randseed=1"

  esac


  if [ ${testid} = "test7b" ] || [ ${testid} = "test7c" ] || [ ${testid} = "sat93" ] ; then 
      echo $cmd_string | tee -a  $LOGFILE
      eval $cmd_string
      mystatus=$?
      echo " -- status of call was $mystatus"  >>  $LOGFILE
      echo " -- status of call was $mystatus"  > ${outfile}.status
  else
    echo $cmd_string | tee -a  $LOGFILE 
    eval $cmd_string | tee -a  $LOGFILE  2>&1
  fi

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image

#  dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
#  if test $? -ne 0; then
#    echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
#    mismatch=0
#  fi

  #  Check the header of the image

  #dmlist $outfile header,clean > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE
  
  #dmlist $savfile header,clean > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  # 2>>$LOGFILE

  # compare

  #diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  # /dev/null 2>>$LOGFILE
  if [ ${testid} = "test7b" ] || [ ${testid} = "test7c" ] || [ ${testid} = "sat93" ]; then 
     # this test checks for expected errors so diff the exit status of run
     diff $outfile.status $savfile.status  > /dev/null 2>>$LOGFILE
     if  test $? -ne 0 ; then
       echo "ERROR: MISMATCH in $outfile.status" >> $LOGFILE
       mismatch=0
     fi
  else
    #echo "dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
      mismatch=0
    fi
  fi
  

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
