#! /bin/sh

# 23 August 2002

# dmgroup.t
# test script for dmgroup

# syntax:
# dmgroup.t [<testid> ... ]


######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Ddaattaa/g;
  s/DATE/Ddaattaa/g;s/CREATOR/Ddaattaa/g;s/HISTORY/Ddaattaa/g;
  s/DATASUM/Ddaattaa/g;s/ASCDSVER/Ddaattaa/g;s/CVSD0001/Ddaattaa/g;
  s/CVST0001/Ddaattaa/g;s/HISTNUM/Ddaattaa/g;
  s/COMMENT/Ddaattaa/g' | \
  grep -v Ddaattaa > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="dmgroup"

# set up list of tests
alltests="test1 test2 test3 test4 test5 test6 test7 test8 test9 test10 test11 test12 test13 test14 test15"

# "short" test to run
shortlist="test1"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*


# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc


# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in
    test1 ) command_string="dmgroup infile=$INDIR/dmgroup_pha1.fits \
              outfile=${outfile} \
              binspec=\"\" tabspec=\"\" \
              grouptype=NONE grouptypeval=0 xcolumn=CHANNEL \
              ycolumn=COUNTS tabcolumn=\"\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	       maxlength=0"
            ;;

    test2 ) command_string="dmgroup infile=$INDIR/dmgroup_pha1.fits \
              outfile=${outfile} \
              binspec=\"1:11:3\" tabspec=\"5:5\" \
              grouptype=BIN grouptypeval=0 xcolumn=CHANNEL \
              ycolumn=\"COUNTS\" tabcolumn=\"CHANNEL\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	       maxlength=0"
           ;;

    test3 ) command_string="dmgroup infile=\"@$INDIR/dmgroup_instack.txt\" \
              outfile=${outfile} \
              binspec=\"\" tabspec=\"\" \
              grouptype=SNR grouptypeval=50 xcolumn=CHANNEL \
              ycolumn=COUNTS tabcolumn=\"\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=STAT_ERR \
	      maxlength=0"
           ;;

    test4 ) command_string="dmgroup infile=$INDIR/dmgroup_pha1.fits \
              outfile=${outfile} \
              binspec=\"\" tabspec=\"\" \
              grouptype=NUM_CTS grouptypeval=1000 xcolumn=CHANNEL \
              ycolumn=COUNTS tabcolumn=\"\" clobber=yes \
	      stopspec=\"4:5\" stopcolumn=CHANNEL errcolumn=\"\" \
	       maxlength=0"
           ;;

    test5 ) command_string="dmgroup infile=$INDIR/dmgroup_pha1.fits \
              outfile=${outfile} \
              binspec=\"\" tabspec=\"\" \
              grouptype=NUM_BINS grouptypeval=5 xcolumn=CHANNEL \
              ycolumn=\"COUNTS\" tabcolumn=\"\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	       maxlength=0"
           ;;

    test6 ) command_string="dmgroup infile=$INDIR/dmgroup_pha1.fits \
              outfile=${outfile} \
              binspec=\"\" tabspec=\"\" \
              grouptype=ADAPTIVE_SNR grouptypeval=17 xcolumn=CHANNEL \
              ycolumn=COUNTS tabcolumn=\"\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	       maxlength=0"
           ;;

    test7 ) command_string="dmgroup infile=$INDIR/dmgroup_pha1.fits \
              outfile=${outfile} \
              binspec=\"\" tabspec=\"\" \
              grouptype=ADAPTIVE grouptypeval=700 xcolumn=CHANNEL \
              ycolumn=COUNTS tabcolumn=\"\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	       maxlength=3"
           ;;

    test8 ) command_string="dmgroup infile=$INDIR/dmgroup_pha2.fits \
              outfile=${outfile} \
              binspec=\"\" tabspec=\"\" \
              grouptype=BIN_WIDTH grouptypeval=4 xcolumn=CHANNEL \
              ycolumn=COUNTS tabcolumn=\"\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	       maxlength=0"
           ;;

    test9 ) command_string="dmgroup infile=$INDIR/dmgroup_pha2.fits \
              outfile=${outfile} \
              binspec=\"$INDIR/binfile.fits\" xcolumn=CHANNEL \
              tabspec=\"\" grouptype=BIN_FILE \
              grouptypeval=0 ycolumn=COUNTS  \
              tabcolumn=\"\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	       maxlength=0"
           ;;

    test10 ) command_string="dmgroup infile=$INDIR/dmgroup_pha1.fits \
              outfile=${outfile} \
              binspec=\"\" xcolumn=CHANNEL \
              tabspec=\"\" grouptype=MAX_SLOPE \
              grouptypeval=50 ycolumn=COUNTS  \
              tabcolumn=\"\" clobber=yes \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	       maxlength=0"
           ;;

    test11 ) command_string="dmgroup infile=$INDIR/decending.fits \
	      outfile=${outfile} grouptype=BIN grouptypeval=0 \
	      binspec=1:100:10 xcolumn=x ycolumn=y tabspec=5:5 \
	      tabcolumn=x stopspec=80:85 stopcolumn=x errcolumn=\"\" \
	      clobber=yes verbose=0 maxlength=0 mode=h"

           ;;

    test12 ) command_string="dmgroup infile=$INDIR/testin12.fits \
	      outfile=${outfile} grouptype=BIN grouptypeval=0 \
	      binspec="0:400:50,600:1000:100" xcolumn=pi ycolumn=counts \
	      tabspec="325:460:#1,650:700:#1" tabcolumn=pi \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	      clobber=yes verbose=0 maxlength=0 mode=h"

           ;;

    test13 ) command_string="dmgroup infile=$INDIR/testin13.fits \
	      outfile=${outfile} grouptype=BIN grouptypeval=0 \
	      binspec="100:200.5:100.5" xcolumn=boo ycolumn=counts \
	      tabspec=\"\" tabcolumn=\"\" \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	      clobber=yes verbose=0 maxlength=0 mode=h"

           ;;

    test14 ) command_string="dmgroup infile=$INDIR/testin14.fits \
	      outfile=${outfile} grouptype=NUM_CTS grouptypeval=28 \
	      binspec=\"\" xcolumn=channel ycolumn=counts \
	      tabspec=":11.5:#1" tabcolumn=pha \
	      stopspec=\"\" stopcolumn=\"\" errcolumn=\"\" \
	      clobber=yes verbose=0 maxlength=0 mode=h"

           ;;

#
# Test ::n 
#
    test15 ) command_string="dmgroup infile=$INDIR/testin15.fits \
	      outfile=${outfile} grouptype='BIN' grouptypeval='0' \
              binspec='::10' xcolumn='CHANNEL' ycolumn='COUNTS' \
              tabspec='' tabcolumn='' stopspec='' stopcolumn='' \
              errcolumn='' clobber='yes'"

           ;;


  esac

  echo $command_string | tee -a  $LOGFILE 
  eval $command_string  | tee -a  $LOGFILE  2>&1

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  ####################################################################
  # FITS table

  # new output  -added array -JW
  #dmlist $outfile header,data,array > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output  -added array -JW
  #dmlist $savfile header,data,array > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #          2>>$LOGFILE

  # compare
  #diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #     /dev/null 2>>$LOGFILE
  dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
    mismatch=0
  fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
