#!/bin/sh
# Time-stamp: "dmgroupreg.t 2003/05/06 14:15:52 crcarter"

##########################################################################
#
#	dmgroupreg.t
#	Regression test script for dmstat
#
#	Clayton Carter
#
##########################################################################

##########################################################################
# Tool specific variables
#####################################
# Aside from these vars, only the section `run the tool' is specific
# to the tool.  So you should be able to just copy this over for a
# different tool, change TOOLNAME, edit the `run the tool' portion and
# then update ALLTESTS and SHORTLIST and all should be well.

 TOOLNAME=dmgroupreg
 ALLTESTS="simple lesssimple twogroups negated complex simple-ex lesssimple-ex"
 ALLTESTS="${ALLTESTS} twogroups-ex negated-ex complex-ex"
SHORTLIST=${ALLTESTS}


#####################################
# This way, we can override the default and run the regressions with
# an arbitrary binary.  Ie, an updated `./dmstat` as opposed to standard
# `which dmstat`
if [ -z "$TOOL_OVERRIDE" ]
then
  TOOL=$TOOLNAME
else
  TOOL=$TOOL_OVERRIDE
fi

##########################################################################
# Subroutines
#####################################
# error_exit <message>
# Fatal error exit
#####################################

error_exit()
{
    eval $ECHO "$1" $TOBOTH
    eval $ECHO "${TOOLNAME} : FAIL" $TOBOTH
    exit 1
}

######################################
# find_tool <toolname>
# Checks that tool exists and is runnable
######################################

find_tool()
{
    s1=`type $1`
    s2=`echo $s1 | awk -F" " '{ print $3 }'`
    if [ -x "$s2" ]
    then
	:
    else
	error_exit "tool $1 not found"
    fi
}

###################################
# run_tool <centroid> <median> <sigma> <clip> <filter>
# Runs the tool/pipe
# params - turn `on'/`off' calculations of <param>
# filter - apply <filter> as a DM filter
###################################

run_tool ()
{ 
    # $1 will be yes or no

    EXEC="${TOOL} infile=${REGFILE} srcoutfile=${OUTSRC} "
    EXEC="${EXEC} bkgoutfile=${OUTBKG} exclude=$1"
    eval $ECHO "Running: " $EXEC $TOBOTH
    eval $EXEC $TOLOG

    return $?

}

##########################################################################
# Setup variables
#####################################
# The basic stuff
#####################################

    DATE=`date +'%y%m%d_%T'`
 LOGBASE="${TOOLNAME}_log"

#####################################
# Nitty Gritty Type of Stuff
#####################################

      RM="rm -f"
    DIFF="/usr/bin/diff"
    ECHO="/bin/echo"
PUNLEARN=punlearn
    PSET=pset
  DMLIST=dmlist
    GREP=egrep
    TAB='\\t'

if [ `uname` = "Linux" ]
then
    ECHO="$ECHO -e"
fi

for prog in $DMLIST $PSET $PUNLEARN $DIFF $PIPEXEC
do
  find_tool $prog
done

#####################################
# Check the TEST* variables
# and die if they're not set
#####################################

if [ -n "$TESTLOG" ]
then
    LOGDIR="${TESTLOG}/${TOOLNAME}"
    if [ ! -d "$LOGDIR" ]
    then
      eval $ECHO "${TOOLNAME}: log directory does not exist ..."
      eval $ECHO "${TOOLNAME}: creating ${LOGDIR}"
      mkdir -p $LOGDIR    
    fi
else
    error_exit "${TOOLNAME} problem: TESTLOG not set"
fi

#####################################
# Stuff we need for redirection
#####################################

    # We have to set this here, otherwise we wouldn't have $LOGDIR.  :(
 LOGFILE=$LOGDIR/$LOGBASE.$DATE

    #####################################
    # We can use these, in combo with eval
    # and $ECHO to redirect the output to
    # exactly where where we want it.
    # - TOLOG will send all output
    #   (STDOUT & STDERR) to LOGFILE
    # - TOSCREEN will send all output to
    #   the screen.  If you want, this could
    #   also be some sort of parser or something.
    #   Otherwise, it's not to fancy.  :)
    # - TOBOTH will copy STDOUT to the LOGFILE
    #   and will also print it to the screen
    #   Note: STDERR isn't (yet) redirected
    # - TOLOGAND can be used to redirect
    #   STDOUT to LOGFILE and to another file.
    ######################################
   TOLOG=">> ${LOGFILE} 2>> ${LOGFILE}"
TOSCREEN=""
  TOBOTH="2>> ${LOGFILE} | tee -a ${LOGFILE}"
TOLOGAND="2>> ${LOGFILE} | tee -a ${LOGFILE} >>"
  TOFILE="2>> ${LOGFILE} >>"

#####################################

if [ -n "$TESTOUT" ]
then
    OUTDIR="${TESTOUT}/${TOOLNAME}"
    if [ ! -d "$OUTDIR" ]
    then
      eval $ECHO \
       "${TOOLNAME}: output directory does not exist, creating: ${OUTDIR}" \
       $TOLOG
      mkdir -p $OUTDIR
    fi
else
    error_exit "${TOOLNAME} problem: TESTOUT not set"
fi

if [ -n "$TESTIN" ]
then
    INDIR=${TESTIN}/${TOOLNAME}
else
    error_exit "${TOOLNAME} problem: TESTIN not set"
fi

if [ -n "$TESTSAV" ]
then
    SAVDIR=${TESTSAV}/${TOOLNAME}
else
    error_exit "${TOOLNAME} problem: TESTSAV not set"
fi

#####################################
# Define the tests to be done, parsing
# args in the process
#####################################

# Any arguments given on the command line?
if [ 0 -lt "$#" ]
then
    ######################################
    # We're given args on the command line,
    # so we go through each one, checking 
    # to see if it's asking for help, the
    # long list, the short list or is just
    # a test.
    for arg in $*
    do
#	eval $ECHO "Examining argument: ${arg}" $TOLOG
	case $arg in
	    s|short) TESTLIST=$SHORTLIST ; break ;;
	    a|all)   TESTLIST=$ALLTESTS  ; break ;;
	    h|help)
		eval $ECHO "" $TOSCREEN
		eval $ECHO "${TOOLNAME} regression tests" $TOSCREEN
		eval $ECHO $TAB "available: ${ALLTESTS}" $TOSCREEN
		eval $ECHO $TAB "s\|short: only run ${SHORTLIST}" $TOSCREEN
		eval $ECHO $TAB "a\|all: run all tests \(def\)" $TOSCREEN
		eval $ECHO $TAB "h\|help: display this message" $TOSCREEN
		eval $ECHO "" $TOSCREEN
		exit 0                             ;;
	    *)       TESTLIST="${TESTLIST} ${arg}" ;;
	esac
    done
else
    
    # Presumably, if $SHORTTEST is of non-zero
    # length, we'll only want to do the short tests.
    if [ -n "$SHORTTEST" ]
    then
	TESTLIST=$SHORTLIST
    else
	TESTLIST=$ALLTESTS
    fi
fi


#########################################################################
# Begin running
#####################################
# Clean up from previous runs
#####################################

$RM $LOGDIR/$LOGBASE*

##########################################################################
# Begin testing
#####################################

eval $ECHO "" $TOBOTH
eval $ECHO "${TOOLNAME} regression" $TOBOTH

eval $ECHO "" $TOBOTH
eval $ECHO "Tests to run: ${TESTLIST}" $TOLOG

SCRIPT_SUCCEEDED=1

#####################################
# Begin per-test loop
#####################################

for testid in $TESTLIST
do
    
    # delete old outputs
    $RM ${OUTDIR}/${TOOLNAME}_${testid}*

    REGFILE=${INDIR}/${TOOLNAME}_${testid}.reg
    REGFILE=`echo $REGFILE | sed 's/-ex//'`

    OUTSRC="${OUTDIR}/${TOOLNAME}_${testid}.${DATE}.src"
    OUTBKG="${OUTDIR}/${TOOLNAME}_${testid}.${DATE}.bkg"
    SAVSRC="${SAVDIR}/${TOOLNAME}_${testid}.ref.src"
    SAVBKG="${SAVDIR}/${TOOLNAME}_${testid}.ref.bkg"

    eval $ECHO "" $TOLOG
    eval $ECHO "+----------+" $TOLOG
    eval $ECHO "Running test: $testid" $TOLOG
    eval $ECHO "+----------+" $TOLOG

    # reset the tool
    EXECSTRING="${PUNLEARN} ${TOOLNAME}"
    eval $ECHO "Running: " $EXECSTRING $TOLOG
    eval $EXECSTRING $TOLOG

#####################################
# run the tool
#####################################
# most of the EXECSTRING= lines are tool specific, as well as some of the
# other lines

    # catch bogus tests
    echo $ALLTESTS | grep $testid > /dev/null
    if ( test ! $? )
    then
	eval $ECHO "Test ${testid} not implemented." $TOBOTH
	eval $ECHO "" $TOBOTH
	SCRIPT_SUCCEEDED=0
	continue
    fi


    case ${testid} in
	*-ex)
	    run_tool yes
	    TEST_STATUS=$?
	    ;;

	*)
	    run_tool no
	    TEST_STATUS=$?
	    ;;

    esac


#####################################
# check the outputs
#####################################

    # Init per-test error flag
    MISMATCH=0

    # compare srcs
    EXEC="${DIFF} ${SAVSRC} ${OUTSRC}"
    eval $ECHO "" $TOLOG
    eval $ECHO "Running: " $EXEC $TOLOG
    eval $EXEC $TOLOG

    if [ 0 -ne "$?" ]
    then
	eval $ECHO "ERROR: MISMATCH in ${OUTSRC}" $TOLOG
	MISMATCH=1
    fi

    # compare bkgs
    EXEC="${DIFF} ${SAVBKG} ${OUTBKG}"
    eval $ECHO "" $TOLOG
    eval $ECHO "Running: " $EXEC $TOLOG
    eval $EXEC $TOLOG

    if [ 0 -ne "$?" ]
    then
	eval $ECHO "ERROR: MISMATCH in ${OUTBKG}" $TOLOG
	MISMATCH=1
    fi


#####################################
# Did we get an error?
#####################################

    eval $ECHO "" $TOBOTH

    if [ 0 -ne "$MISMATCH" ]
    then
	eval $ECHO $TAB "test ${testid}:" $TAB "NOT-OK" $TOBOTH
	SCRIPT_SUCCEEDED=0
    else
	eval $ECHO $TAB "test ${testid}:" $TAB "OK" $TOBOTH
    fi

    eval $ECHO "" $TOBOTH

done # for testid in $TESTLIST

##########################################################################
# Report results
#####################################

if [ 0 -eq "$SCRIPT_SUCCEEDED" ]
then
    eval $ECHO "${TOOLNAME}: FAIL" $TOBOTH
    SCRIPT_SUCCEEDED=1
else
    eval $ECHO "${TOOLNAME}: PASS" $TOBOTH
    SCRIPT_SUCCEEDED=0
fi

eval $ECHO "" $TOBOTH
eval $ECHO "Log file is ${LOGFILE}" $TOSCREEN
eval $ECHO "" $TOSCREEN

exit $SCRIPT_SUCCEEDED
