#! /bin/sh

# 19 Feb 2003

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.

# 19 Feb 2003:
# !!10, !!11, !!12 and !!14, !!15, !!16 have been replaced with
# a dmdiff's instead of dmlist + keyfilter

# !!1
# dmstat.t
# test script for mytool


# !!2
# syntax:
# dmstat.t [<testid> ... ]




######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM,
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
  punlearn $1
}

######################################################################
# subroutine
# zerotest <file>
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="dmstat"

# set up list of tests
# !!4
alltests="table image table-norows table-rows table-elems table-clip image-clip neg_cntrd_img"
alltests="${alltests} image-region image-median table-zero-tnull large_fitsfile null_filter"
alltests="${alltests} pipe_region complex_region"

# "short" test to run
# !!5
shortlist="table image"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*

# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined"
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on
# both the output and reference files of a tool the resultant utility output
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc
find_tool dmcopy



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
#bose=`pget $toolname verbose`
#punlearn $toolname
#pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${toolname}_${testid}*

  # Set up file names
  tblfile="$INDIR/${toolname}_table.fits"
  imgfile="$INDIR/${toolname}_image.fits"
  pipefile="$INDIR/${toolname}_pipe_image.fits"
  complexfile="$INDIR/${toolname}_complex_image.fits"
  regfile="$INDIR/${toolname}_reg.fits"
  zoofile="$INDIR/${toolname}_zoo.reg"
  acisfile="$INDIR/${toolname}_acis.fits"
  largeacisfile="$INDIR/${toolname}_acisf02810_001N001_r0023_rmf3.fits"
  outfile="$OUTDIR/${toolname}_${testid}.txt"
  parfile="$OUTDIR/${toolname}_${testid}.param.txt"
  savfile="$SAVDIR/${toolname}_${testid}.ref.txt"

  echo "" | tee -a $LOGFILE
  echo "Running test:$testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
      table ) test_string="dmstat infile=$tblfile \
	  centroid=no median=yes sigma=yes clip=no"
      	  ;;

      image ) test_string="dmstat infile=$imgfile \
	  centroid=yes median=no sigma=yes clip=no"
	  ;;

      table-norows ) test_string="dmstat infile=$tblfile'[detsize<0][cols pos]' \
	  centroid=no median=yes sigma=yes clip=no"
	  ;;

      table-rows ) test_string="dmstat infile=$zoofile'[cols pos]' \
	  centroid=no median=yes sigma=yes clip=no"
	  ;;

      table-elems ) test_string="dmstat infile=$acisfile'[cols phas]' \
	  centroid=no median=no sigma=yes clip=no"
	  ;;

      table-clip ) test_string="dmstat infile=$tblfile \
	  centroid=no median=no sigma=yes clip=yes"
	  ;;

      image-clip ) test_string="dmstat infile=$imgfile \
	  centroid=no median=no sigma=yes clip=yes"
	  ;;

      image-region ) test_string="dmstat infile=$imgfile'[(x,y)=region($regfile)]' \
	  centroid=yes median=no sigma=yes clip=no"
	  ;;

      image-median ) test_string="dmstat infile=$imgfile'[(x,y)=region($regfile)]' \
	  centroid=no median=yes  sigma=no clip=no"
	  ;;

      table-zero-tnull ) test_string="dmstat infile=$INDIR/dmstat_tab_with_0_tnull.fits"
         ;;

      neg_cntrd_img ) test_string="dmstat infile=$INDIR/neg_cntrd_img.fits cent+ verb=1"
         ;;

      large_fitsfile ) test_string="dmstat infile=$largeacisfile"
	  ;;

      null_filter ) test_string="dmstat infile=$imgfile'[pos=circle(16500,16500,10)*circle(17500,17500,10)][opt full]' cen- sig- med-"
	  ;;

    pipe_region ) test_string="dmcopy $pipefile'[(col1,col2)=box(49.5,49.5,100,100)-box(49.5,49.5,50,50)]' - \
                               | dmstat - cen- sig- med- "
    ;;

    complex_region ) test_string="dmstat $complexfile'[(cx,cy)=box(20,20,40,40)]' \
                                  cen- sig- med- "
    ;;


  esac

  echo $test_string | tee -a $LOGFILE
# eval $test_string | tee -a  $LOGFILE  2>&1                #Original
  eval $test_string | tee -a  $LOGFILE  >> $outfile         #Works fine
  eval pdump $toolname | egrep -v infile >> $parfile
  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  #   dmlist $outfile header,data,array > $OUTDIR/${testid}.dmp1  2>> ${LOGFILE}
  #   keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>> ${LOGFILE}

  # reference output
  # !!11
  #   dmlist $savfile header,data,array > $OUTDIR/${testid}.dmp1_std  2>> ${LOGFILE}
  #   keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #	  2>> ${LOGFILE}

  # compare
  # !!12
  #    diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #      /dev/null 2>> ${LOGFILE}
  #    dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>> ${LOGFILE}
  #    if  test $? -ne 0 ; then
  #	  echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
  #	  mismatch=0
  #    fi

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check imge
  # !!13
  #    dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>> ${LOGFILE}
  #    if test $? -ne 0; then
  #	  echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #	  mismatch=0
  #    fi

  #  Check the header of the image

  # !!14
  #    dmlist $outfile header,data,array > $OUTDIR/${testid}.dmp1  2>> ${LOGFILE}
  #   keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>> ${LOGFILE}

  # !!15
  #    dmlist $savfile header,data,array > $OUTDIR/${testid}.dmp1_std  2>> ${LOGFILE}
  #   keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #	  2>> ${LOGFILE}

  # compare
  # !!16
  #    diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #	  /dev/null 2>> ${LOGFILE}
  #    dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>> ${LOGFILE}
  #    if  test $? -ne 0 ; then
  #	  echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #	  mismatch=0
  #    fi

  ######################################################################
  # ascii files
  # !!17
#   diff $OUTDIR/${toolname}_${testid}.txt $OUTDIR/${toolname}_${testid}.txt_std > \
#         /dev/null 2>> ${LOGFILE}

   diff $OUTDIR/${toolname}_${testid}.txt $SAVDIR/${toolname}_${testid}.ref.txt > \
         /dev/null 2>> ${LOGFILE}

   if  test $? -ne 0 ; then
     echo "ERROR: TEXT MISMATCH in $OUTDIR/${toolname}_${testid}.txt" >> $LOGFILE
     mismatch=0
   fi

   diff $OUTDIR/${toolname}_${testid}.param.txt $SAVDIR/${toolname}_${testid}.param.ref.txt > \
       /dev/null 2>> ${LOGFILE}

   if  test $? -ne 0 ; then
     echo "ERROR: TEXT MISMATCH in $OUTDIR/${toolname}_${testid}.par" >> $LOGFILE
     mismatch=0
   fi
  ####################################################################
  # Did we get an error?
  echo ""
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
