#!/usr/bin/env perl
# 
#  Copyright (C) 2020,2022  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

#
#############################################################
# Regression Test Script Template
#############################################################
# Turn off buffering
select(STDOUT); $| = 1;

my $tool = dslib; # Name of tool to be tested

#----------------------------------------------------------------------
# Find Test Utilities File
#----------------------------------------------------------------------
use File::Basename;
$cwd = dirname($0); # relative to where this script is run from (cwd).
$tutils = "${cwd}/regtest_utils.pl";
if ( ! -e "$tutils" )
{
   print "ERROR: Test Utilities file not found. '${tutils}' \n";
   exit(1);
}
else
{
   require "$tutils";
}

#----------------------------------------------------------------------
# Define base directories
#----------------------------------------------------------------------
#Get test paths with tool specific extensions
($indir, $outdir, $savdir, $logdir) = &get_test_dirs( $tool );

#Create output directories as needed
&gen_test_dirs( $outdir, $logdir );
  
#----------------------------------------------------------------------
# Define Test Suite.
#----------------------------------------------------------------------
my @testlist = define_test_suite();

#----------------------------------------------------------------------
# Standard Test Run.
#----------------------------------------------------------------------
do_standard_test( "${tool}_gai", $outdir, $logdir, $savdir, \@testlist );



#----------------------------------------------------------------------
# SUBROUTINES
#----------------------------------------------------------------------
# ===================================================================
# define_test_suite( void )
#   Create TestCase objects defining each test.
#
#   Arguments
#     none
#
#   Globals
#     tool    = Tool being tested
#     indir   = Test input directory
#     outdir  = Test output directory
#
#   Returns
#     tests: array of TestCase objects
# --------------------------------------------------------------------
sub define_test_suite
{
  my @testlist = [];
  my $ntests   = 7;   #!! how many test cases

  # pre-size the testslist array
  $endndx = $ntests - 1;
  $#testlist = $endndx;  # this sizes the array

  #!! Find test executable - adjust as needed to support command generation.
  my $exec = "test_get_axis_info";
  my $toolexe = "";
  $toolexe = find_in_path( "${tool}/${exec}", $ENV{'PATH'}, ':');
  if ( $toolexe eq "" )
  { #try local to test script
    if ( -e "${cwd}/${exec}" ){ $toolexe = "${cwd}/${exec}"; }
  }
  if ( $toolexe eq "" )
  {
    print "ERROR: Tool executable not found. \n";
  }

  # Find system utilities used
  #!! Add any system utilities used in test command strings
  #my $CP = find_in_path("cp", $ENV{'PATH'}, ':');

  my $comstr = ""; # command string

  for $ii (1 .. $ntests) 
  {
    #NOTE: added qualifier to the test id, so that the 
    #      tests specific for this method are isolated
    #      from any other with the dslib test suite.
    my $testid = sprintf("gai_test%02d", $ii);
    my $t = TestCase->new( id => $testid, mksubdir => 0 );

    #Setup each TestCase.
    #!! Fill in TestCase details for each test
    if ( $testid eq "gai_test01" )
    {
      my $outfile = "${testid}_out.txt";
      $comstr  = "${toolexe} ${indir}/test_image1.fits > ${outdir}/${outfile}"; # Build Test Command
    
      $t->desc("Image with no specified WCS");  # describe what is being tested
      $t->type("unit");             # flavor "unit" or "regression"
      $t->command( $comstr );       # set command string
      $t->verify($outfile, diff);   # validation criteria (diff)
    }
    if ( $testid eq "gai_test02" )
    {
      my $outfile = "${testid}_out.txt";
      $comstr  = "${toolexe} ${indir}/test_image2.fits > ${outdir}/${outfile}"; # Build Test Command
    
      $t->desc("Image with 2-1D Physical axes (no vector) only.");  # describe what is being tested
      $t->type("unit");             # flavor "unit" or "regression"
      $t->command( $comstr );       # set command string
      $t->verify($outfile, diff);   # validation criteria (diff)
    }
    if ( $testid eq "gai_test03" )
    {
      my $outfile = "${testid}_out.txt";
      $comstr  = "${toolexe} ${indir}/test_image3.fits > ${outdir}/${outfile}"; # Build Test Command
    
      $t->desc("Image with 1-2D Physical axes only.");  # describe what is being tested
      $t->type("unit");             # flavor "unit" or "regression"
      $t->command( $comstr );       # set command string
      $t->verify($outfile, diff);   # validation criteria (diff)
    }
    if ( $testid eq "gai_test04" )
    {
      my $outfile = "${testid}_out.txt";
      $comstr  = "${toolexe} ${indir}/test_image4.fits > ${outdir}/${outfile}"; # Build Test Command
    
      $t->desc("Image with 2-1D Physical axes and 2-1D World axes");  # describe what is being tested
      $t->type("unit");             # flavor "unit" or "regression"
      $t->command( $comstr );       # set command string
      $t->verify($outfile, diff);   # validation criteria (diff)
    }
    if ( $testid eq "gai_test05" )
    {
      my $outfile = "${testid}_out.txt";
      $comstr  = "${toolexe} ${indir}/test_image5.dtf > ${outdir}/${outfile}"; # Build Test Command
    
      $t->desc("Image with 2-1D Physical axes and 1-2D World axes; DTF format");  # describe what is being tested
      $t->type("unit");             # flavor "unit" or "regression"
      $t->command( $comstr );       # set command string
      $t->verify($outfile, diff);   # validation criteria (diff)
    }
    if ( $testid eq "gai_test06" )
    {
      my $outfile = "${testid}_out.txt";
      $comstr  = "${toolexe} ${indir}/test_image6.dtf > ${outdir}/${outfile}"; # Build Test Command
    
      $t->desc("Image with 1-2D Physical axes and 1-2D World axes; DTF format");  # describe what is being tested
      $t->type("unit");             # flavor "unit" or "regression"
      $t->command( $comstr );       # set command string
      $t->verify($outfile, diff);   # validation criteria (diff)
    }
    if ( $testid eq "gai_test07" )
    {
      my $outfile = "${testid}_out.txt";
      $comstr  = "${toolexe} ${indir}/test_image7.dtf > ${outdir}/${outfile}"; # Build Test Command
    
      $t->desc("Image not 2D - invalid image");  # describe what is being tested
      $t->type("unit");             # flavor "unit" or "regression"
      $t->command( $comstr );       # set command string
      $t->verify('fail', 65280);    # validation criteria (fail test)
    }

    $ndx = $ii - 1;
    $testlist[$ndx] = $t;
  }

  return(@testlist);
}
