#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# hrc_build_badpix.t
# test script for hrc_build_badpix


# !!2
# syntax:
# hrc_build_badpix.t [<testid> ... ]
 


######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo ""   | tee -a $LOGFILE
  echo "$1" | tee -a $LOGFILE
  echo ""   | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/DATE-OBS/date-obs/g;s/DATE-END/date-end/g;
  s/CHECKSUM/Dataset/g;s/DATE/Dataset/g;s/CREATOR/Dataset/g;
  s/DEGAP /Dataset /g;s/BPIXFILE /Dataset /g; s/CONTINUE /Dataset /g
  s/HISTORY/Dataset/g;s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;
  s/HISTNUM/Dataset/g;s/COMMENT/Dataset/g;' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}

######################################################################
# Initialization

# !!3
toolname="hrc_build_badpix"

# set up list of tests
# !!4
alltests="test_hrcI test_hrcS  test_hrcS_II test_hrcS_III test1 test2 test3 test4 test4_a test4_a2 test4_a3 test_hrcI_b test4_hrcS S_5168_degap_3 S_5168_degap_4 I_degap_3 I_degap_4 S_810_degap_4"

# "short" test to run
# !!5
shortlist="test_hrcI test_hrcS test_hrcS_II test_hrcS_III"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# Make sure we have an input directory
if test -d $INDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find input directory : $INDIR"
  fi
fi

# Make sure we have a save directory
if test -d $SAVDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find save directory : $SAVDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo "" | tee -a $LOGFILE

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # Init per-test error flag
  # 
  mismatch=1
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test_hrcI ) 
            tstring="hrc_build_badpix infile=${INDIR}/hrci_badpix.fits \
                 outfile=${outfile} \
                 obsfile=${INDIR}/hrc_gihicr4005i001n000_obs.par \
                 degapfile=${INDIR}/hrciD1997-03-19gapN0000.fits \
                 logfile=${OUTDIR}/hbb_hrci_badpix_out.log \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 \
                 clobber=yes > /dev/null"
            echo $tstring | tee -a  $LOGFILE 
            echo "" | tee -a  $LOGFILE
	    eval  $tstring
            ;;

    # !!7
    test_hrcS )
            tstring=" hrc_build_badpix
                 infile ="${INDIR}/hrcs_badpix.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/hrcs_gihicr4005i001n000_obs.par" \
                 degapfile ="NONE" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes \
                 > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;

    # !!8 ;  (7/2002) fap's new spec ; fix edge as well; old degap file ; 
    test_hrcS_II ) 
            tstring=" hrc_build_badpix
                 infile ="${INDIR}/hrcsD1999-07-22badpixN0002.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/axaf3399_obs0a.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gapN0001.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.18 cfu2=-0.16 cfv1=1.11 cfv2=-0.1 clobber=yes \
                 > /dev/null"
	    echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
	    eval  $tstring
            ;;

    # !!8 ;  (7/2002) fap's new spec ; fix edge as well; new degap file ; 
    test_hrcS_III )
            tstring=" hrc_build_badpix
                 infile ="${INDIR}/hrcsD1999-07-22badpixN0002.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/axaf3399_obs0a.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gapN0002.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.18 cfu2=-0.16 cfv1=1.11 cfv2=-0.1 clobber=yes \
                 > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # !!new hrcS caldb ( hrcs_5168 )
     test1 ) tstring=" hrc_build_badpix
                 infile ="${INDIR}/hrcsD1999-07-22badpixN0003.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gapN0002.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # !!new hrcS caldb ( hrcs_3702 )
     test2 ) tstring=" hrc_build_badpix
                 infile ="${INDIR}/hrcsD1999-07-22badpixN0003.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs03702.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gapN0002.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # from test1;tstart/stop cross over boundary;test warning;test3.fits=test1.fits;
     test3 ) tstring=" hrc_build_badpix
                 infile ="${INDIR}/test_badpix.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gapN0002.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # from test1; status=0x20 and T0=NaN; test warning; test4.fits!=test1.fits;
     test4 ) tstring=" hrc_build_badpix
                 infile ="${INDIR}/test_badpix2.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gapN0002.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # from test4; test new degap format 
     test4_a ) tstring=" hrc_build_badpix
                 infile ="${INDIR}/test_badpix2.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gaplookupN0001.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # set degapfile=CALDB to test new degap format (hrcsD1999-07-22gaplookupN0002.fits)
     test4_a2 ) tstring=" hrc_build_badpix
                 infile ="${INDIR}/test_badpix2.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile =CALDB \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # copy from test4_a2 except set infile=CALDB ; 
     test4_a3 ) tstring=" hrc_build_badpix
                 infile =CALDB \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile =CALDB \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # 4/2018: same cmd as 'test4_a' except different degap file versions; both tests are using degap #3;
     test4_hrcS ) tstring=" hrc_build_badpix
                 infile ="${INDIR}/test_badpix2.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gaplookupN0004.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # 4/2018: same cmd as 'test4_hrcS' except usning the new badpix caldb file as infile 
     S_5168_degap_3 ) tstring=" hrc_build_badpix
                 infile ="${INDIR}//hrcsD1999-07-22badpixN0004.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gaplookupN0004.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # 4/2018: same cmd as S_5168_degap_3 except using degap #4 
     S_5168_degap_4 ) tstring=" hrc_build_badpix
                 infile ="${INDIR}//hrcsD1999-07-22badpixN0004.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/obs05168.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gaplookupN0004_fake.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # 10/19/2018: degap #4 ; use adjusted rawY to compute chipY for 2nd point ; 
     S_810_degap_4 ) tstring=" hrc_build_badpix
                 infile ="${INDIR}//hrcsD1999-07-22badpixN0004.fits" \
                 outfile ="${outfile}" \
                 obsfile ="${INDIR}/axaf00810_obs0a.par" \
                 degapfile ="${INDIR}/hrcsD1999-07-22gaplookupN0004_fake.fits" \
                 logfile="${OUTDIR}/hbb_hrcs_badpix_out.log" \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 clobber=yes  > /dev/null"
            echo $tstring | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval  $tstring
            ;;
     # 4/2018: same cmd as test_hrcI except using degap #3;
     test_hrcI_b ) 
            tstring="hrc_build_badpix infile=${INDIR}/hrci_badpix.fits \
                 outfile=${outfile} \
                 obsfile=${INDIR}/hrc_gihicr4005i001n000_obs.par \
                 degapfile=${INDIR}/hrciD1999-07-22gaplookupN0004.fits \
                 logfile=${OUTDIR}/hbb_hrci_badpix_out.log \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 \
                 clobber=yes > /dev/null"
            echo $tstring | tee -a  $LOGFILE 
            echo "" | tee -a  $LOGFILE
	    eval  $tstring
            ;;
     # 4/2018: same cmd as test_hrcI_b except using the badpix caldb file as infile; degap #3 ;
     I_degap_3 ) 
            tstring="hrc_build_badpix \
                 infile=${INDIR}/hrciD1999-07-22badpixN0002.fits \
                 outfile=${outfile} \
                 obsfile=${INDIR}/hrc_gihicr4005i001n000_obs.par \
                 degapfile=${INDIR}/hrciD1999-07-22gaplookupN0004.fits \
                 logfile=${OUTDIR}/hbb_hrci_badpix_out.log \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 \
                 clobber=yes > /dev/null"
            echo $tstring | tee -a  $LOGFILE 
            echo "" | tee -a  $LOGFILE
	    eval  $tstring
            ;;
     # 4/2018: same cmd as I_degap_3 except using degap #4;
     I_degap_4 ) 
            tstring="hrc_build_badpix \
                 infile=${INDIR}/hrciD1999-07-22badpixN0002.fits \
                 outfile=${outfile} \
                 obsfile=${INDIR}/hrc_gihicr4005i001n000_obs.par \
                 degapfile=${INDIR}/hrciD1999-07-22gaplookupN0004_fake.fits \
                 logfile=${OUTDIR}/hbb_hrci_badpix_out.log \
                 cfu1=1.0 cfu2=0.0 cfv1=1.0 cfv2=0.0 \
                 clobber=yes > /dev/null"
            echo $tstring | tee -a  $LOGFILE 
            echo "" | tee -a  $LOGFILE
	    eval  $tstring
            ;;
  esac

  ####################################################################
  # if the tool failed to run, set mismatch to 0.
  #
  #if test $? -ne 0; then
  #  echo "" | tee -a $LOGFILE
  #  echo "$toolname failed to run" | tee -a $LOGFILE
  #  echo "" | tee -a $LOGFILE
  #  mismatch=0
  #fi


  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  #dmlist $outfile header,data > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
  #dmlist $savfile header,data > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #          2>>$LOGFILE

  # compare
  # !!12
  #diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #     /dev/null 2>>$LOGFILE
  dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
    mismatch=0
  fi
  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  echo ""
  echo ""
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK" | tee -a $LOGFILE
    script_succeeded=1
  else
    # No
    echo "${testid} OK" | tee -a $LOGFILE
  fi

  echo "" | tee -a $LOGFILE

done
# end per-test loop
######################################################################

######################################################################
# report results

# blank line
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo ""
echo "log file in ${LOGFILE}"
echo ""

exit $script_succeeded
