#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# mkwarf.t
# test script for mkwarf


# !!2
# syntax:
# mkwarft. [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="mkwarf"

# set up list of tests
# !!4
alltests="test1 test2 test3 test4 test_mask test_kywds test_fef1 test_fef2 test_fef3 test_blank_wf test_blank_ff test_minrmf test_same_outfiles"

# "short" test to run
# !!5
shortlist="test1 test2 test3 test4 test_mask test_kywds test_fef1 test_fef2 test_fef3"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*

# original CALDB
orig_caldb=${CALDB}

# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo ""
  echo "running $testid" | tee -a $LOGFILE
  echo ""

  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test1 ) test1_string="mkwarf infile=$INDIR/mkwarf_input.fits\"[wmap]\" outfile=$OUTDIR/${testid}_arf.fits weightfile=$OUTDIR/${testid}_wght.fits spectrumfile=$INDIR/mkwarf_spectrum.txt egrid=0.5:9.5:0.02 clobber+ mskfile=none detsubsys='' dafile=NONE pbk="
            ;;

    test2 ) test1_string="mkwarf infile=$INDIR/mkwarf_input.fits\"[wmap]\" outfile=$OUTDIR/${testid}_arf.fits weightfile=$OUTDIR/${testid}_wght.fits spectrumfile=none egrid=0.5:9.5:0.02 clobber+ mskfile=$INDIR/mkwarf_msk1.fits detsubsys=IDEAL dafile=NONE pbk="
            ;;

    test3 ) test1_string="mkwarf infile=$INDIR/mkwarf_input.fits\"[wmap]\" outfile=$OUTDIR/${testid}_arf.fits weightfile=$OUTDIR/${testid}_wght.fits spectrumfile=none egrid=0.5:9.5:0.02 clobber+ mskfile=$INDIR/mkwarf_msk1.fits detsubsys=IDEAL dafile="CALDB" pbk=$INDIR/acisf066534098N003_pbk0.fits"
            ;;
            
    test4 ) test1_string="mkwarf infile=$INDIR/test4_mkwarf_cti_input.fits\"[wmap]\" outfile=$OUTDIR/${testid}_arf.fits weightfile=$OUTDIR/${testid}_wght.fits spectrumfile=none egrid=0.5:9.5:0.02 clobber+ mskfile=$INDIR/test4_msk1.fits detsubsys=IDEAL dafile="CALDB" pbk=$INDIR/test4_pbk0.fits"
            ;;

    test_mask ) test1_string="mkwarf infile=$INDIR/spectra.fits\"[wmap]\" outfile=$OUTDIR/${testid}_arf.fits weightfile=$OUTDIR/${testid}_wght.fits spectrumfile=none egrid=0.5:9.5:0.02 clobber+ mskfile=$INDIR/acisf00770_000N002_msk1.fits detsubsys='' dafile=NONE pbk="
            ;;


    test_kywds ) test1_string="mkwarf infile=$INDIR/test_kywds_in.fits\"[wmap]\" outfile=$OUTDIR/${testid}_arf.fits weightfile=$OUTDIR/${testid}_wght.fits spectrumfile=none egrid=0.3:9.3:0.1 clobber+ pbkfile='' threshold='0' feffile='CALDB' mskfile='' asolfile='' mirror='HRMA' detsubsysmod='' dafile='NONE' ardlibpar='ardlib' geompar='geom'"
            ;;


    test_fef1 ) test1_string="mkwarf infile=$INDIR/hrcwmap.fits outfile=$OUTDIR/${testid}_arf.fits weightfile='none' spectrumfile=none egrid=0.3:9.3:0.1 clobber+ pbkfile='' threshold='0' feffile='none' mskfile='' asolfile='' mirror='HRMA' detsubsysmod='' dafile='CALDB' ardlibpar='ardlib' geompar='geom'"
            ;;

    
    test_fef2 ) test1_string="mkwarf infile=/data/regression_test/master/in/mkwarf/hrcwmap.fits outfile=/stage/pool14/dtn/TOOLS/regtests/mkwarf/test_hrc_arf.fits weightfile=CALDB spectrumfile=none egrid=0.3:9.3:0.1 clobber+ pbkfile='' threshold='0' feffile=none mskfile='' asolfile='' mirror='HRMA' detsubsysmod='' dafile='CALDB' ardlibpar='ardlib' geompar='geom'"
            ;;


    test_fef3 ) test1_string="mkwarf infile=$INDIR/hrcwmap.fits outfile=$OUTDIR/${testid}_arf.fits weightfile=CALDB spectrumfile=none egrid=0.3:9.3:0.1 clobber+ pbkfile='' threshold='0' feffile=CALDB mskfile='' asolfile='' mirror='HRMA' detsubsysmod='' dafile='CALDB' ardlibpar='ardlib' geompar='geom'"
            ;;

    test_blank_wf ) test1_string="mkwarf infile=$INDIR/hrcwmap.fits outfile=$OUTDIR/${testid}_arf.fits weightfile='' spectrumfile=none egrid=0.3:9.3:0.1 clobber+ pbkfile='' threshold='0' feffile=none mskfile='' asolfile='' mirror='HRMA' detsubsysmod='' dafile='CALDB' ardlibpar='ardlib' geompar='geom'"
            ;;

    test_blank_ff ) test1_string="mkwarf infile=$INDIR/hrcwmap.fits outfile=$OUTDIR/${testid}_arf.fits weightfile=None spectrumfile=none egrid=0.3:9.3:0.1 clobber+ pbkfile='' threshold='0' feffile='' mskfile='' asolfile='' mirror='HRMA' detsubsysmod='' dafile='CALDB' ardlibpar='ardlib' geompar='geom'"
            ;;
    
    test_minrmf ) test1_string="mkwarf infile=$INDIR/hrcwmap.fits outfile=$OUTDIR/${testid}_arf.fits weightfile=None spectrumfile=none egrid=0.06:9.3:0.05 clobber+ pbkfile='' threshold='0' feffile='' mskfile='' asolfile='' mirror='HRMA' detsubsysmod='' dafile='CALDB' ardlibpar='ardlib' geompar='geom'"
            ;;

    test_same_outfiles ) test1_string="mkwarf infile=$INDIR/mkwarf_input.fits\"[wmap]\" outfile=$OUTDIR/${testid}_out.fits weightfile=$OUTDIR/${testid}_out.fits spectrumfile=none egrid=0.5:9.5:0.02 clobber+ mskfile=$INDIR/mkwarf_msk1.fits detsubsys= dafile=NONE pbk="
        ;;
    
  esac
  echo $test1_string | tee -a $LOGFILE 
  eval $test1_string  | tee -a $LOGFILE  2>&1


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
#dmlist $OUTDIR/${testid}_arf.fits header,clean,data,array > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
#keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
#dmlist $SAVDIR/${testid}_arf.fits header,clean,data,array > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
#keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std 2>>$LOGFILE

  # compare
  # !!12
#diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > /dev/null 2>>$LOGFILE

if [ ${testid} != "test_fef2" ] && [ ${testid} != "test_same_outfiles" ]; then
    if [ ${testid} != "test_fef3" ]; then

	dmdiff $OUTDIR/${testid}_arf.fits $SAVDIR/${testid}_arf.fits tol=$SAVDIR/tolerance error_on_comment=no > /dev/null 2>>$LOGFILE
	if  test $? -ne 0 ; then
	    echo "ERROR: MISMATCH in $OUTDIR/${testid}_arf.fits" >> $LOGFILE
	    mismatch=0
	fi
    fi
fi


#dmlist $OUTDIR/${testid}_wght.fits header,clean,data,array > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
#keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
#dmlist $SAVDIR/${testid}_wght.fits header,clean,data,array > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
#keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std 2>>$LOGFILE

  # compare
  # !!12
#diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > /dev/null 2>>$LOGFILE

if [ ${testid} != "test_fef1"  ] &&  [ ${testid} != "test_fef2" ] && [ ${testid} != "test_fef3" ] &&  [ ${testid} != "test_blank_wf" ] &&  [ ${testid} != "test_blank_ff" ] &&  [ ${testid} != "test_minrmf" ] && [ ${testid} != "test_same_outfiles" ]; then
    dmdiff $OUTDIR/${testid}_wght.fits $SAVDIR/${testid}_wght.fits tol=$SAVDIR/tolerance error_on_comment=no > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
	echo "ERROR: MISMATCH in $OUTDIR/${testid}_wght.fits" >> $LOGFILE
	mismatch=0
    fi
fi

if [ ${testid} != "test_fef3" ]; then
    CALDB=${orig_caldb}
fi


  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
