#===============================================================================
# File: regtest_utils.pl
#
# 1) Defines TestCase structure to hold specs for individual test runs.
# 2) Provides a set of utility sub-routines for use in Regression Test scripts.
#     + compare_files_diff()
#     + compare_files_dmdiff()
#     + compare_files_link()
#     + compare_files_imgcompare()
#     + file_exists()
#     + filter_text_file()
#     + find_in_path()
#     + gen_test_dirs()
#     + get_test_dirs()
#     + print_test_summary()
#     + run_test_suite()
# 3) Provides infrastructure routine to setup and run a suite of TestCase-s
#     + do_standard_test()
#===============================================================================
use strict;
use vars '$DIFF', '$DMDIFF', '$COMPARE', '$CONVERT', '$FILE';

use Class::Struct;
#===============================================================================
# Structure for Test Cases (individual test runs)
#   id      = Test id
#   type    = Test type; "unit" or "regression"
#   desc    = Short description of test purpose
#   mksubdir= Boolean flag for creating test subdirectory (default=0)
#   command = Command string
#   verify  = Hash of validation criteria. Typically files to compare.
#              key   = <file>
#              value = compare method "diff", "dmdiff", "dmimgcalc"
#             Special pair for expected failure
#              key='fail' value=<errno>
#   filters = Array of user-defined substitutions for text file verification.
#             These are applied in addition to the standard substitution set.
#              see filter_text_file() for format expectations.
#   imgargs = A string list of arguments to be used in ImageMagick convert and 
#             compare commands, specifically for compare_files_imgcompare() 
#             (See http://www.imagemagick.org/script/command-line-options.php)
#   grade   = Result of executing test and comparing results (PASS|FAIL)
#
#===============================================================================
struct( TestCase => { id       => '$',
		      type     => '$',
		      desc     => '$',
		      mksubdir => '$',
		      command  => '$',
		      verify   => '%',
		      filters  => '@',
	              imgargs  => '$',
		      grade    => '$',
	            } 
      );

# ==============================================================================
# Subroutine definitions
# ==============================================================================
# Locate system executables used by these subroutines.
#
$DIFF    = find_in_path( 'diff', $ENV{'PATH'}, ':' );
$DMDIFF  = find_in_path( 'dmdiff', $ENV{'PATH'}, ':' );
$COMPARE = find_in_path( 'compare', $ENV{'PATH'}, ':' );
$CONVERT = find_in_path( 'convert', $ENV{'PATH'}, ':' );
$FILE    = find_in_path( 'file', $ENV{'PATH'}, ':' );


# ------------------------------------------------------------------------------
# compare_files_diff( fileA, fileB )
#   use 'diff' to compare 2 text files.
#
#   Arguments
#     fileA: full path to file 1.
#     fileB: full path to file 2.
#
#   Returns
#     ndiffs: number of differences 
#             (-1 if file 1 is not found)
#             (-2 if file 2 is not found)
# ------------------------------------------------------------------------------
sub compare_files_diff
{
  my ( $fileA, $fileB ) = @_ ;

  my $line;
  my $ndiff;

  #Check that both files exist
  if ( ! file_exists( $fileA ) ){ return(-1); }
  if ( ! file_exists( $fileB ) ){ return(-2); }
  
  $ndiff = 0;
  open(TSTHND,"$DIFF ${fileA} ${fileB} |");
  while ( $line = <TSTHND> )
  {
       $ndiff++;
  }
  close(TSTHND);
  return($ndiff);
}

# ------------------------------------------------------------------------------
# compare_files_dmdiff( fileA, fileB, tolfile )
#   use 'dmdiff' to compare 2 files.
#
#   Arguments
#     fileA: full path to file 1.
#     fileB: full path to file 2.
#     tolfile: full path to tolerance file. ('' for none )
#     diffile: full path to write diffs.    ('' for none )
#
#   Returns
#     0 if no differences. 
#     1 if differences found.
#    -1 if file 1 is not found
#    -2 if file 2 is not found
# ------------------------------------------------------------------------------
sub compare_files_dmdiff
{
  my ( $fileA, $fileB, $tolfile, $diffile ) = @_ ;

  my $comstr;
  my $ndiff = 0;

  #Check that both files exist
  if ( ! file_exists( $fileA ) ){ return(-1); }
  if ( ! file_exists( $fileB ) ){ return(-2); }

  # Generate dmdiff command.
  my $comstr = "$DMDIFF \"${fileA}\" \"${fileB}\" ";
  if (($tolfile ne "") && (-e "$tolfile") )
  {
    $comstr .= "tol=$tolfile ";
  }
  if ( $diffile ne "" )
  {
    $comstr .= "verbose=1 > $diffile 2>>/dev/null ";
  }
  else
  {
    $comstr .= "verbose=0 2>>/dev/null ";
  }

  # Execute
  my $rc = system($comstr);
  if ( $rc != 0 ){ $ndiff = 1; }

  return($ndiff);
}


# ------------------------------------------------------------------------------
# compare_files_imgcompare( fileA, fileB, imagargs )
#   use ImageMagick 'compare' to compare 2 image files.
#
#   Arguments
#     fileA: full path to image file 1.
#     fileB: full path to image file 2.
#        **NOTE:  Acceptable image formats are jpeg, png, and postscript
#     imgargs:  arguments for the convert|compare command-line string.
#               See http://www.imagemagick.org/script/command-line-options.php
#               for further details.
#   Returns
#     0 if no differences. 
#     1 if differences found.
#    -1 if file 1 is not found or not in valid format
#    -2 if file 2 is not found or not in valid format
# ------------------------------------------------------------------------------
sub compare_files_imgcompare
{
  my ( $fileA, $fileB, $imgargs ) = @_ ;

  my $comstr;
  my $ndiff = 0;

  #Check that both files exist
  if ( ! file_exists( $fileA ) ){ return(-1); }
  if ( ! file_exists( $fileB ) ){ return(-2); }

  #Check that both files are in the correct format
  my $typeA = `$FILE ${fileA}`;
  my $typeB = `$FILE ${fileB}`;

  unless ($typeA =~ /JPEG/i || $typeA =~ /PNG/i || $typeA =~ /PostScript/i) {
      print "The file ${fileA} is not a valid JPEG, PNG, or POSTSCRIPT.";
      return(-1);
  }
  unless ($typeB =~ /JPEG/i || $typeB =~ /PNG/i || $typeB =~ /PostScript/i) {
      print "The file ${fileB} is not a valid JPEG, PNG, or POSTSCRIPT.";
      return(-2);
  }

  #Check that both files are in the same format
  if ($typeA != $typeB) {
      print "Input images are not in the same format.";
      return(1);
  }

  # Generate ImageMagick convert|compare command string
  my $comstr = '';
 
  if ($imgargs) {
      #Time-stamped images need to be cropped before being compared:
      #e.g. $comstr .= "$CONVERT ${fileA} ${fileB} -crop 760x450+0-125 +repage miff:- | $COMPARE -metric AE - null:";

      $comstr .= "$CONVERT ${fileA} ${fileB} ${imgargs} | $COMPARE -metric AE - null:";
  }
  else
  {
      $comstr .= "$COMPARE -metric AE \"${fileA}\" \"${fileB}\" null:";
  }

  # Execute
  my $rc = qx($comstr 2>&1);

  if ( $rc != 0 )
  { 
      $ndiff = 1; 
  }

  return($ndiff);
}



# ------------------------------------------------------------------------------
# compare_files_link( fileA, fileB )
#   Test that fileA is a link.
#   If fileB is given, checks link target matches fileB.
#
#   Arguments
#     fileA: full path to link source.
#     fileB: full path to link target. ("" to bypass)
#
#   Returns
#     0  if fileA is a link to fileB
#     1  if fileA is NOT a link
#     2  if fileA is not a link to fileB
#    -1  if fileA is not found
# ------------------------------------------------------------------------------
sub compare_files_link
{
  my ( $fileA, $fileB ) = @_ ;

  use Cwd 'abs_path';

  my $target;
  my $result = 0;


  #Check that file exists
  if ( ! file_exists( $fileA ) ){ $result = -1; }

  if ( ! -l "$fileA" ){ $result = 1; }

  $target = abs_path( $fileA );

  if (( $fileB ne "") && ( $fileB ne $target ) ){ $result = 2; }
  
  return($result);
}

# ------------------------------------------------------------------------------
# file_exists( file )
#   Test that file exists.
#
#   Removes any extended DM filter syntax, 
#   and checks for file existance
#
#   Arguments
#     file: full path to file.
#
#   Returns
#     1  if file exists
#     0  if file does NOT exist
# ------------------------------------------------------------------------------
sub file_exists
{
  my ( $file ) = @_ ;
  my $tmpstr;
  my $result = 0;

  # strip extended DM syntax, if any
  ($tmpstr = $file) =~ s/(.+)\[+.*/$1/ ;

  # check file existance
  if ( -e "$tmpstr" ){ $result = 1; }

}

# ------------------------------------------------------------------------------
# filter_text_file( ifile, ofile, filters )
#   Apply filter substitutions to file.  Standard substitutions for several
#   environment variables are made automatically.  Additional filters may be
#   provided using the 'filters' argument.
#
#   Arguments
#     ifile:   full path to input file.
#     ofile:   full path to output file.
#     filters: set of user-defined substitutions to apply.
#
#   Returns
#     none
#
#   Note:
#     Filter items are provides as strings with the following syntax:
#       "<cmd> <source string> -> <dest string>"
#     where <cmd> is
#       'sub' = substitute instances of source string with dest string
#       'del' = delete lines containing instances of source string.
#
# ------------------------------------------------------------------------------
sub filter_text_file
{
  my ( $inFile, $outFile, @userFilt ) = @_ ;

  my $line;
  my $item;

  #Set up array of substitutions to make
  #Note: a regular array is used so that the substitutions
  #      will happen in the order specified.
  my @actions = (  "sub $ENV{'TESTLOG'} -> <TESTLOG>",
		   "sub $ENV{'TESTIN'}  -> <TESTIN>" ,
		   "sub $ENV{'TESTOUT'} -> <TESTOUT>",
		   "sub $ENV{'TESTSAV'} -> <TESTSAV>",
		   "sub $ENV{'CALDB'}   -> <CALDB>"  ,
		   "sub $ENV{'PWD'}     -> <PWD>"    ,
		   "sub $ENV{'USER'}    -> <USER>"   ,
		   );

  #Add any user specified actions.
  if ( $#userFilt >= 0 )
  {
    push( @actions, @userFilt );
  }

  #Open in/out files
  unless( open(INHND, "< $inFile") )
  { print "Cannot open input file $inFile \n"; return; }
  unless( open(OUTHND, "> $outFile") )
  { print "Cannot open output file $outFile\n"; return; }

  #Loop through input file and make the actions
  while ( $line = <INHND> )
  {
    my $skipit = 0;
    my $comm;
    my $rest;
    my $from;
    my $to;
    foreach $item ( @actions )
    {
      $rest = "";
      ($comm, $rest) = split(/\s+/, $item, 2 );

      if ( $comm eq "sub" )
      {
	($from, $to) = split(/\s+->\s+/, $rest, 2 );
	$line =~ s/$from/$to/g ;
      }
      elsif ( $comm eq "del" )
      {
	if ( $line =~ $rest )
	{
	  $skipit++;
	}
      }

    }

    ($skipit == 0 ) && (print OUTHND "$line");
  }

  #Close files
  close(INHND);
  close(OUTHND);

}

# ------------------------------------------------------------------------------
# find_in_path
#   Searches input path list for specified executable
#
#   Arguments
#     tool:  nname of executable to locate in PATH
#     path:  string of PATH locations to search
#     delim: Optional, delimeter of path string records
#               default = ' '
#
#   Returns
#     loc: Location of first occurance.
#
# Note:
#   This function is snagged from the DM test scripts (test_utils.pl)
#
# ------------------------------------------------------------------------------
sub find_in_path
{
  my ($file, $pathStr, $delim);
  my $result = "";

  if(scalar @_ == 2) 
  {
    ($file, $pathStr) = @_;
    $delim = ' ';
  }
  else 
  {
    ($file, $pathStr, $delim) = @_;
  }
   
  my @pathArr = split(/$delim/, $pathStr);

  foreach my $path (@pathArr) 
  {
    if(-e "$path/$file") 
    {
      $result = "$path/$file";
      last;
    }
  }

  return $result;
}

# ------------------------------------------------------------------------------
# gen_test_dirs()
#   Creates output Test directories locations.
#
#   Arguments
#     outdir  TESTOUT directory
#     logdir  TESTLOG directory
#
#   Returns
#     none
# ------------------------------------------------------------------------------
sub gen_test_dirs
{
  my ($outdir, $logdir) = @_ ;

  my $MKDIR = find_in_path("mkdir", $ENV{'PATH'}, ':');
  if (! -d $outdir ) 
  {
    system("$MKDIR -p $outdir");
  }
  
  if (! -d $logdir ) 
  {
    system("$MKDIR -p $logdir");
  }

}

# ------------------------------------------------------------------------------
# get_test_dirs()
#   Gets Test related directory locations from user Environment.
#
#   Arguments
#     subdir  additional sub-directory from base [Optional]
#
#   Returns
#     dirs:  simple array of directories in order
#             TESTIN, TESTOUT, TESTSAV, TESTLOG
# ------------------------------------------------------------------------------
sub get_test_dirs
{

  my ($subdir) = @_ ;
  my ($indir, $outdir, $savdir, $logdir);

  my $user = $ENV{'USER'};

  # Define directories for input/output
  $indir = $ENV{'TESTIN'};
  if ( $indir eq "" ) {
      die "get_test_dirs() could not find environment variable for TESTIN\n";
  }

  $outdir = $ENV{'TESTOUT'};
  if ( $outdir eq "" ) {
      die "get_test_dirs() could not find environment variable for TESTOUT\n";
  }

  $savdir = $ENV{'TESTSAV'};
  if ( $savdir eq "" )
  {
      die "get_test_dirs() could not find environment variable for TESTSAV\n";
  }

  $logdir = $ENV{'TESTLOG'};
  if ( $logdir eq "" )
  {
      die "get_test_dirs() could not find environment variable for TESTLOG\n";
  }

  my @dirlist;
  $dirlist[0] = $indir;
  $dirlist[1] = $outdir;
  $dirlist[2] = $savdir;
  $dirlist[3] = $logdir;


  if ($subdir ne "" )
  {
      $dirlist[0] .= "/$subdir";
      $dirlist[1] .= "/$subdir";
      $dirlist[2] .= "/$subdir";
      $dirlist[3] .= "/$subdir";
  }


  return(@dirlist);
}

# ------------------------------------------------------------------------------
# print_test_summary()
#   Print results of test suite.
#
#   Arguments
#     logfile   Test log file.
#     outdir    Output directory of results.
#
#   Returns
#     none
# ------------------------------------------------------------------------------
sub print_test_summary
{
  my ( $tool, $logfile, $outdir, $testref ) = @_ ;
  my @testarr = @{ $testref };

  # Check test results..
  my $anyprobs = 0;
  my $failures = "";
  my $test;

  if ( $#testarr < 0 )
  {
      $anyprobs++;
      $failures .= "No Tests Defined";
  }
  else
  {
    foreach $test (@testarr) 
    {
      my $id = $test->id;
      my $grade = $test->grade;
      if ( $grade eq "FAIL" )
      {
        $failures .= "$id ";
        $anyprobs++;
      }
    }
    $failures =~ s/test//g;
  }

  # Print summary 
  print " Tests complete\n";
  print " Log file in ${logfile} \n";
  print " Output location...\n";
  print "   $outdir\n";
  if ( $anyprobs == 0 )
  {
     print STDOUT " ${tool} : PASS \n";
  }
  else
  {
     print STDOUT " ${tool} : FAIL : $failures\n";
  }


}

# ------------------------------------------------------------------------------
# run_test_suite()
#   Execute tests
#    + populate TestCase.grade with status of test (PASS|FAIL)
#
#   Arguments
#     outdir    output directory for results.
#     savdir    baseline directory.
#     testref   reference to array of TestCase structures.
#
#   Returns
#     rc        return code
#               0 = all tests passed.
#              !0 = some test failures.
# ------------------------------------------------------------------------------
sub run_test_suite
{
  my ( $outdir, $savdir, $testref ) = @_ ;
  my @testarr = @{ $testref };

  my $MKDIR = find_in_path("mkdir", $ENV{'PATH'}, ':');

  my $anyprobs = 0;
  my $comstr = "";
  my $test;
  my $rc;

  print STDOUT "\tRunning tests ....\n";
  foreach $test (@testarr) 
  {
    my $testid = $test->id;
    print STDOUT "${testid}:\n";
    print STDOUT "  \'", $test->desc, "\'\n";

    my $outpath = $outdir;
    my $savpath = $savdir;
    if ( $test->mksubdir )
    {   # Add test subdirectory to paths
	$outpath .= "/${testid}";
	$savpath .= "/${testid}";
    }

    # create subdirectory for test output
    system("$MKDIR -p ${outpath}");
  
    # Execute test command
    $comstr = $test->command;
    if ($comstr ne "" )
    {
      print STDOUT "  COMSTR = $comstr \n";
      $rc = system($comstr);
    }
  
    # Validate Results
    my $count = 0;
    my $item;
    for $item (sort keys %{ $test->verify } )
    {
      if ($item eq "fail")
      {
        if ($rc ne $test->verify('fail') )
        {
          print STDOUT "  Expected failure did not occur, (rc=$rc).\n";
          $count++;
        }
        else
        {
          print STDOUT "  Expected failure occurred, (rc=$rc).\n";
        }
      }
      else
      {
        my $file = $item;
        my $method = $test->verify($item);
        my $outfile = "${outpath}/$item";
        my $savfile = "${savpath}/$item";
        my $tolfile = "${savpath}/tolerance";
        my $diffile = "${outfile}.diff";
  
        # Compare against baseline. 
        my $ndiffs = 0;
        if ( $method eq "dmdiff" )
        {
          # Compare using dmdiff
          $ndiffs = &compare_files_dmdiff( $savfile, $outfile, $tolfile, $diffile );
        }
        elsif ( $method eq "dmimgcalc" )
        {
          print STDOUT "Compare using dmimgcalc not yet implemented.\n";
        }
        elsif ( $method eq "diff" )
        {
          my $filtref  = $test->filters;
	  my @userFilt = @{ $filtref };

	  # Apply filters to both files
	  my $outfilt = "${outpath}/${item}_outfilt";
	  my $savfilt = "${outpath}/${item}_savfilt";
	  filter_text_file( $outfile, $outfilt, @userFilt );
	  filter_text_file( $savfile, $savfilt, @userFilt );

          # Compare using diff
          $ndiffs = &compare_files_diff( $savfilt, $outfilt );
        }
        elsif ( $method eq "link" )
        {
          # Test file link
          $ndiffs = &compare_files_link( $outfile, "" );
        }
          elsif ( $method eq "imgcompare" )
        {
          # Test image
          $ndiffs = &compare_files_imgcompare( $savfile, $outfile, $test->imgargs);
        }

        # Report comparison results
        if ( $ndiffs == -1 )
        {
          print STDOUT "ERROR: savfile does not exist: $savfile\n";
          $count++;
        }
        elsif ( $ndiffs == -2 )
        {
          print STDOUT "ERROR: outfile does not exist: $outfile\n";
          $count++;
        }
        elsif ( $ndiffs > 0 )
        {
          printf STDOUT "%33s ... DIFFERS\n", $file;
          $count++ ;
        }
        else
        {
          printf STDOUT "%33s ... OK\n", $file;
        }
      }
    }
    if ( $count > 0 )
    {
      # Keep track of tests with problems.
      $test->grade("FAIL");
      $anyprobs += $count;
    }
    else
    {
      $test->grade("PASS");
    }
    print STDOUT "\n";
  }

  return $anyprobs;
}

# ------------------------------------------------------------------------------
# do_standard_test()
#   Encapsulates a standard thread for setting up and running a set of TestCases.
#   + setup test directories 
#   + prep to run test cases, setup log file
#   + execute test cases, accumulate resulting grades
#   + print test summary
#   + cleanup 
#
#   Arguments
#     tool      Name of tool/package to be tested
#     outdir    Test output directory
#     logdir    Test log directory
#     savdir    Test baseline directory
#     testref   reference to array of TestCase structures to be processed.
#
#   Globals
#
#   Returns
#     none
# ------------------------------------------------------------------------------
sub do_standard_test
{
  my ( $tool, $outdir, $logdir, $savdir, $testref ) = @_ ;
  my @testlist = @{ $testref };
  my @filters = [];

  my $date = "";
  my $logfile = "";

  #----------------------------------------------------------------------
  # Find System utilities 
  #----------------------------------------------------------------------
  my $RM    = find_in_path("rm", $ENV{'PATH'}, ':');
  my $RMDIR = find_in_path("rmdir", $ENV{'PATH'}, ':');
  
  #----------------------------------------------------------------------
  # Prepare to run tests.
  #----------------------------------------------------------------------
  printf("\t %s REGRESSION TEST\n", uc($tool));
  
  #set env variables
  $ENV{"ASCDS_VERSION"}="TEST";
  
  #Get the date for log name
  $date=`date +%d%b%Y_%H:%M:%S`;
  chop($date);
  
  #Define files
  $logfile="${tool}_log.${date}";
  
  #Clear out old output files
  system("${RM} -rf ${outdir}/test* ");
  system("${RM} -f ${outdir}/* ");
  
  #----------------------------------------------------------------------
  # Run Test(s)
  #----------------------------------------------------------------------
  #Open log file to contain run dump
  #  capture stdout and stderr msgs.
  #
  open(SAVOUT,">&STDOUT");
  open(SAVERR,">&STDERR");
  open(STDOUT, "> ${logdir}/${logfile}");
  open(STDERR, ">&STDOUT");
  
  
  my $anyprobs = run_test_suite( $outdir, $savdir, \@testlist );
  
  
  # Close Log
  close(STDOUT);
  close(STDERR);
  open(STDOUT,">&SAVOUT");
  open(STDERR,">&SAVERR");
  
  #----------------------------------------------------------------------
  # Print Summary
  #----------------------------------------------------------------------
  #
  print_test_summary( $tool, "${logdir}/${logfile}", $outdir, \@testlist );
  
  #----------------------------------------------------------------------
  # Clean up
  #----------------------------------------------------------------------
  if ( $anyprobs == 0 )
  {
     # If PASS, clean out current results
     unlink("${logdir}/${logfile}");
     system("$RM -rf ${outdir}/test* ");
     system("$RM -f ${outdir}/* ");
     system("$RMDIR ${outdir}");
  }
}
1;
