#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# mytool.t
# test script for mytool


# !!2
# syntax:
# mytool.t [<testid> ... ]
 
# Developer's comments:
# lack of comparison of the higher extensions of outputs (FITS)
#
######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/WasteDataset/g;s/COMMENT/WasteDataset/g;
  s/DATE/WasteDataset/g;s/CREATOR/WasteDataset/g;s/HISTORY/WasteDataset/g; 
  s/DATASUM/WasteDataset/g;s/ASCDSVER/WasteDataset/g;s/HISTNUM/WasteDataset/g; 
  s/CVSD0001/WasteDataset/g; s/CVST0001/WasteDataset/g' | \
  grep -v WasteDataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable
find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}
######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="skyfov"
methodlist="minmax convexhull"
verbosity=0

# set up list of tests
# !!4
alltests="skyfov_id214 skyfov_id1450 skyfov_id1700 skyfov_id1905 skyfov_filt568 skyfov_filt7 skyfov_id2433 skyfov_nom skyfov_msk skyfov_obi skyfov_sra skyfov_mult_asol skyfov_id11864 skyfov_id13659 skyfov_staticRoll skyfov_no_asp_offsets skyfov_null_gtis skyfov_chip_ids skyfov_chip_ids_range skyfov_ccd_ids skyfov_msk_chip_ids skyfov_msk_ccd_ids skyfov_no_asp_rows skyfov_subspace_mults skyfov_subspace_mults_filt"
#alltests="skyfov_nom"
# "short" test to run
# !!5
shortlist="skyfov_nom"
iotests="skyfov_no_pnt skyfov_mismatch_obsids skyfov_no_row_no_pnt skyfov_no_valid_aspect"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*


# Any tests specified on command line?
# If so, need to sort out which are the regular tests
# and which are the I/O tests

if test $# -gt 0; then
  # yes, do those tests
  for item in $*
  do  
    for all in $alltests
    do
      if [ $item = $all ]; then
        testlist=$testlist$item
      fi
    done
    for io in $iotests
    do
      if [ $item = $io ]; then
        iotestlist=$iotestlist$item
      fi
    done
  done 
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
    iotestlist=$iotests
  else
    # yes, do short test
    testlist=$shortlist
    iotestlist=$shortlist
  fi
fi

# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


for method in ${methodlist}
do
  # Make sure we have an output directory
  if test -d $OUTDIR/${method} ; then
   :
  else
    mkdir -p $OUTDIR/${method} >> $LOGFILE 2>&1
    if test $? -ne 0 ; then
      error_exit "can't create output directory $OUTDIR"
    fi
  fi
done

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo "------------------------------------"
echo "   ${toolname} regression" | tee $LOGFILE
echo "------------------------------------"
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

#case "$OSTYPE" in
  #solaris*) echo "SOLARIS" ;;
  #darwin*)  echo "OSX" ;; 
  #linux*)   echo "LINUX" ;;
  #bsd*)     echo "BSD" ;;
  #msys*)    echo "WINDOWS" ;;
  #*)        echo "unknown: $OSTYPE" ;;
#esac

######################################################################
# Begin I/O and Warning per-test loop
# These tests are the same for minmax or convexhull methods.  Testing the I/O and consistency
# so there is no need to run them twice as for the second loop further below in this script.

for testid in $iotestlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${method}/${testid}*

  # Set up file names
  outfile=$OUTDIR/${method}/${testid}.fits
  savfile=$SAVDIR/${method}/${testid}.fits

  echo ""
  echo "running $testid" >> $LOGFILE
  cmd_status=0

  ####################################################################
  # run the tool
  case ${testid} in

    # testing aspect=file_does_not_exist.
    skyfov_no_valid_aspect) command_str="${toolname} \
    infile=${INDIR}/acisf01198_000N002_evt1.fits\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/file_does_not_exist.fits \
    verbose=$verbosity mskfile=none clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # Testing mismatch obs ids between event file and MSK & ASOL files 
    skyfov_mismatch_obsids ) command_str="${toolname} \
    infile=${INDIR}/acisf01198_000N002_evt1.fits\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect='${INDIR}/pcadf1674*N001_asol1.fits' \
    mskfile=${INDIR}/acisf04383_002N001_msk1.fits \
    verbose=0 clobber=yes> /dev/null" 
    if [ $OSTYPE != *"ubuntu"* ]; then
      echo "$command_str >> $LOGFILE 2>&1 &" 
      # Preform the eval of the command and append output to log
      eval $command_str >> $LOGFILE 2>&1 &
      # eval again allowing access to command output status through $?. 
      # The skyfov error is expected to return 255, and if so, then the test will pass. 
      eval $command_str 
      cmd_status=`expr $? - 255`
    else
      echo $command_str  | tee -a $LOGFILE
      eval "$command_str  | tee -a $LOGFILE  2>&1"
    fi
    ;;

    # Testing no _PNT values in header infile 
    # and no aspect input which produces an error
    skyfov_no_pnt ) command_str="${toolname} \
    infile=${INDIR}/acisf04383N001_evt2.fits  \
    outfile=$outfile kernel=fits logfile=\"\" \
    mskfile=${INDIR}/acisf04383_002N001_msk1.fits \
    verbose=0 clobber=yes> /dev/null" 
    if [ $OSTYPE != *"ubuntu"* ]; then
      echo "$command_str >> $LOGFILE 2>&1 &" 
      # Preform the eval of the command and append output to log
      eval $command_str >> $LOGFILE 2>&1 &
      # eval again allowing access to command output status through $?. 
      # The skyfov error is expected to return 255, and if so, then the test will pass. 
      eval $command_str 
      cmd_status=`expr $? - 255`
    else
      echo $command_str  | tee -a $LOGFILE
      eval "$command_str  | tee -a $LOGFILE  2>&1"
    fi
    ;;

    # Testing no rows in aspect file and no _PNT 
    # values in header infile which produces an error
    skyfov_no_row_no_pnt ) command_str="${toolname} \
    infile=${INDIR}/acisf04383N001_evt2.fits  \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect='${INDIR}/pcadf1674*N001_asol1.fits[#row=0]' \
    mskfile=${INDIR}/acisf04383_002N001_msk1.fits \
    verbose=0 clobber=yes> /dev/null" 
    if [ $OSTYPE != *"ubuntu"* ]; then
      echo "$command_str >> $LOGFILE 2>&1 &" 
      # Preform the eval of the command and append output to log
      eval $command_str >> $LOGFILE 2>&1 &
      # eval again allowing access to command output status through $?. 
      # The skyfov error is expected to return 255, and if so, then the test will pass. 
      eval $command_str 
      cmd_status=`expr $? - 255`
    else
      echo $command_str  | tee -a $LOGFILE
      eval "$command_str  | tee -a $LOGFILE  2>&1"
    fi
    ;;

 esac

  # Init per-test error flag
  mismatch=1

  if test ${cmd_status} -ne 0 ; then
    echo "ERROR: Either BAD CMD STATUS (${cmd_status}) is not zero, or MISMATCH in $outfile" >> $LOGFILE
    echo "ERROR: Either BAD CMD STATUS or MISMATCH in " >> $LOGFILE
    mismatch=0
  fi

  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# End I/O and Warning per-test loop
 
######################################################################
# Begin per-test loop

for method in $methodlist
do

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${method}/${testid}*

  # Set up file names
  outfile=$OUTDIR/${method}/${testid}.fits
  savfile=$SAVDIR/${method}/${testid}.fits

  echo ""
  echo "running $testid" >> $LOGFILE
  cmd_status=0

  ####################################################################
  # run the tool
  case ${testid} in

    # testing aspect with no rows. Pickup offsets and pointing from infile.
    skyfov_no_asp_rows) command_str="${toolname} \
    infile=${INDIR}/acisf16136N002_evt2.fits.gz\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect='${INDIR}/pcadf498642854N002_asol1.fits.gz[#row=0]' \
    mskfile=${INDIR}/acisf16136_001N002_msk1.fits.gz \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # testing aspect=none. Pickup offsets from infile (DY_AVG, DZ_AVG, DTH_AVG).
    skyfov_no_asp_offsets) command_str="${toolname} \
    infile=${INDIR}/acisf16136N002_evt2.fits.gz\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect="" \
    mskfile=${INDIR}/acisf16136_001N002_msk1.fits.gz \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # testing with null gtis - skyfov used to fail silently.  Now warn that ASOL is 
    # unused and proceed without asol input.  
    # Command used to exit quietly with an error code and now we monitor it.
    skyfov_null_gtis) command_str="${toolname} \
    infile=${INDIR}/null_gtis/acisf00963_001N004_evt1.fits\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect='@${INDIR}/null_gtis/asol.lis[@${INDIR}/null_gtis/acisf00963_001N004_flt1.fits]' \
    method=${method} verbose=$verbosity mskfile=none clobber=yes> /dev/null"
    echo $command_str  | tee -a $LOGFILE
    eval "$command_str  | tee -a $LOGFILE  2>&1"
    ;;

    # testing specification of ACIS ccd ids to use - without mask file.
    skyfov_ccd_ids) command_str="${toolname} \
    infile=${INDIR}/acisf01198_000N002_evt1.fits'[ccd_id=7,8]'\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/pcadf063832996N002_asol1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # testing specification of ACIS ccd ids to use - with mask file.
    skyfov_msk_ccd_ids) command_str="${toolname} \
    infile=${INDIR}/acisf01198_000N002_evt1.fits'[ccd_id=7,8]'\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/pcadf063832996N002_asol1.fits \
    mskfile=${INDIR}/acisf01198_000N002_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # testing specification of HRC chip ids to use - without mask file.
    skyfov_chip_ids) command_str="${toolname} \
    infile=${INDIR}/16595/hrcf16595N002_evt2.fits.gz'[chip_id=1,3]'\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/16595/pcadf509538560N002_asol1.fits.gz 
    method=${method} verbose=$verbosity mskfile=none clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # testing specification of HRC chip ids to use - without mask file.
    skyfov_chip_ids_range) command_str="${toolname} \
    infile=${INDIR}/16595/hrcf16595N002_evt2.fits.gz'[chip_id=1:2]'\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/16595/pcadf509538560N002_asol1.fits.gz 
    method=${method} verbose=$verbosity mskfile=none clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # testing specification of HRC chip ids to use - with mask file.
    skyfov_msk_chip_ids) command_str="${toolname} \
    infile=${INDIR}/16595/hrcf16595N002_evt2.fits.gz'[chip_id=1,3]'\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/16595/pcadf509538560N002_asol1.fits.gz 
    mskfile=${INDIR}/16595/hrcf16595_000N002_msk1.fits.gz
    method=${method} verbose=$verbosity mskfile=none clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # testing infile=event  without mask file
    skyfov_nom) command_str="${toolname} \
    infile=${INDIR}/acisf01198_000N002_evt1.fits\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/pcadf063832996N002_asol1.fits \
    method=${method} verbose=$verbosity mskfile=none clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
            ;;

    # testing infile=event with mask file
    skyfov_msk ) command_str="${toolname} \
    infile=${INDIR}/acisf01198_000N002_evt1.fits\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/pcadf063832996N002_asol1.fits\
    mskfile=${INDIR}/acisf01198_000N002_msk1.fits \
    method=${method} verbose=$verbosity> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

    # testing infile=obi_par file without mask file
    skyfov_obi ) command_str="${toolname} \
    infile=${INDIR}/acisf02489_001N001_obs0a_modified.par\
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/pcadf063832996N002_asol1.fits\
    method=${method} verbose=$verbosity mskfile=none> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing multiple pcad*asol inputs 
    skyfov_mult_asol ) command_str="${toolname} \
    infile=${INDIR}/934/acisf00934N004_evt2.fits  \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/934/pcadf0704*N004_asol1.fits\
    mskfile=${INDIR}/934/acisf00934_000N004_msk1.fits \
    method=${method} verbose=0 clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing small aspect roll angle
    skyfov_sra ) command_str="${toolname} \
    infile=${INDIR}/acisf04383N001_evt2.fits  \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/pcadf1674*N001_asol1.fits\
    mskfile=${INDIR}/acisf04383_002N001_msk1.fits \
    method=${method} verbose=0 clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing mask-off all but acis-7 chip
    skyfov_id214 ) command_str="${toolname} \
    infile=${INDIR}/214/acisf00214N003_evt2.fits \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/214/pcadf051495313N003_asol1.fits\
    mskfile=${INDIR}/214/acisf00214_000N003_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing mask-off all but acis-7 chip
    skyfov_id1450 ) command_str="${toolname} \
    infile=${INDIR}/1450/acisf01450N001_evt2.fits \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/1450/pcadf056511497N002_asol1.fits\
    mskfile=${INDIR}/1450/acisf01450_000N002_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing mask-off all but acis-7 chip
    skyfov_id1700 ) command_str="${toolname} \
    infile=${INDIR}/1700/acisf01700N006_evt2.fits \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/1700/pcadf076825438N004_asol1.fits\
    mskfile=${INDIR}/1700/acisf01700_000N004_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;
   
    # testing mask-off all but acis-7 chip
    skyfov_id1905 ) command_str="${toolname} \
    infile=${INDIR}/1905/acisf01905N004_evt2.fits \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/1905/pcadf113046221N003_asol1.fits\
    mskfile=${INDIR}/1905/acisf01905_000N003_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing filtering of a complex mask - multiples on chip 7
    skyfov_filt568 ) command_str="${toolname} \
    infile=${INDIR}/1905/acisf01905N004_evt2.fits'[ccd_id=5,6,8]' \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/1905/pcadf113046221N003_asol1.fits\
    mskfile=${INDIR}/1905/acisf01905_000N003_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing filtering of a complex mask - multiples on chip 7
    skyfov_filt7 ) command_str="${toolname} \
    infile=${INDIR}/1905/acisf01905N004_evt2.fits'[ccd_id=7]' \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/1905/pcadf113046221N003_asol1.fits\
    mskfile=${INDIR}/1905/acisf01905_000N003_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing mask-off all but HRC-S1 chip
    skyfov_id2433 ) command_str="${toolname} \
    infile=${INDIR}/2433/hrcf02433N004_evt2.fits  \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/2433/pcadf106189827N002_asol1.fits\
    mskfile=${INDIR}/2433/hrcf02433_000N003_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing asol RA values wrap around the 0|360 RA border
    skyfov_id11864 ) command_str="${toolname} \
    infile=${INDIR}/11864/acisf11864N003_evt2.fits \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/11864/pcadf368717826N004_asol1.fits \
    mskfile=${INDIR}/11864/acisf11864_000N003_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing asol ROLL values wrap around the 0|360 ROLL border
    skyfov_id13659 ) command_str="${toolname} \
    infile=${INDIR}/13659/acisf13659N002_evt2.fits \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect='${INDIR}/13659/pcadf442270645N001_asol1.fits[time=442269964.9350900:442281863.1357200]' \
    mskfile=${INDIR}/13659/acisf13659_000N002_msk1.fits \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing static ROLL
    skyfov_staticRoll ) command_str="${toolname} \
    infile=${INDIR}/staticRoll/evt.fits \
    outfile=$outfile kernel=fits logfile=\"\" \
    aspect=${INDIR}/staticRoll/asol.fits\
    mskfile=\"\" \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing multiple ccd_ids in one subspace gti block.
    skyfov_subspace_mults ) command_str="${toolname} \
    infile=${INDIR}/subspace_mults/acisf06327N004_image_evt1bad_o0.fits \
    outfile=$outfile kernel=fits logfile=\"\" \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;

    # testing multiple ccd_ids in one subspace gti block and filter.
    skyfov_subspace_mults_filt ) command_str="${toolname} \
    infile=${INDIR}/subspace_mults/acisf06327N004_image_evt1bad_o0.fits'[ccd_id=2,5]' \
    outfile=$outfile kernel=fits logfile=\"\" \
    method=${method} verbose=$verbosity clobber=yes> /dev/null" 
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
    ;;


  esac


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  # dmlist $outfile header,data > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
  # dmlist $savfile header,data > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
             2>>$LOGFILE

  # compare
  # !!12
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > /dev/null 2>>$LOGFILE

  #echo "dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
  dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE

  if (test $? -ne 0); then
    echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
    mismatch=0
  fi


  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
done
# end of the two methods

######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"

exit $script_succeeded
