#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.


# sso_freeze.t
# test script for sso_freeze


# syntax:
# sso_freeze.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="sso_freeze"

# set up list of tests
alltests="test1 test2 test3 test4 test5 test6"

# "short" test to run
shortlist="test1 test2 test3 test4 test5 test6"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}_evt.fits
  outaspfile=$OUTDIR/${testid}_asp.fits
  savfile=$SAVDIR/${testid}_evt.fits
  savaspfile=$SAVDIR/${testid}_asp.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in

    test1 ) # basic test - single input file, angular output coordinates,
            #              multiple files in s/c ephem stack
	    #...Define input files, params 
	    infile="${INDIR}/sso_freeze_c1754_evt.fits"
	    scEphmIn="@${INDIR}/sso_freeze_comet_sc_ephem.lis"
	    ssoEphmIn="${INDIR}/sso_freeze_comet_ephem.fits"
	    test1_string="sso_freeze infile=$infile \
	     scephemfile=$scEphmIn ssoephemfile=$ssoEphmIn \
	     outfile=$outfile asolfile="" scale=0"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;
   test2 )  # testing multiple input files with sky{x,y} reprojection
	    #...Define input files, params 
	    infile="@${INDIR}/in_test2.lis"
	    scEphmIn="${INDIR}/orbitf124113900N001_eph1.fits"
	    ssoEphmIn="${INDIR}/asteroid_1998_wt24f124416300N001_eph1.fits"
	    test2_string="sso_freeze infile=$infile \
	     scephemfile=$scEphmIn ssoephemfile=$ssoEphmIn \
	     outfile=$outfile asolfile="" scale=0"
	    echo $test2_string | tee -a  $LOGFILE 
	    eval $test2_string  | tee -a  $LOGFILE  2>&1
            ;;
   test3 )  # testing multiple input files, no sky{x,y} reprojection
	    #...Define input files, params 
	    infile="@${INDIR}/in_test3.lis"
	    scEphmIn="${INDIR}/orbitf124113900N001_eph1.fits"
	    ssoEphmIn="${INDIR}/asteroid_1998_wt24f124416300N001_eph1.fits"
	    test3_string="sso_freeze infile=$infile \
	     scephemfile=$scEphmIn ssoephemfile=$ssoEphmIn \
	     outfile=$outfile asolfile="" scale=0"
	    echo $test3_string | tee -a  $LOGFILE 
	    eval $test3_string  | tee -a  $LOGFILE  2>&1
            ;;
   test4 )  # Testing linear coord output option
	    #...Define input files, params 
	    infile="${INDIR}/sso_freeze_c1754_evt.fits"
	    scEphmIn="${INDIR}/sso_freeze_comet_sc_ephem.fits"
	    ssoEphmIn="${INDIR}/sso_freeze_comet_ephem.fits"
	    test1_string="sso_freeze infile=$infile \
	     scephemfile=$scEphmIn ssoephemfile=$ssoEphmIn \
	     outfile=$outfile asolfile="" scale=-1"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;
   test5 )  # Testing SSO RA, dec output and OC aspect sol
	    #...Define input files, params 
	    infile="${INDIR}/test5_in.fits"
	    scEphmIn="${INDIR}/orbitf160142700N001_eph0.fits"
	    ssoEphmIn="${INDIR}/jupiterf162475200N001_eph1.fits"
            inaspfile="${INDIR}/test5_aspin.fits"
	    test1_string="sso_freeze infile=$infile \
	     scephemfile=$scEphmIn ssoephemfile=$ssoEphmIn \
	     asolfile=$inaspfile ocsolfile=$outaspfile \
	     outfile=$outfile scale=0"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;
   test6 )  # Testing SSO aspect sol without ocsolfile set (SAT-08)
	    infile="${INDIR}/test5_in.fits"
	    scEphmIn="${INDIR}/orbitf160142700N001_eph0.fits"
	    ssoEphmIn="${INDIR}/jupiterf162475200N001_eph1.fits"
            inaspfile="${INDIR}/test5_aspin.fits"
	    test1_string="sso_freeze infile=$infile \
	     scephemfile=$scEphmIn ssoephemfile=$ssoEphmIn \
	     asolfile=$inaspfile ocsolfile='' \
	     outfile=$outfile scale=0"
	    echo $test1_string | tee -a  $LOGFILE 
            eval $test1_string > $OUTDIR/${testid}_scrub.0.tmp 2>&1
            echo "status: " $? >> $OUTDIR/${testid}_scrub.0.tmp
            ;;

  esac

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  ####################################################################
  # FITS table

  # new output
  #dmlist $outfile opt=header,data,clean rows=700:1400 > \
  # $OUTDIR/${testid}.dmp1 2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2 \
  # 2>>$LOGFILE
  
  # reference output
  #dmlist $savfile opt=header,data,clean rows=700:1400 > \
  # $OUTDIR/${testid}.dmp1_std 2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  # 2>>$LOGFILE
  
  # compare
  if [ ${testid} = "test6" ]; then
    # handle tests which check failure cases and stdout/err
    # strip out paths and version strings from stdout
    sed -e "s;${INDIR};;g" $OUTDIR/${testid}_scrub.0.tmp | sed -e "s;${OUTDIR};;g" | sed -e "s;# sso_freeze (.*):;# sso_freeze:;g" > $OUTDIR/${testid}_output.txt
    diff $OUTDIR/${testid}_output.txt $SAVDIR/${testid}_output.txt > /dev/null 2>> ${LOGFILE}
    if  test $? -ne 0 ; then
      echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}_output.txt" >> $LOGFILE
      mismatch=0
    fi
  else 
    # handle regular succussful test cases 
    # 1. check evt file
    dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
      mismatch=0
    fi
  
    # 2. check asp file, if one was used
    if test -f $savaspfile
    then
      dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
      if  test $? -ne 0 ; then
        echo "ERROR: MISMATCH in $outaspfile" >> $LOGFILE
        mismatch=0
      fi
    fi  
  fi    # else if [ ${testid} = "test6" ]

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
