#! /bin/sh
#
# November, 2004

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# tg_resolve_events.t
# test script for tg_resolve_events


# !!2
# syntax:
# tg_resolve_events.t [<testid> ... ]


######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo ""   | tee -a $LOGFILE
  echo "$1" | tee -a $LOGFILE
  echo ""   | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/ROLL_RMS/Dataset/g;s/COMMENT/Dataset/g;
  s/CHECKSUM/Dataset/g; s/DATE/Dataset/g;s/CREATOR/Dataset/g;
  s/HISTORY/Dataset/g; s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;
  s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="tg_resolve_events"

# set up list of tests
# !!4
alltests="acishetg acisletg acisletg_1b acisletg_raw acisletg_2 acisletg_3 acisletg_4 acisletg_4_B hrcletg hrcletg_rawy hrcletg_rawx hrcletg_raw standard acishetgCC acishetg_2A acishetg_2B acishetg_3 acishetgCC_B acishetgCC_1stRun acishetgCC_2ndRun acishetgCC_2ndRun_B  eventdef_blank missing_inCol_1 missing_inCol_2 missing_inCol_3 missing_inCol_4 missing_inCol_5 hrcletg_samp_a hrcletg_samp_b acisletg_2_B hrcletg_sumamps_a hrcletg_sumamps_b hrci_hetg_overlap"


# "short" test to run
# !!5
shortlist="standard"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname
TOL=$SAVDIR/tolerance

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# Make sure we have an input directory
if test -d $INDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find input directory : $INDIR"
  fi
fi

# Make sure we have a save directory
if test -d $SAVDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find save directory : $SAVDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff

# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo "" | tee -a $LOGFILE


# ----- pset geom files -----
# pset geom instruments="$INDIR/telD1999-07-23geomN0002.fits"
# pset geom aimpoints="$INDIR/telD1999-07-23aimptsN0001.fits"
# pset geom tdet="$INDIR/telD1999-07-23tdetN0001.fits"
# pset geom sky="$INDIR/telD1999-07-23skyN0001.fits"

# ----- copy some files -----
cd ${OUTDIR}

script_succeeded=0

######################################################################
# Begin per-test loop

# grating_obs=HEADER_VALUE
# detector=HEADER_VALUE
# osort_lo=0.3 
# osort_hi=0.3 
# energy_lo_adj=1.0 
# energy_hi_adj=1.0 
# rand_seed=1 
# rand_pix_size=0.0 
# clobber=yes 
# geompar=geom
# verbose=0

for testid in $testlist
do
  # All parameters except verbose should be set anyway, but clear them
  # to be safe.
  bose=`pget $toolname verbose`
  punlearn $toolname
  pset $toolname verbose=$bose

  ####################################################################
  # Init per-test error flag
  #
  mismatch=1    

  # delete old outputs
  rm -f $OUTDIR/*${testid}_evt1a.fits

  # Set up file names
  infile="$INDIR/${toolname}_${testid}_evt1.fits"
  outfile="$OUTDIR/${toolname}_${testid}_evt1a.fits"
  regionfile="$INDIR/${toolname}_${testid}_reg1a.fits"
  logfile="$OUTDIR/${toolname}_${testid}_${DT}.log"

  savfile="$SAVDIR/${toolname}_${testid}_evt1a.fits"

  # default 
  osipfile="CALDB"
  eventdef='\)stdlev1_ACIS'
  stdlev1='\)eventdef'
  acaofffile="$INDIR/${toolname}_${testid}_aoff1.fits"

  #  upd_acis_evtdef  = eventdef l:pha_ro,f:chipy_tg,f:chipy_zo
  upd_acis_evtdef="\{d:time,i:expno,f:rd,s:chip,s:tdet,f:det,f:sky,s:ccd_id,l:pha,s:pi,f:energy,s:grade,s:fltgrade,s:node_id,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status,l:pha_ro,f:chipy_tg,f:chipy_zo\}"


  #  raw_acis_evtdef = stdlev_ACIS+l:raw ;  
  raw_acis_evtdef="\{d:time,i:expno,f:rd,s:chip,s:tdet,f:det,f:sky,s:ccd_id,l:pha,s:pi,f:energy,s:grade,s:fltgrade,s:node_id,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status,l:raw\}"

  #  OLD stdlev1_HRC  ( ie. without crsu,crsv,rawx,rawy,amp_sf,samp )
  old_stdlev1_HRC="\{d:time,f:rd,s:chip,l:tdet,f:det,f:sky,s:chip_id,s:pha,s:pi,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status\}"

  #  rawy_hrc_evtdef  = OLD stdlev_HRC + l:rawy
  rawy_hrc_evtdef="\{d:time,f:rd,s:chip,l:tdet,f:det,f:sky,s:chip_id,s:pha,s:pi,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status,l:rawy\}"

  #  rawx_hrc_evtdef  = OLD stdlev_HRC + l:rawx
  rawx_hrc_evtdef="\{d:time,f:rd,s:chip,l:tdet,f:det,f:sky,s:chip_id,s:pha,s:pi,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status,l:rawx\}"

  #  raw_hrc_evtdef  = OLD stdlev_HRC + l:raw
  raw_hrc_evtdef="\{d:time,f:rd,s:chip,l:tdet,f:det,f:sky,s:chip_id,s:pha,s:pi,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status,l:raw\}"

  #  amp_sf_hrc_evtdef  = OLD stdlev_HRC + s:amp_sf
  amp_sf_hrc_evtdef="\{d:time,f:rd,s:chip,l:tdet,f:det,f:sky,s:chip_id,s:pha,s:pi,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status,s:amp_sf\}"

  # PTPL-14 / REF#1 :
  # 3/2018 add samp_hrc_evtdef ; ie. stdlev_HRC+l:raw+ crsv, crsu, amp_sf, samp
  samp_hrc_evtdef="\{d:time,f:rd,s:chip,l:tdet,f:det,f:sky,s:chip_id,s:pha,s:pi,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status,l:raw,s:crsv,s:crsu,s:amp_sf,f:samp\}"

  # PTPL-71 :
  # 3/2019 add sumamps_hrc_evtdef ; ie. stdlev_HRC:  s:av1,s:av2,s:av3,s:au1,s:au2,s:au3,s:sumamps
  sumamps_hrc_evtdef="\{d:time,f:rd,s:chip,l:tdet,f:det,f:sky,s:chip_id,s:pha,s:pi,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,x:status,l:raw,s:crsv,s:crsu,s:amp_sf,f:samp,s:av1,s:av2,s:av3,s:au1,s:au2,s:au3,s:sumamps\}"

  #  define hetgCC_ACIS
  hetgCC_ACIS="\{d:time,d:time_ro,i:expno,f:rd,s:chip,s:tdet,f:det,f:sky,s:ccd_id,s:phas,l:pha,s:pi,f:energy,s:grade,s:fltgrade,s:node_id,s:tg_m,f:tg_lam,f:tg_mlam,s:tg_srcid,s:tg_part,s:tg_smap,f:chipy_tg,f:chipy_zo,l:pha_ro,x:status\}"
  #  missing_evtdef  = hetgCC_ACIS - tg_mlam 
  evtdef_no_tg_mlam="\{d:time,i:expno,f:rd,s:chip,s:tdet,f:det,f:sky,s:ccd_id,l:pha,s:pi,f:energy,s:grade,s:fltgrade,s:node_id,s:tg_m,f:tg_lam,s:tg_srcid,s:tg_part,s:tg_smap,f:chipy_tg,f:chipy_zo,l:pha_ro,x:status\}"



  # data to compare with dmdiff 
  m1=50000 
  m2=55000 

  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    acishetg ) 
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3  osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=${stdlev1} eventdef=${eventdef} clobber=yes geompar=geom verbose=0 "
      echo $test_string | tee -a  $LOGFILE 
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti4 gti5 gti6 gti7 gti8 gti9" 
        ;;
    # !!6
    # !!Remove all tg_* columns from the acishetg's outfile (acishetg_evt1a.fits),
    # !!    then feed it back to 'acishetg_2A'.
    # !!Run it with the same region file as 'acishetg_2B'.
    # !!The output should be same as 'acishetg_2B'
    # !!
    acishetg_2A )
      infile=$TESTIN/$toolname/acishetg_noTGcol_evt1a.fits
      regionfile="$TESTIN/$toolname/acishetg_reg1a_bigCircle.fits"
      acaofffile=$TESTIN/$toolname/tg_resolve_events_acishetg_aoff1.fits
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3  osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=${stdlev1} eventdef=${eventdef} clobber=yes geompar=geom verbose=0 "
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti4 gti5 gti6 gti7 gti8 gti9"
        ;;
    # !!6   feed back acishetg's output (evt1a) as 'infile' and 
    # !!    run it with a different region file
    acishetg_2B ) 
      regionfile="$TESTIN/$toolname/acishetg_reg1a_bigCircle.fits"
      infile="$TESTIN/$toolname/acishetg_evt1a.fits"
      acaofffile="$TESTIN/$toolname/tg_resolve_events_acishetg_aoff1.fits"

      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile}  outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3  osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=${stdlev1} eventdef=${eventdef} clobber=yes geompar=geom verbose=0 "
      echo $test_string | tee -a  $LOGFILE 
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti4 gti5 gti6 gti7 gti8 gti9" 
        ;;
    #
    # !!6   feed back acishetg's output as 'infile' 
    # !!    and run it with a different region file
    acishetg_3 ) 

      regionfile="$TESTIN/$toolname/acishetg_reg1a_smallCircle.fits"

      infile=$TESTIN/$toolname/acishetg_evt1a.fits
      acaofffile="$TESTIN/$toolname/tg_resolve_events_acishetg_aoff1.fits"

      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3  osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=${stdlev1} eventdef=${eventdef} clobber=yes geompar=geom verbose=0 "
      echo $test_string | tee -a  $LOGFILE 
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti4 gti5 gti6 gti7 gti8 gti9" 
        ;;
    # !!7
    acisletg )
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${eventdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7 gti4 gti5 gti6 gti8 gti9" 
        ;;

    #same as acisletg except using upd_acis_evtdef w/ pha_ro in it.
    #infile doesn't have pha_ro; create the output pha_ro column w/ 0's and w/o range
    acisletg_1b )   infile="$INDIR/${toolname}_acisletg_evt1.fits"
                    outfile="$OUTDIR/${testid}_evt1a.fits"
                    regionfile="$INDIR/${toolname}_acisletg_reg1a.fits"
                    acaofffile="$INDIR/${toolname}_acisletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${upd_acis_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7 gti4 gti5 gti6 gti8 gti9"
        ;;

    #same as acisletg_1b except eventdef is raw_acis_evtdef;
    #infile doesn't have raw;  create the output vector column raw(rawx,rawy) 
    #with the value of 0 and w/o setting the column range;
    acisletg_raw )   infile="$INDIR/${toolname}_acisletg_evt1.fits"
                    outfile="$OUTDIR/${testid}_evt1a.fits"
                    regionfile="$INDIR/${toolname}_acisletg_reg1a.fits"
                    acaofffile="$INDIR/${toolname}_acisletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${raw_acis_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7 gti4 gti5 gti6 gti8 gti9"
        ;;

#
# Input evt file is modified from acis 680.
# osipfile=CALDB. If $CALDB points to test_2, then caldb lib returns *osip_cti*.fits.
#
    acisletg_2 )
               infile=${INDIR}/${testid}_evt1.fits
               acaofffile="$INDIR/${testid}_aoff1.fits"
               regionfile="$INDIR/${testid}_reg1a.fits"
               outfile=${OUTDIR}/${testid}_evt1a.fits
               savfile=${SAVDIR}/${testid}_evt1a.fits
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${eventdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7 gti4 gti5 gti6 gti8 gti9"
        ;;
    # 10/22/2018 :  PTPL-1 : 
    # test both of the input zip file and the output non-zip file have the same name and are in the same dir.
    # Note: acisletg_2_B_evt1a.fits.gz is the output of testId=acisletg_2, then renamed and gzipped;
    acisletg_2_B )
               infile=${OUTDIR}/${testid}_evt1a.fits
               outfile=${OUTDIR}/${testid}_evt1a.fits
               acaofffile="$INDIR/${testid}_aoff1.fits"
               regionfile="$INDIR/${testid}_reg1a.fits"
               savfile=${SAVDIR}/${testid}_evt1a.fits
      cp ${INDIR}/${testid}_evt1a.fits.gz $OUTDIR/.
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${eventdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7 gti4 gti5 gti6 gti8 gti9"
        ;;
    # same as acisletg_2 except osipfile=acisD2000-01-29osip_ctiN0005.fits
    acisletg_3 )
               osipfile=${INDIR}/acisD2000-01-29osip_ctiN0005.fits
               infile=${INDIR}/acisletg_2_evt1.fits
               acaofffile="$INDIR/acisletg_2_aoff1.fits"
               regionfile="$INDIR/acisletg_2_reg1a.fits"
               outfile=${OUTDIR}/${testid}_evt1a.fits
               savfile=${SAVDIR}/${testid}_evt1a.fits
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${eventdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7 gti4 gti5 gti6 gti8 gti9"
        ;;
    # same as acisletg_2 except osipfile=acisD2000-01-29osipN0006.fits
    acisletg_4 )
               osipfile=${INDIR}/acisD2000-01-29osipN0006.fits
               infile=${INDIR}/acisletg_2_evt1.fits
               acaofffile="$INDIR/acisletg_2_aoff1.fits"
               regionfile="$INDIR/acisletg_2_reg1a.fits"
               outfile=${OUTDIR}/${testid}_evt1a.fits
               savfile=${SAVDIR}/${testid}_evt1a.fits
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${eventdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7 gti4 gti5 gti6 gti8 gti9"
        ;;
  # same as acisletg_4 except using upd_acis_evtdef;
  # infile has pha_ro; output pha_ro has same value and range as infile;
    acisletg_4_B )
               osipfile=${INDIR}/acisD2000-01-29osipN0006.fits
               infile=${INDIR}/acisletg_2_evt1.fits
               acaofffile="$INDIR/acisletg_2_aoff1.fits"
               regionfile="$INDIR/acisletg_2_reg1a.fits"
               outfile=${OUTDIR}/${testid}_evt1a.fits
               savfile=${SAVDIR}/${testid}_evt1a.fits
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0  stdlev1=\)eventdef eventdef=${upd_acis_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7 gti4 gti5 gti6 gti8 gti9"
        ;;
    # !!7
    hrcletg )

      # eventdef='\)stdlev1_HRC'     # 3/2018: set to ${old_stdlev1_HRC}

      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${old_stdlev1_HRC} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;

    # same as hrcletg except eventdef is rawy_hrc_evtdef ;
    # infile has rawy; create the output scalar column rawy
    # and copy the value and range of rawy from infile;
    hrcletg_rawy )    infile="$INDIR/${toolname}_hrcletg_evt1.fits"
                    outfile="$OUTDIR/${testid}_evt1a.fits"
                    regionfile="$INDIR/${toolname}_hrcletg_reg1a.fits"
                    acaofffile="$INDIR/${toolname}_hrcletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${rawy_hrc_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;

    # same as hrcletg except eventdef is rawx_hrc_evtdef ;
    # infile has rawx ; create the output scalar column rawx 
    # and copy the value and range of rawx from infile;
    hrcletg_rawx )    infile="$INDIR/${toolname}_hrcletg_evt1.fits"
                    outfile="$OUTDIR/${testid}_evt1a.fits"
                    regionfile="$INDIR/${toolname}_hrcletg_reg1a.fits"
                    acaofffile="$INDIR/${toolname}_hrcletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${rawx_hrc_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;
    # same as hrcletg except eventdef is raw_hrc_evtdef ;
    # infile doesn't have raw, but has (rawx,rawy);  
    # create the output vector column raw(rawx,rawy) and copy
    # the value and range of (rawx, rawy) from infile;
    hrcletg_raw )    infile="$INDIR/${toolname}_hrcletg_evt1.fits"
                    outfile="$OUTDIR/${testid}_evt1a.fits"
                    regionfile="$INDIR/${toolname}_hrcletg_reg1a.fits"
                    acaofffile="$INDIR/${toolname}_hrcletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${raw_hrc_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;

    # same as hrcletg except different eventdef ; small infile ; 
    # infile has no amp_sf, output amp_sf are 0's and tlmin/tlmax set to inf ;) 
    hrcletg_amp_sf )    infile="$INDIR/hrcletg_small_evt1_no_amp_sf.fits"
                    outfile="$OUTDIR/${testid}_evt1a.fits"
                    regionfile="$INDIR/${toolname}_hrcletg_reg1a.fits"
                    acaofffile="$INDIR/${toolname}_hrcletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${amp_sf_hrc_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;

    # PTPL-14: retain crsu,crsv,amp_sf,rawx,rawy,samp
    # Same as hrcletg except with smaller infile and set eventdef to samp_hrc_evtdef;
    # infile doesn't have samp, output samp with all 0's and tlmin/tlmax set to inf ;
    hrcletg_samp_a ) infile="$INDIR/hrcletg_small_evt1_no_samp.fits"
                     outfile="$OUTDIR/${testid}_evt1a.fits"
                     regionfile="$INDIR/${toolname}_hrcletg_reg1a.fits"
                     acaofffile="$INDIR/${toolname}_hrcletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${samp_hrc_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;

    # PTPL-14: retain crsu,crsv,amp_sf,rawx,rawy,samp
    # Same as hrcletg_samp_a except infile contains samp ;
    # copy samp values and tlmin/tlmax from input to output;
    hrcletg_samp_b )    infile="$INDIR/hrcletg_small_evt1_w_samp.fits"
                    outfile="$OUTDIR/${testid}_evt1a.fits"
                    regionfile="$INDIR/${toolname}_hrcletg_reg1a.fits"
                    acaofffile="$INDIR/${toolname}_hrcletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${samp_hrc_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;

    # PTPL-71: remove dmmerge col.filter from hrc lev2, ie. retain AU1,2,3, AV1,2,3, SUMAMPS 
    # Same as hrcletg except with smaller infile and set eventdef to sumamps_hrc_evtdef;
    # infile doesn't have AU,AV,SUMAMPS, output AU,AV,SUMAMPSwith all 0's and tlmin/tlmax set to inf ;
    hrcletg_sumamps_a ) infile="$INDIR/hrcletg_small_evt1_no_au_av_sumamps.fits"
                     outfile="$OUTDIR/${testid}_evt1a.fits"
                     regionfile="$INDIR/${toolname}_hrcletg_reg1a.fits"
                     acaofffile="$INDIR/${toolname}_hrcletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${sumamps_hrc_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;

    # PTPL-71: remove dmmerge col.filter from hrc lev2, ie. retain AU1,2,3, AV1,2,3, SUMAMPS 
    # Same as hrcletg_sumamps_a except infile contains AU,AV,SUMAMPS;
    # copy AU,AV,SUMAMPS values and tlmin/tlmax from input to output;
    hrcletg_sumamps_b )    infile="$INDIR/hrcletg_small_evt1_w_au_av_sumamps.fits"
                    outfile="$OUTDIR/${testid}_evt1a.fits"
                    regionfile="$INDIR/${toolname}_hrcletg_reg1a.fits"
                    acaofffile="$INDIR/${toolname}_hrcletg_aoff1.fits"
                    savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${sumamps_hrc_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;


    # !!7
    standard )

      osipfile="${INDIR}/${toolname}_${testid}_rmf.fits"

      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${eventdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti8"
        ;;

    # !!7  make the size of evt1.fits smaller
    acishetgCC )
            infile="$INDIR/acishetgCC_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"

      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${eventdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9" 
        ;;

    # !!7  same as testId acishetgCC except using upd_acis_evtdef 
    acishetgCC_B )
            infile="$INDIR/acishetgCC_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${upd_acis_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9" 
        ;;

    # Test variable hetgCC_2ndRun=NO (col chipy_tg exists, time_adj!=GRATING, grating=HETG)
    # ( same as acishetgCC_2ndRun below except inKey time_adj=NONE # )
    # ( expect a warning msg )
    # ( infile = outfile )
    #
    # Use testID acishetgCC_B's outfile to create an InFile for this case.
    # Added TIME_ADJ="NONE". 
    #
    acishetgCC_1stRun )
            infile="$INDIR/acishetgCC_1stRun_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef eventdef=${upd_acis_evtdef} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;

    # Test variable hetgCC_2ndRun=YES  ( col chipy_tg exists, time_adj=GRATING, grating=HETG )
    #
    # to test:  running L15 pipe w/ the same input energy for tg_r_e's 1stRun and 2ndRun.
    #   e.g. when L15 pipe runs a_p_e for 2nd time and creates the same energy values.
    #   later on, when runs tg_r_e for 2nd time, tgre re-computes (tg_m,tg_lam) with
    #   the same energy as the 1stRun, therefore (tg_m,tg_lam) expect no change.
    #
    # Use testID acishetgCC_B's outfile to create an InFile for this case.
    # Added TIME_ADJ="GRATING". 
    #
    # Use eventdef = hetgCC_ACIS  ( same as upd_acis_evtdef ) 
    # 
    # Expect outfile = infile 
    #    ie.  outCol (tg_m, tg_lam ) will be same as infile
    # (Note: 4/1/2015:  we now recompute tg_part,tg_mlamda and chipy_tg-use-aspect,
    #                   so outfile != infile )
    #
    acishetgCC_2ndRun )
            infile="$INDIR/acishetgCC_2ndRun_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef  eventdef=${hetgCC_ACIS} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;

    # Test variable hetgCC_2ndRun=YES  ( col chipy_tg exists, time_adj=GRATING, grating=HETG )
    #
    # to test: running L15 pipe w/ different input energy for tg_r_e's 1stRun and 2ndRun.
    #   e.g. when L15 pipe runs a_p_e for 2nd time and creates different energy values.
    #   later on, when runs tg_r_e for 2nd time, tgre re-computes (tg_m,tg_lam) with
    #   different energy as the 1stRun, therefore (tg_m,tg_lam) expect different.
    #
    # Same as previous testId acishetgCC_2ndRun, except some of input energy were changed
    #
    # Use eventdef = hetgCC_ACIS  ( same as upd_acis_evtdef )
    #
    # Expect outFile might be different from inFile 
    #    ie. (tg_m, tg_lam) might be different
    # (Note: 4/1/2015:  we now recompute tg_part,tg_mlamda and chipy_tg-use-aspect,
    #                   so, they might be different from infile  )
    #
    acishetgCC_2ndRun_B )
            infile="$INDIR/acishetgCC_2ndRun_B_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef  eventdef=${hetgCC_ACIS} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;

    # 3/2015 :
    # Test variable hetgCC_2ndRun was reset from YES to NO, because the eventdef parameter is blank
    # This will be executed as hetgCC 1stRun.
    # 
    # same as testId acishetgCC_2ndRun_B, except we have eventdef = "", expect a warning:
    #     WARNING: HETG/CC-mode detected, but eventdef parameter is blank, proceeding with 
    #              normal tg_resolve_events processing.
    # 
    # expect  ORDER_ADJ = NONE
    # 
    eventdef_blank )
            infile="$INDIR/acishetgCC_2ndRun_B_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef  eventdef="" clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;

    # 3/2015
    # Missing a NON-Required input column ( tg_m )
    #
    # This will be run as hetgCC 2ndRun ( Not 1st Run) ; The output Tg_m will be computed.
    # 
    # same as testId acishetgCC_2ndRun_B, except inEvtFile missing tg_m column :
    # 
    # expect  ORDER_ADJ = HETG
    # 
    # (Note: 4/1/2015:  we now recompute tg_part,tg_mlamda and chipy_tg-use-aspect, 
    #                   so, inCol != outCol )
    # 
    missing_inCol_1 )
            infile="$INDIR/acishetgCC_2ndRun_C_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef  eventdef=${hetgCC_ACIS} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;


    # 3/2015
    # Missing one of the Required input columns ( tg_mlam )
    # in the Input acishetgCC_2ndRun_D_evt1.fits file.
    #
    # The hetgCC_2ndRun flag was reset from YES to NO, because it is a required column.
    # 
    # set   eventdef = ${evtdef_no_tg_mlam}
    # Input acishetgCC_2ndRun_D_evt1.fits file does not have tg_mlam column
    #
    # same as testId acishetgCC_2ndRun_B, except tg_mlam is missing in both inEvtFile & eventdef parameter,
    #
    # expect a warning:
    #    "WARNING: HETG/CC-mode detected, but input event file and eventdef parameter missing some required
    #          columns (for example: energy,ccd_id,chipy_tg,tg_part,tg_srcid,tg_mlam), proceeding with 
    #          normal tg_resolve_events processing.
    # 
    # expect  ORDER_ADJ = NONE
    # 
    missing_inCol_2 )
            infile="$INDIR/acishetgCC_2ndRun_D_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef  eventdef=${evtdef_no_tg_mlam} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;


    # 3/2015
    # Missing NON-Required input column ( rd (tg_r, tg_d) )
    # in the Input acishetgCC_2ndRun_E_evt1.fits file.
    #
    # same as testId acishetgCC_2ndRun_B, except RD column is missing in inEvtFile 
    # 
    # Run it as hetgCC 2ndRun ( Not 1st Run)
    # and the output RD vector col. will be all zero's
    #
    # expect  ORDER_ADJ = HETG
    # 
    # (Note: 4/1/2015:  we now recompute tg_part,tg_mlamda and chipy_tg-use-aspect, 
    #                   so, inCol != outCol )
    # 
    missing_inCol_3 )
            infile="$INDIR/acishetgCC_2ndRun_E_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef  eventdef=${hetgCC_ACIS} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;

    # 3/4/2015 : NEW
    #
    # Missing All Required input columns ( EXCEPT chipy_tg - see NOTE below )
    #        (ie. columns of  energy,ccd_id,chip,tg_part,tg_srcid,tg_mlam )
    # in the Input acishetgCC_2ndRun_F_evt1.fits file.
    #
    # ( NOTE: If chipy_tg missing in inEvtFile, then hetgCC_2ndRun was NEVER set to 1 in the 1st place.
    #         so I keep chipy_tg in the input evt file 
    # )
    #
    # The hetgCC_2ndRun flag was reset from YES to NO, because they are the required columns.
    #
    # set   eventdef = \)hetgCC_ACIS
    # Input acishetgCC_2ndRun_F_evt1.fits file does not have all required columns 
    #
    # same as testId acishetgCC_2ndRun_B, except inEvtFile missing all required columns 
    #
    # Expect a warning: 
    #    "WARNING: HETG/CC-mode detected, but input event file and eventdef parameter missing some required
    #          columns (for example: energy,ccd_id,chipy_tg,tg_part,tg_srcid,tg_mlam), proceeding with
    #          normal tg_resolve_events processing.
    #
    # expect  ORDER_ADJ = NONE
    # 
    missing_inCol_4 )
            infile="$INDIR/acishetgCC_2ndRun_F_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef  eventdef=${hetgCC_ACIS} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;


    # 3/2015 : new
    # Missing (NON required) grating columns in inEvtFile 
    #        (ie. columns of  rd(tg_r,tg_d), tg_m, tg_lam, tg_smap,  
    # in the Input acishetgCC_2ndRun_G_evt1.fits file.
    #
    # The hetgCC_2ndRun flag was kept as YES, because they are NOT the required columns.
    #
    # set   eventdef = \)hetgCC_ACIS
    # Input acishetgCC_2ndRun_G_evt1.fits file missing NON-required Grating Columns 
    #
    # same as testId acishetgCC_2ndRun_B, except inEvtFile missing non-required grating columns 
    #
    # NO WARNINGS are expected.
    #
    # 
    # expect  ORDER_ADJ = HETG
    # 
    # (Note: 4/1/2015:  we now recompute tg_part,tg_mlamda and chipy_tg-use-aspect, 
    #                   so, inCol != outCol )
    # 
    missing_inCol_5 )
            infile="$INDIR/acishetgCC_2ndRun_G_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/acishetgCC_reg1a.fits"
            acaofffile="${INDIR}/acishetgCC_asol1.fits"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=${osipfile} grating_obs=HEADER_VALUE detector=HEADER_VALUE osort_lo=0.3 osort_hi=0.3 energy_lo_adj=1.0 energy_hi_adj=1.0 rand_seed=1 rand_pix_size=0.0 stdlev1=\)eventdef  eventdef=${hetgCC_ACIS} clobber=yes geompar=geom verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti7  gti5  gti6  gti8  gti9"
        ;;
    
    hrci_hetg_overlap )
            infile="$INDIR/hrci_hetg_overlap_evt1.fits"
            outfile="$OUTDIR/${testid}_evt1a.fits"
            regionfile="$INDIR/${testid}_reg1a.fits"
            acaofffile="@${INDIR}/${testid}_asol1.lis"
            savfile="$SAVDIR/${testid}_evt1a.fits"
      test_string="tg_resolve_events infile=${infile} regionfile=${regionfile} outfile=${outfile} acaofffile=${acaofffile} logfile=${logfile} osipfile=none grating_obs=HEADER_VALUE eventdef=')stdlev1_HRC' clobber=yes verbose=0"
      echo $test_string | tee -a  $LOGFILE
      echo "" | tee -a  $LOGFILE
      eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
      GTI="gti"
        ;;

  esac

  ####################################################################
  # if the tool failed to run, set mismatch to 0.
  #
  if test $? -ne 0; then
     echo "" | tee -a $LOGFILE
     echo "$toolname failed to run" | tee -a $LOGFILE
     echo "" | tee -a $LOGFILE
     mismatch=0           
  fi

  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  #dmlist $outfile header,data,clean > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
  #dmlist $savfile header,data,clean > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #          2>>$LOGFILE

  # compare the output for tg_resolve_events
  # !!12
  #dmdiff $outfile"[#row>=${m1}&&#row<${m2}]" $savfile"[#row>=${m1}&&#row<${m2}]" keys=yes comment=no tol=$TOL >>$LOGFILE
  dmdiff $outfile $savfile keys=yes comment=no tol=$TOL >>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: -- tg_resolve_events mismatch in $outfile" >> $LOGFILE
    mismatch=0
  fi

  ### compare - REGION ( all tests have REGION blocks )
  dmdiff $outfile"[REGION]" $savfile"[REGION]" keys=yes comment=no tol=$TOL >>$LOGFILE
  if  test $? -ne 0 ; then
     echo "ERROR: -- tg_resolve_events mismatch in $outfile[REGION]" >> $LOGFILE
     mismatch=0
  fi

  ### compare gti 
  for gti in $GTI
  do 
    dmdiff $outfile"[${gti}]" $savfile"[${gti}]" comment=no tol=$TOL >>$LOGFILE
    if  test $? -ne 0 ; then
       echo "ERROR: -- tg_resolve_events mismatch in $outfile[REGION]">>$LOGFILE
       mismatch=0
    fi
  done

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi
  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Bad
    echo "${testid} NOT-OK" | tee -a $LOGFILE
    script_succeeded=1
  else
    # Good
    echo "${testid} OK" | tee -a $LOGFILE
  fi

  echo "" | tee -a $LOGFILE

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo ""
echo "log file in ${LOGFILE}"
echo ""

exit $script_succeeded
