#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# vtpdetect.t
# test script for mytool


# syntax:
# vtpdetect.t 
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="vtpdetect"

# set up list of tests
alltests="evts_no_emap evts_with_emap img_no_emap img_with_emap img_with_wcs_rot wcs_separate wcs_linear negative_coords"

shortlist="$alltests"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist


# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits


  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in

    evts_no_emap) test_string="vtpdetect infile=$INDIR/'acis_650_ccd6_evts.fits' expfile='none' outfile=$outfile regfile='none' ellsigma='3' scale='1' limit='1e-06' coarse='10' maxiter='10' edge='2' superdo='no' maxbkgflux='0.8' mintotflux='0.8' maxtotflux='2.6' mincutoff='1.2' maxcutoff='3' fittol='1e-06' fitstart='1.5' clobber='yes' logfile='stderr' "
    ;;

    evts_with_emap) test_string="vtpdetect infile=$INDIR/'acis_650_ccd6_evts.fits' expfile=$INDIR/'acis_650_ccd6_emap.fits' outfile=$outfile regfile='none' ellsigma='3' scale='1' limit='1e-06' coarse='10' maxiter='10' edge='2' superdo='no' maxbkgflux='0.8' mintotflux='0.8' maxtotflux='2.6' mincutoff='1.2' maxcutoff='3' fittol='1e-06' fitstart='1.5' clobber='yes' logfile='stderr' "
    ;;

    img_no_emap) test_string="vtpdetect infile=$INDIR/'acis_650_ccd6_img.fits' expfile='none' outfile=$outfile regfile='none' ellsigma='3' scale='1' limit='1e-06' coarse='10' maxiter='10' edge='2' superdo='no' maxbkgflux='0.8' mintotflux='0.8' maxtotflux='2.6' mincutoff='1.2' maxcutoff='3' fittol='1e-06' fitstart='1.5' clobber='yes' logfile='stderr' "
    ;;

    img_with_emap) test_string="vtpdetect infile=$INDIR/'acis_650_ccd6_img.fits' expfile=$INDIR/'acis_650_ccd6_emap.fits' outfile=$outfile regfile='none' ellsigma='3' scale='1' limit='1e-06' coarse='10' maxiter='10' edge='2' superdo='no' maxbkgflux='0.8' mintotflux='0.8' maxtotflux='2.6' mincutoff='1.2' maxcutoff='3' fittol='1e-06' fitstart='1.5' clobber='yes' logfile='stderr' "
    ;;

   img_with_wcs_rot) test_string="vtpdetect infile=$INDIR/'ifake.img' expfile='' outfile=$outfile regfile='none' ellsigma='3' scale='1' limit='1e-06' coarse='10' maxiter='10' edge='2' superdo='no' maxbkgflux='0.8' mintotflux='0.8' maxtotflux='2.6' mincutoff='1.2' maxcutoff='3' fittol='1e-06' fitstart='1.5' clobber='yes' logfile='stderr' "
    ;;

   wcs_separate) test_string="vtpdetect infile=$INDIR/separate_linear_wcs.img outfile=$outfile mode=h clob+ scale=1 edge=4"
    ;;

   wcs_linear) test_string="vtpdetect infile=$INDIR/chip.img outfile=$outfile mode=h clob+ scale=1 edge=4"
    ;;

   negative_coords) test_string="vtpdetect infile=$INDIR/'shifted_events.fits[cols a,b]' outfile=$outfile mode=h clob+"
    ;;

  esac


  echo $test_string | tee -a $LOGFILE
  eval $test_string  | tee -a $LOGFILE  2>&1


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  #dmlist $outfile header,data > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  #dmlist $savfile header,data > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
            2>>$LOGFILE

  # compare
  #diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #     /dev/null 2>>$LOGFILE

  #echo "dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
  dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
    mismatch=0
  fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
