#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.


# wcs_match.t
# test script for wcs_match


# syntax:
# wcs_match.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="wcs_match"

# set up list of tests
alltests="test1 test2 test3 test4 test5 test6 test7 test8 test9 test10 test11 test12"

# "short" test to run
shortlist="test1"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  transformfile=$OUTDIR/${testid}_xfm.fits
  transformsavfile=$SAVDIR/${testid}_xfm.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in

    test1 ) # basic test - 66 src input, calculate xfm, update asol
	    #...Define input files, params 
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_refImg.fits
	    test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
	     wcsfile=$refimgfile outfile=$transformfile \
	     method=rst radius=12"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    test2 ) # CDFN, obsids 2386 & 3388 test with wavdetect srcs
	    #  - absolute residual value test
	    #...Define input files, params 
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_refImg.fits
	    test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
	     wcsfile=$refimgfile outfile=$transformfile \
	     method=rst radius=2 residlim=1 residfac=0 residtype=0"
	    #...run test
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;
    test3 ) # CDFN, obsids 2386 & 3388 test with wavdetect srcs
	    #  - residual-to-position error ratio test
	    #...Define input files, params 
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_refImg.fits
	    test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
	     wcsfile=$refimgfile outfile=$transformfile \
	     method=rst radius=2 residlim=0.0 residfac=0.755 residtype=1"
	    #...run test
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    test4 ) # basic test - 66 src input, calculate xfm, method=trans
	    #...Define input files, params 
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_refImg.fits
	    test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
	     wcsfile=$refimgfile outfile=$transformfile \
	     method=trans radius=12"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    test5 ) # check verbose output of trans method (bug 13905)
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_wcsevt.fits
            test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
             wcsfile=$refimgfile outfile=$transformfile \
             method=trans radius=12 verbose=2"
            echo $test1_string | tee -a  $LOGFILE
            rm -f $OUTDIR/scrub_${testid}.0.tmp
            eval $test1_string 2>&1 | tee -a $OUTDIR/scrub_${testid}.0.tmp
            cat $OUTDIR/scrub_${testid}.0.tmp >> ${LOGFILE}
            ;;

    test6 ) # test fix to L3 Trac bug #1221, poor matches are sometimes
            # erroneously kept if they are at the end of the matched source list
            #...Define input files, params 
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_refImg.fits
            test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
             wcsfile=$refimgfile outfile=$transformfile \
             method=trans radius=3 residlim=1.99 residtype=0 residfac=0 residtype=0"
            echo $test1_string | tee -a  $LOGFILE 
            eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    test7 ) # check verbose output to log file (bug 13939)
            infile1=$INDIR/test5_infile1.fits
            infile2=$INDIR/test5_infile2.fits
            refimgfile=$INDIR/test5_wcsevt.fits
            test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
             wcsfile=$refimgfile outfile=$transformfile \
             method=trans radius=12 verbose=4 logfile=$OUTDIR/${testid}_scrub_logfile"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            sed -e "s;${INDIR};;g" $OUTDIR/${testid}_scrub_logfile | sed -e "s;${OUTDIR};;g" >  $OUTDIR/${testid}_logfile.txt
            rm -f $OUTDIR/${testid}_scrub_logfile
            ;;

    test8 ) # check that residual ratios are computed correctly when not all ref sources are
	    # used (ptpl-262). This is ~test3, but with residtype=0
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_refImg.fits
            test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
             wcsfile=$refimgfile outfile=$transformfile \
	     method=rst radius=2 residlim=0.0 residfac=0.755 residtype=0 \
             verbose=2 logfile=$OUTDIR/${testid}_scrub_logfile"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            sed -e "s;${INDIR};;g" $OUTDIR/${testid}_scrub_logfile | sed -e "s;${OUTDIR};;g" >  $OUTDIR/${testid}_logfile.txt
            rm -f $OUTDIR/${testid}_scrub_logfile
            ;;

    test9 ) # check multiple matches to both ref and dup srcs with multimatch=yes
	    # test9 input is a duplicate of test6 with a tweak to match ref srcs 2 and 3 to the same dup src
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_wcsevt.fits
            test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
             wcsfile=$refimgfile outfile=$transformfile \
             method=trans radius=12 multimatch=yes"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    test10) # Verify that input ref src files with 0 srcs do not cause a segV.
	    # This was seen after changes as part of ptcat-3102.
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_regImg.fits
            test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
             wcsfile=$refimgfile outfile=$transformfile \
             method=trans radius=2 residlim=0.5 multimatch=yes"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    test11) # Verify that input will all ref srcs matching dup srcs
	    # do not cause a segV. (KJG dsdevel email 07OCT21)
            infile1=$INDIR/${testid}_infile1.dat
            infile2=$INDIR/${testid}_infile2.dat
            refimgfile=$INDIR/${testid}_wcsevt.fits
            test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
             wcsfile=$refimgfile outfile=$transformfile \
             method=trans radius=30 residlim=2.0 multimatch=yes"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;

    test12) # Verify that when deleted, last source match which is a 
	    # 'poor match' (ie, ref src matches 2+ dup srcs) does not
	    # mistakenly also delete the last valid source pair
	    # ptcat-3405
            infile1=$INDIR/${testid}_infile1.fits
            infile2=$INDIR/${testid}_infile2.fits
            refimgfile=$INDIR/${testid}_singlepix3.fits
            test1_string="wcs_match refsrcfile=$infile1 infile=$infile2 \
             wcsfile=$refimgfile outfile=$transformfile \
             method=trans radius=2 residlim=0.5 multimatch=no"
	    echo $test1_string | tee -a  $LOGFILE 
	    eval $test1_string  | tee -a  $LOGFILE  2>&1
            ;;


  esac

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  ####################################################################
  # FITS table

  # new output
  #dmlist $outfile opt=header,data,clean rows=700:1400 > \
  # $OUTDIR/${testid}.dmp1 2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2 \
  # 2>>$LOGFILE
  
  # reference output
  #dmlist $savfile opt=header,data,clean rows=700:1400 > \
  # $OUTDIR/${testid}.dmp1_std 2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  # 2>>$LOGFILE
  
  # compare
  #diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  # /dev/null 2>>$LOGFILE

  dmdiff $transformfile $transformsavfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: MISMATCH in $transformfile" >> $LOGFILE
    mismatch=0
  fi

  if test ${testid} = "test5" ; then
    sed -e "s;${INDIR};;g" $OUTDIR/scrub_${testid}.0.tmp | sed -e "s;${OUTDIR};;g" > $OUTDIR/${testid}_output.txt

    diff $OUTDIR/${testid}_output.txt $SAVDIR/${testid}_output.txt > /dev/null 2>> ${LOGFILE}

    if  test $? -ne 0 ; then
      echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}_output.txt" >> $LOGFILE
      mismatch=0
    fi
  elif test ${testid} = "test7" ; then
    diff $OUTDIR/${testid}_logfile.txt $SAVDIR/${testid}_logfile.txt > /dev/null 2>> ${LOGFILE}

    if  test $? -ne 0 ; then
      echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}_logfile.txt" >> $LOGFILE
      mismatch=0
    fi
  fi 
  
  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
