#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.


# wcs_update.t
# test script for wcs_update


# syntax:
# wcs_update.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="wcs_update"

# set up list of tests
alltests="test2 test3 test4 test5 test6 test7 testblockspec testblockdne testfiledne testdmvsyntax"

# "short" test to run
shortlist="test2"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}_xfm.fits
  asoloutfile=$OUTDIR/${testid}_asolout.fits
  asoloutsavfile=$SAVDIR/${testid}_asolout.fits
  wcsupdfile=${testid}_wcsupdfile.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in

    test2 ) # CDFN, obsids 2386 & 3388 test with wavdetect srcs
	    #  - absolute residual value test
            #  - wcsupdate image file test
	    #...Define input files, params 
	    test_string="wcs_update infile=$OUTDIR/$wcsupdfile \
	     outfile=\"\" transformfile=$INDIR/${testid}_xfm.fits \
	     wcsfile=\"\""
            #...copy wcsupdate file into outdir
	    cp $INDIR/$wcsupdfile $OUTDIR/$wcsupdfile
	    #...run test
	    echo $test_string | tee -a  $LOGFILE 
	    eval $test_string  | tee -a  $LOGFILE  2>&1
            ;;
    test3 ) # CDFN, obsids 2386 & 3388 test with wavdetect srcs
	    #  - residual-to-position error ratio test
            #  - asol file update test
	    #...Define input files, params 
            asolinfile=$INDIR/${testid}_asol.fits
	    test_string="wcs_update \
             infile=$asolinfile outfile=$asoloutfile \
	     transformfile=$INDIR/${testid}_xfm.fits \
	     wcsfile=\"\""
	    #...run test
	    echo $test_string | tee -a  $LOGFILE 
	    eval $test_string  | tee -a  $LOGFILE  2>&1
            ;;
    test4 ) # CDFN, obsids 2386 & 3388 test with wavdetect srcs
	    #  - residual-to-position error ratio test
            #  - wcsupdate evt file test
	    #...Define input files, params 
	    test_string="wcs_update \
	     infile=$OUTDIR/$wcsupdfile outfile=\"\" \
	     transformfile=$INDIR/${testid}_xfm.fits \
	     wcsfile=\"\""
            #...copy wcsupdate file into outdir
	    cp $INDIR/$wcsupdfile $OUTDIR/$wcsupdfile
	    #...run test
	    echo $test_string | tee -a  $LOGFILE 
	    eval $test_string  | tee -a  $LOGFILE  2>&1
            ;;
    test5 ) # CDFN, obsids 2386 & 3388 test with wavdetect srcs
	    #  - reuse transform from test2
            #  - wcsupdate table test using EQSRC coords
	    #...Define input files, params 
	    test_string="wcs_update infile=$OUTDIR/$wcsupdfile \
	     outfile=\"\" transformfile=$INDIR/${testid}_xfm.fits \
	     wcsfile=\"\""
            #...copy wcsupdate file into outdir
	    cp $INDIR/$wcsupdfile $OUTDIR/$wcsupdfile
	    #...run test
	    echo $test_string | tee -a  $LOGFILE 
	    eval $test_string  | tee -a  $LOGFILE  2>&1
            ;;

    test6 ) # CDFN, obsids 12220 & 2313 test with wavdetect srcs
            #  - from ciao science thread- asol file update
            #    cxc.harvard.edu/ciao/threads/reproject_aspect
            #   
            asolinfile=$INDIR/${testid}_asol.fits
	    test_string="wcs_update \
             infile=$asolinfile outfile=$asoloutfile \
             transformfile=$INDIR/${testid}_xfm.fits \
             wcsfile=$INDIR/${testid}_cdfs_broad_thresh.img"
            #...run test
            echo $test_string | tee -a  $LOGFILE
            eval $test_string  | tee -a  $LOGFILE  2>&1
            ;;


    test7 ) # CDFN, obsids 12220 & 2313 test with wavdetect srcs
            #  - from ciao science thread- event file update
            #    cxc.harvard.edu/ciao/threads/reproject_aspect
            #
            test_string="wcs_update infile=$OUTDIR/$wcsupdfile \
             outfile=\"\" transformfile=$INDIR/${testid}_xfm.fits \
             wcsfile=$INDIR/${testid}_cdfs_broad_thresh.img"
            #...copy wcsupdate file into outdir
            cp $INDIR/$wcsupdfile $OUTDIR/$wcsupdfile
            #...run test
            echo $test_string | tee -a  $LOGFILE
            eval $test_string  | tee -a  $LOGFILE  2>&1
            ;;

    testblockspec )
            # test access of specific block using dm spec syntax
            test_string="wcs_update infile=\"$OUTDIR/$wcsupdfile[mask]\" \
             outfile=\"\" transform=\"\" \
             wcsfile=$INDIR/$wcsupdfile deltax=-100 \
             deltay=200" 
            #...copy wcsupdate file into outdir
            cp $INDIR/$wcsupdfile $OUTDIR/$wcsupdfile
            #...run test
            echo $test_string | tee -a  $LOGFILE
            eval $test_string  | tee -a  $LOGFILE  2>&1
            ;;

    testblockdne )
            # test access of specific block using dm spec syntax
            test_string="wcs_update infile=\"$OUTDIR/$wcsupdfile[mast]\" \
             outfile=\"\" transform=\"\" \
             wcsfile=$INDIR/$wcsupdfile deltax=-100 \
             deltay=200" 
            #...copy wcsupdate file into outdir
            cp $INDIR/$wcsupdfile $OUTDIR/$wcsupdfile
            #...run test
            \rm -f $OUTDIR/scrub_${testid}.0.tmp
            echo $test_string | tee -a  $LOGFILE
            eval $test_string 2>&1  | tee -a $OUTDIR/scrub_${testid}.0.tmp
            cat $OUTDIR/scrub_${testid}.0.tmp >> ${LOGFILE}
            ;;

    testfiledne )
            # test access of specific block using dm spec syntax
            test_string="wcs_update infile=\"$OUTDIR/$wcsupdfile\" \
             outfile=\"\" transform=\"\" \
             wcsfile=$INDIR/$wcsupdfile deltax=-100 \
             deltay=200" 
            #...run test
            \rm -f $OUTDIR/scrub_${testid}.0.tmp
            echo $test_string | tee -a  $LOGFILE
            eval $test_string 2>&1  | tee -a $OUTDIR/scrub_${testid}.0.tmp
            cat $OUTDIR/scrub_${testid}.0.tmp >> ${LOGFILE}
            ;;

    testdmvsyntax )
            # test access of specific block using dm spec syntax
            test_string="wcs_update infile=\"$OUTDIR/$wcsupdfile\" \
             outfile=\"\" transform=\"\" \
             wcsfile=\"$OUTDIR/$wcsupdfile[bin sky=128]\" deltax=-10 \
             deltay=-100" 
            #...copy wcsupdate file into outdir
            cp $INDIR/$wcsupdfile $OUTDIR/$wcsupdfile
            #...run test
            echo $test_string | tee -a  $LOGFILE
            eval $test_string  | tee -a  $LOGFILE  2>&1
            ;;

  esac

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  ####################################################################
  # FITS table

  # new output
  #dmlist $outfile opt=header,data,clean rows=700:1400 > \
  # $OUTDIR/${testid}.dmp1 2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2 \
  # 2>>$LOGFILE
  
  # reference output
  #dmlist $savfile opt=header,data,clean rows=700:1400 > \
  # $OUTDIR/${testid}.dmp1_std 2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  # 2>>$LOGFILE
  
  # compare
  #diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  # /dev/null 2>>$LOGFILE
  
  case ${testid} in
      
  testblockdne) 

    sed -e "s;${INDIR};;g" $OUTDIR/scrub_${testid}.0.tmp | sed -e "s;${OUTDIR};;g"  | sed -e "s; (.*): ;: ;g" > $OUTDIR/${testid}_output.txt
    diff $OUTDIR/${testid}_output.txt $SAVDIR/${testid}_output.txt > /dev/null 2>> ${LOGFILE}
    if  test $? -ne 0 ; then
       echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}_output.txt" >> $LOGFILE
       mismatch=0
    fi
    dmdiff $OUTDIR/$wcsupdfile $SAVDIR/$wcsupdfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
	echo "ERROR: MISMATCH in $OUTDIR/$wcsupdfile" >> $LOGFILE
	mismatch=0
    fi
    ;;

  testfiledne)

    sed -e "s;${INDIR};;g" $OUTDIR/scrub_${testid}.0.tmp | sed -e "s;${OUTDIR};;g"  | sed -e "s; (.*): ;: ;g" > $OUTDIR/${testid}_output.txt
    diff $OUTDIR/${testid}_output.txt $SAVDIR/${testid}_output.txt > /dev/null 2>> ${LOGFILE}
    if  test $? -ne 0 ; then
       echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}_output.txt" >> $LOGFILE
       mismatch=0
    fi
    ;;

  test2|test4|test5|test7|testblockspec|testdmvsyntax) 

    dmdiff $OUTDIR/$wcsupdfile $SAVDIR/$wcsupdfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
	echo "ERROR: MISMATCH in $OUTDIR/$wcsupdfile" >> $LOGFILE
	mismatch=0
    fi
    ;;
      
  test3|test6)

    dmdiff $asoloutfile $asoloutsavfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
	echo "ERROR: MISMATCH in $asoloutfile" >> $LOGFILE
	mismatch=0
    fi
    ;;

  esac
  
  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
