#! /bin/sh

# wrecon.t
# test script for wrecon


# syntax:
# wrecon.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="wrecon"

# set up list of tests
alltests="test1 test2 test3"

# "short" test to run
shortlist="test2"

image="ple"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
  # delete old outputs
  rm -f $OUTDIR/${image}_${testid}*

  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool

      case ${testid} in
        test1 ) test_string="wrecon  outputinfix=_test1 
	clobber=no  log=no 
	xscales=\"1.0 1.41 2 2.82 4 5.65 8 11.31 16\"
	yscales=\"1.0 1.41 2 2.82 4 5.65 8 11.31 16\"
	expthresh=0.1
	ellsigma=3.0 
	fluxscales=\"1 3 5 7 9\"  stall=no 
	infile=${INDIR}/${image}.fits
	expfile=${INDIR}/${image}_emap.fits
	correl=${INDIR}/${OUTPUT_INFIX}${image}_${testid}_correl_stk.lis
        nbkg=${INDIR}/${OUTPUT_INFIX}${image}_${testid}_nbkg_stk.lis
        psffile=${INDIR}/t1_psf_flat.fits \
	bkginput=none	bkgerrinput=no
	srclist=${INDIR}/${OUTPUT_INFIX}${image}_${testid}_src_stk.lis
	sourcefile=${OUTDIR}/${OUTPUT_INFIX}${image}_${testid}_src.fits
	defnbkgfile=${OUTDIR}/${OUTPUT_INFIX}${image}_${testid}_nbkg.fits
	fluxfile=${OUTDIR}/${OUTPUT_INFIX}${image}_${testid}_flux_stk.lis
	scellfile=${OUTDIR}/${OUTPUT_INFIX}${image}_${testid}_scell.fits
	imagefile=${OUTDIR}/${OUTPUT_INFIX}${image}_${testid}_image.fits "
	;;

        test2 ) test_string="wrecon  log=no verbose=0 \
        regfile=  ellsigma=3  \
        infile=${INDIR}/img.fits expfile= \
        correl=${INDIR}/wd_correl_stk \
        srclist=${INDIR}/wd_srclist_stk nbkg=${INDIR}/wd_nbkg_stk \
        xscales=\"1 2 4 8\"  yscales=\"1 2 4 8\"  expthresh=0.1 \
        bkgtime=0 exptime=0 \
        sourcefile=${OUTDIR}/wav_out.fits \
        scellfile=${OUTDIR}/wav_cell.fits \
        imagefile=${OUTDIR}/wav_recon.fits \
        defnbkgfile=${OUTDIR}/wav_nbkg.fits \
        fluxfile=${OUTDIR}/wd_flux_stk \
        bkginput= bkgerrinput=no \
        psffile=${INDIR}/psf_arcsec.fits \
        fluxscale= \"1 2 3 4\" \
        outputinfix= stall=no  \
        clobber=yes "
	;;
        test3 ) test_string="wrecon log=no verbose=0 \
        regfile=${OUTDIR}/region.txt ellsigma=3 \
        infile=${INDIR}/P0551690201PNS001IMAGE_8000.FTZ expfile= \
        correl=${INDIR}/wd_correl_stk_3  nbkg=${INDIR}/wd_nbkg_stk_3 \
        srclist=${INDIR}/wd_srclist_stk_3 \
        xscales=\"2 4 6 8 10 12 14 16 18 20\" \
        yscales=\"2 4 6 8 10 12 14 16 18 20\"  expthresh=0.1 \
        bkgtime=0 exptime=0 \
        sourcefile=${OUTDIR}/yes_src \
        scellfile=${OUTDIR}/yes_cell \
        imagefile=${OUTDIR}/yes_recon \
        defnbkgfile=${OUTDIR}/yes_nbkg \
        fluxfile=${OUTDIR}/wd_flux_stk_3 \
        bkginput=  bkgerrinput=no \
        psffile=${INDIR}/pn_psfsize.fits \
        fluxscales=\" 1 2 3 4 5 6 7 8 9 10\" \
        outputinfix= stall=no clobber=yes "
	;;

      esac

   echo $test_string | tee -a $LOGFILE
   eval $test_string  | tee -a $LOGFILE  2>&1

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # begin: AA
  # allimgs for test1,test2,test3
  case ${testid} in
     test1) allimgs="ple_test1_src.fits ple_test1_scell.fits ple_test1_nbkg.fits \
                ple_test1_image.fits ple_test1_000010_000010_flux.fits \
                ple_test1_000020_000020_flux.fits ple_test1_000040_000040_flux.fits \
                ple_test1_000080_000080_flux.fits ple_test1_000160_000160_flux.fits "
         ;;

     test2 ) allimgs="wav_out.fits wav_cell.fits wav_recon.fits wav_nbkg.fits \
                 img_000010_000010_flux.fits img_000020_000020_flux.fits \
                 img_000040_000040_flux.fits img_000080_000080_flux.fits "
         ;;

     test3 ) allimgs="yes_nbkg yes_src yes_cell yes_recon region.txt\
                   P0551690201PNS001IMAGE_8000_000020_000020_flux.fits \
                   P0551690201PNS001IMAGE_8000_000040_000040_flux.fits \
                   P0551690201PNS001IMAGE_8000_000060_000060_flux.fits \
                   P0551690201PNS001IMAGE_8000_000080_000080_flux.fits \
                   P0551690201PNS001IMAGE_8000_000100_000100_flux.fits \
                   P0551690201PNS001IMAGE_8000_000120_000120_flux.fits \
                   P0551690201PNS001IMAGE_8000_000140_000140_flux.fits \
                   P0551690201PNS001IMAGE_8000_000160_000160_flux.fits \
                   P0551690201PNS001IMAGE_8000_000180_000180_flux.fits "
         ;;
  esac

  for img in $allimgs
  do
     outfile=${OUTDIR}/$img
     savfile=${SAVDIR}/$img
     dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
     if  test $? -ne 0 ; then
        echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
        mismatch=0
     fi

     if test $img = "ple_test1_nbkg.fits" || test $img = "wav_nbkg.fits" || test $img = "yes_nbkg" ; then
        dmdiff "$outfile[2]" "$savfile[2]" tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
        if test $? -ne 0; then
           echo "ERROR: DATA MISMATCH in $outfile[2]" >> $LOGFILE
           mismatch=0
        fi
     fi
  done
  #################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
     # Yes
     echo "${testid} NOT-OK"
     script_succeeded=1
  else
     # No
     echo "${testid} OK"
  fi
  # end: AA 

done

# end per-test loop
######################################################################

######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded

