#! /bin/sh

# wtransform.t
# test script for wtransform


# syntax:
# wtransform.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}


######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="wtransform"

# set up list of tests
# test9 removed; problematic thresholding causing insig. diffs
alltests="test1 test2 test3 test4 test5 test6 test7 test8 test10 test11"

# "short" test to run
shortlist="test1 test3 test5 test8"

# list of stacks
stacks="src_stk.lis correl_stk.lis bkg_stk.lis nbkg_stk.lis"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
#bose=`pget $toolname verbose`
#punlearn $toolname
#pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # by default, no backround error messages
  errors=1
    ####################################################################
  # run the tool
  case ${testid} in

    test1 ) test_string="wtransform infile=${INDIR}/trapez_small.fits
                expcor=fast expfile=none exptime=0.0
                expthresh=0.1 sigthresh=1e-6 bkgsigthresh=0.001
                xscales=\"1.414 2.0\" yscales=\"1.414 2.0\"
                maxiter=2 iterstop=0.0001
                correlerr=yes nbkgerr=yes bkgerr=no thresh=none
                bkgtime=0  bkginput=none bkgerrinput=no
                clobber=yes log=no stall=no 
                srclist=${OUTDIR}/test1_src_stk.lis
                correl=${OUTDIR}/test1_correl_stk.lis
                nbkg=${OUTDIR}/test1_nbkg_stk.lis
                bkg=${OUTDIR}/test1_bkg_stk.lis
		falsesrc=-1
                outputinfix=\"_${testid}\" "

                PREFIX="trapez_small_test1_"
                scales="000014_000014 000020_000020"
		;;

    test2 ) test_string="wtransform infile=${INDIR}/trapez_small.fits
                expcor=full expfile=none exptime=0.0
                expthresh=0.1 sigthresh=1e-6 bkgsigthresh=0.001
                xscales=\"1.2 2.4\" yscales=\"2.4 1.2\"
                maxiter=2 iterstop=0.0001
                correlerr=yes nbkgerr=yes bkgerr=no thresh=none
                bkgtime=0  bkginput=none bkgerrinput=no
                clobber=yes log=no stall=no 
                srclist=${OUTDIR}/test2_src_stk.lis
                correl=${OUTDIR}/test2_correl_stk.lis
                nbkg=${OUTDIR}/test2_nbkg_stk.lis
                bkg=${OUTDIR}/test2_bkg_stk.lis
		falsesrc=-1
                outputinfix=\"_${testid}\" "

                PREFIX="trapez_small_test2_"
                scales="000012_000024 000024_000012"
		;;

	test3 ) test_string="wtransform infile=${INDIR}/wtran_snr_small.fits
		expcor=fast expfile=none exptime=0.0
		expthresh=0.2 sigthresh=1.e-4 bkgsigthresh=1.e-2 
		xscales=\"8.0 16.0\" yscales=\"8.0 16.0\"
	        correlerr=yes nbkgerr=yes bkgerr=yes thresh=none
		maxiter=2 iterstop=1.e-4 
		bkgtime=0  bkginput=none bkgerrinput=no 
		clobber=yes log=no stall=no 
		srclist=${OUTDIR}/test3_src_stk.lis 
		correl=${OUTDIR}/test3_correl_stk.lis 
		nbkg=${OUTDIR}/test3_nbkg_stk.lis
		bkg=${OUTDIR}/test3_bkg_stk.lis
                outputinfix=\"_${testid}\" "

                PREFIX="wtran_snr_small_test3_"
		error=0
		scales="000080_000080 000160_000160" ;;

        test4 ) test_string="wtransform infile=${INDIR}/test_small.fits
		expcor=fast exptime=0.0  expfile=none
		expthresh=0.1 sigthresh=1.e-5 bkgsigthresh=1.e-2 
		xscales=\"1.5 3.0\" yscales=\"1.5 3.0\" 
	        correlerr=yes  nbkgerr=yes bkgerr=yes thresh=none
		maxiter=2 iterstop=1.e-4 falsesrc=-1.0 
		bkgtime=0  bkginput=none bkgerrinput=no 
		log=no stall=no clobber=yes 
		srclist=${OUTDIR}/test4_src_stk.lis
		correl=${OUTDIR}/test4_correl_stk.lis
		nbkg=${OUTDIR}/test4_nbkg_stk.lis
		bkg=${OUTDIR}/test4_bkg_stk.lis
                outputinfix=\"_${testid}\" "

                PREFIX="test_small_test4_"
		errors=0
		scales="000015_000015 000030_000030" ;;

        test5 ) test_string="wtransform infile=${INDIR}/test_small.fits
		expcor=full exptime=0.0  expfile=none
		expthresh=0.1 sigthresh=1.e-5 bkgsigthresh=1.e-2 
		xscales=\"1.0 4.0\" yscales=\"1.0 4.0\"
	        correlerr=yes nbkgerr=yes bkgerr=yes thresh=none
		maxiter=2 iterstop=1.e-4 falsesrc=1.0 
		bkgtime=0  bkginput=none bkgerrinput=no 
		log=no stall=no  clobber=yes 
		srclist=${OUTDIR}/test5_src_stk.lis
		correl=${OUTDIR}/test5_correl_stk.lis
		nbkg=${OUTDIR}/test5_nbkg_stk.lis
		bkg=${OUTDIR}/test5_bkg_stk.lis
                outputinfix=\"_${testid}\" "

                PREFIX="test_small_test5_"
		errors=0
		scales="000010_000010 000040_000040" ;;

        test6 ) test_string="wtransform infile=${INDIR}/test_small.fits
		expcor=fast exptime=0.0  expfile=none
		expthresh=0.1 sigthresh=1.e-5 bkgsigthresh=1.e-2 
		xscales=\"1.0 1.2\" yscales=\"1.0 1.2\"
	        correlerr=yes  nbkgerr=yes bkgerr=yes 
                thresh=${OUTDIR}/test6_thresh_stk.lis
		maxiter=2 iterstop=1.e-4 falsesrc=10.0 
		bkgtime=0  bkginput=none bkgerrinput=no 
		log=no stall=no  clobber=yes 
		srclist=${OUTDIR}/test6_src_stk.lis
		correl=${OUTDIR}/test6_correl_stk.lis
		nbkg=${OUTDIR}/test6_nbkg_stk.lis
		bkg=${OUTDIR}/test6_bkg_stk.lis
                outputinfix=\"_${testid}\" "

                PREFIX="test_small_test6_"
		errors=0
		scales="000010_000010 000012_000012" ;;

        test7 ) test_string="wtransform infile=${INDIR}/test_small.fits
		expcor=none exptime=0.0  expfile=none
		expthresh=0.1 sigthresh=1.e-5 bkgsigthresh=1.e-2 
		xscales=4.0 yscales=4.0 
	        correlerr=yes  nbkgerr=yes bkgerr=yes 
                thresh=${OUTDIR}/test7_thresh_stk.lis
		maxiter=1 iterstop=1.e-4 falsesrc=10.0 
		bkgtime=0  bkginput=none bkgerrinput=no 
		log=no stall=no  clobber=yes 
		srclist=${OUTDIR}/test7_src_stk.lis
		correl=${OUTDIR}/test7_correl_stk.lis
		nbkg=${OUTDIR}/test7_nbkg_stk.lis
		bkg=${OUTDIR}/test7_bkg_stk.lis
                outputinfix=\"_${testid}\" "

                PREFIX="test_small_test7_"
		errors=0
		scales="000040_000040" ;;

        test8 ) test_string="wtransform infile=${INDIR}/M7.fits
		expcor=fast exptime=0.0  expfile=${INDIR}/M7_emap.fits
		expthresh=0.1 sigthresh=1.e-5 bkgsigthresh=1.e-2 
		xscales=\"1.0 4.0\" yscales=\"1.0 4.0\"
	        correlerr=yes  nbkgerr=yes bkgerr=yes
                thresh=none maxiter=2 iterstop=1.e-4 falsesrc=-1.0 
		bkgtime=0  bkginput=none bkgerrinput=no 
		log=no stall=no clobber=yes 
		srclist=${OUTDIR}/test8_src_stk.lis
		correl=${OUTDIR}/test8_correl_stk.lis
		nbkg=${OUTDIR}/test8_nbkg_stk.lis
		bkg=${OUTDIR}/test8_bkg_stk.lis
                outputinfix=\"_${testid}\" "

                PREFIX="M7_test8_"
		errors=0
		scales="000010_000010 000040_000040" ;;

        test9 ) test_string="wtransform infile=${INDIR}/M7.fits
		expcor=full exptime=1800.0  expfile=${INDIR}/M7_emap.fits
		expthresh=0.1 sigthresh=1.e-5 bkgsigthresh=1.e-2 
		xscales=\"1.0 4.0\" yscales=\"1.0 4.0\"
	        correlerr=yes  nbkgerr=yes bkgerr=yes
                thresh=none maxiter=2 iterstop=1.e-4 falsesrc=8.0 
		bkgtime=0  bkginput=none bkgerrinput=no 
		log=no stall=no clobber=yes 
		srclist=${OUTDIR}/test9_src_stk.lis
		correl=${OUTDIR}/test9_correl_stk.lis
		nbkg=${OUTDIR}/test9_nbkg_stk.lis
		bkg=${OUTDIR}/test9_bkg_stk.lis
                outputinfix=\"_${testid}\" "

                PREFIX="M7_test9_"
		errors=0
		scales="000010_000010 000040_000040" ;;

        test10 ) test_string="wtransform infile=${INDIR}/M7.fits
		expcor=fast exptime=0.0  expfile=${INDIR}/M7_emap.fits
		expthresh=0.1 sigthresh=1.e-5 bkgsigthresh=1.e-2 
		xscales=\"1.0 2.0\" yscales=\"1.0 2.0\"
	        correlerr=no  nbkgerr=no bkgerr=no
                thresh=none maxiter=2 iterstop=1.e-4 falsesrc=-1.0 
		bkgtime=0  bkginput=${INDIR}/M7_nbkg.fits 
                bkgerrinput=yes log=no stall=no clobber=yes 
		srclist=${OUTDIR}/test10_src_stk.lis
		correl=${OUTDIR}/test10_correl_stk.lis
		nbkg=${OUTDIR}/test10_nbkg_stk.lis
		bkg=${OUTDIR}/test10_bkg_stk.lis
                outputinfix=\"_${testid}\" "

                PREFIX="M7_test10_"
		scales="000010_000010 000020_000020" ;;

        test11 ) test_string="wtransform infile=${INDIR}/whiz_broad_thresh.img
		expcor=fast exptime=0.0  expfile=${INDIR}/whiz_broad_thresh.expmap
		expthresh=0.1 sigthresh=1.e-06 bkgsigthresh=0.001
		xscales=\"1.0 1.4\" yscales=\"1.0 1.4\"
	        correlerr=no  nbkgerr=yes bkgerr=no
                thresh=none maxiter=2 iterstop=1.e-4 falsesrc=0.5
		bkgtime=0  bkginput="" 
                bkgerrinput=no log=no stall=no clobber=yes 
		srclist=${OUTDIR}/${testid}_src_stk.lis
		correl=${OUTDIR}/${testid}_correl_stk.lis
		nbkg=${OUTDIR}/${testid}_nbkg_stk.lis
		bkg=none
                outputinfix=\"_${testid}\" "

                PREFIX="whiz_broad_thresh_${testid}_"
		scales="000010_000010 000014_000014" ;;

  esac

  echo $test_string | tee -a $LOGFILE
  eval $test_string  | tee -a $LOGFILE  2>&1


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1


  ############################################################
  # FITS tables

  for scale in $scales
  do
    outfile=${OUTDIR}/${PREFIX}${scale}.fits
    savfile=${SAVDIR}/${PREFIX}${scale}.fits

    dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
      mismatch=0
    fi
  done

  ####################################################################
  # check images

  if test ${testid} = "test10" ; then

    for img in correl
    do
      outfile=${OUTDIR}/${PREFIX}${scale}_${img}.fits
      savfile=${SAVDIR}/${PREFIX}${scale}_${img}.fits

      # check image

      dmdiff $outfile"[1]" $savfile"[1]" tol=$SAVDIR/tolerance > \
        /dev/null 2>>$LOGFILE
      if  test $? -ne 0 ; then
        echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
        mismatch=0
      fi
    done

  elif test ${testid} = "test11" ; then

    for img in nbkg correl
    do

      outfile=${OUTDIR}/${PREFIX}${scale}_${img}.fits
      savfile=${SAVDIR}/${PREFIX}${scale}_${img}.fits

      # check image

      dmdiff $outfile"[1]" $savfile"[1]" tol=$SAVDIR/tolerance > \
        /dev/null 2>>$LOGFILE
      if  test $? -ne 0 ; then
        echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
        mismatch=0
      fi

      # check error image
      if test $errors -ne 1; then
        dmdiff "$outfile[2]" "$savfile[2]" tol=$SAVDIR/tolerance > /dev/null \
          2>>$LOGFILE
        if test $? -ne 0; then
          echo "ERROR: DATA MISMATCH in $outfile[2]" >> $LOGFILE
          mismatch=0
        fi
      fi
    done

  else

    for img in nbkg correl bkg
    do

      outfile=${OUTDIR}/${PREFIX}${scale}_${img}.fits
      savfile=${SAVDIR}/${PREFIX}${scale}_${img}.fits

      # check image

      dmdiff $outfile"[1]" $savfile"[1]" tol=$SAVDIR/tolerance > \
        /dev/null 2>>$LOGFILE
      if  test $? -ne 0 ; then
        echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
        mismatch=0
      fi

      # check error image
      if test $errors -ne 1; then
        dmdiff "$outfile[2]" "$savfile[2]" tol=$SAVDIR/tolerance > /dev/null \
          2>>$LOGFILE
        if test $? -ne 0; then
          echo "ERROR: DATA MISMATCH in $outfile[2]" >> $LOGFILE
          mismatch=0
        fi
      fi
    done

  fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded

